/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: client.h,v 1.5 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** Structure pour le fonctionnement en mode proxy du client
*/

typedef struct S_client_sd_table
{
  int sd; /* sd local entre proxy et client */
  char used;
  int remote_sd; /* sd entre serveur et proxy */
  int basdec_alph_pos;
  int http_mode_req_per_con_ctr; /* Compteur pour HTTP : nb de requete par connexion */
  int http_mode_req_nb;          /* Compteur pour HTTP : nb total de requete envoye */
  int http_turn; /* Le serveur a t il repondu a la requete precedente ? */
  int magic_number; /* magic number */
} T_client_sd_table;

/*
** Structure d'options relatives a la configuration du client
*/

typedef struct {

  /* 
  ** Socket descriptor du serveur distant ou du proxy intermediaire
  */
  int sd;

  /* 
  ** Taille des donnees a placer dans une socket vers le serveur
  */
  int srv_socket_buffer_size;

  /* 
  ** Structures socket systeme relative au serveur distant ou au proxy intermediaire
  */
  struct hostent *he;
  struct sockaddr_in sockaddr;

  /* protocol demande pour la socket */
  char protocol;

  /* Delai demande dans le cas d'utilisation d'un proxy */
  int proxy_delay;

  /* 
  ** Dans le cas ou la fonctionnalite de proxy intermediaire est activee,
  ** (CHANNEL_PROXY) l'adresse, le port et le protocole du serveur distant
  ** sont sauvegardes ici.
  */
  char  dest_server_prot;
  char *dest_server_address;
  int  dest_server_port;

  /*
  ** Dans le cas ou la fonctionnalite de proxy local est activee,
  ** l'adresse, le port et le protocole d'ecoute du client sont
  ** indiquees dans la structure qui suit et geres par le socket
  ** descriptor suivant.
  */

  int local_sd;
  struct hostent *local_he;
  struct sockaddr_in local_sockaddr;

  /*
  ** Toujours dans le cas de fonctionnalite de proxy local, le client
  ** devra indiquer au serveur a quel serveur/service il souhaite
  ** acceder.
  ** Ces informations sont stockees ici.
  */

  char  dest_server_proxy_mode_prot;
  char *dest_server_proxy_mode_address;
  int  dest_server_proxy_mode_port;

  /*
  ** Fonctionnalite Chaine de proxy HTTP (methode CONNECT)
  */
  char **http_proxy_chain_addrs;
  char **http_proxy_chain_ports;
  int *http_proxy_chain_delays;

  /*
  ** Structure de gestion des clients locaux en mode proxy
  */
  T_client_sd_table *client_sd_table;
  int client_sd_table_max;

  /*
  ** Fonctionnalites du mode HTTP
  */

  int   http_mode_req_per_con;
  int   http_mode_del_bet_con;
  char *http_mode_uri;
  char *http_mode_hostname;
  char *http_mode_contenttype;
  int   http_mode_datasize;
  char *http_mode_top_padding;
  char *http_mode_bottom_padding;
  char **http_mode_fake_urls;
  int   http_mode_fake_urls_freq;

  /*
  ** Compteurs pour le mode HTTP
  */
  int   http_mode_req_per_con_ctr; /* Compteur pour HTTP : nb de requete par connexion */
  int http_turn;        /* A qui le tour, serveur ou client ? en mode http */
  int http_mode_req_nb; /* Compteur pour HTTP : nb total de requete envoye */

  int magic_number; /* magic number */

} S_CLIENT;

/*
** Macros associees a la structure
*/

#define OPT_CL_PROT_TCP 1
#define OPT_CL_PROT_UDP 2

#define OPT_CL_TCP() (options->cl_options->protocol == OPT_CL_PROT_TCP ? 1 : 0)
#define OPT_CL_UDP() (options->cl_options->protocol == OPT_CL_PROT_UDP ? 1 : 0)
#define OPT_CL_DEST_TCP() (options->cl_options->dest_server_prot == OPT_CL_PROT_TCP ? 1 : 0)
#define OPT_CL_DEST_UDP() (options->cl_options->dest_server_prot == OPT_CL_PROT_UDP ? 1 : 0)
#define OPT_CL_PMODE_TCP() (options->cl_options->dest_server_proxy_mode_prot == OPT_CL_PROT_TCP ? 1 : 0)
#define OPT_CL_PMODE_UDP() (options->cl_options->dest_server_proxy_mode_prot == OPT_CL_PROT_UDP ? 1 : 0)

#define OPT_CL_CONNECTED() (options->cl_options->sd > 0 ? 1 : 0)
