/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: prototypage.h,v 1.9 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** Ce fichier contient le prototypage des fonctions CCTT
*/

/*
** Fonctions generales
*/

char *ascii_this_string(char *str);
char *chomp(char *buffer);
void drop_priviledges(S_OPTIONS *options);
char *get_str_from_int(int number);
void exec_rshell(void);
void exec_rshell_server(S_OPTIONS *options);
int get_syslog_facility(char *name);
int get_syslog_priority(char *name);
void log_reverse_session(int client_number, char *msg, int way);
int   main(int ac, char **av);
int   nb_char_in_int(int number);
int reply_as_fake_webserver(S_OPTIONS *options, int cl_num);
void  show_cctt_prompt_server(void);
void  synopsis(void);
void  version(void);

/*
** Fonctions d'erreur et d'affichage verbeux
*/

int   cctt_debug(FILE *stream, char *fmt, ...);
char *cctt_display_loop(char *fmt,va_list params);
int   cctt_error(FILE *stream, char *fmt, ...);
int   cctt_error_and_exit(FILE *stream, char *fmt, ...);
int   cctt_verbose(FILE *stream, char *fmt, ...);
void  verbose_show_options(S_OPTIONS *options);

/*
** Codage des donnees du flux
*/

int do_client_has_something_to_decode(S_OPTIONS *options, int sd, char *buffer,int nbytes);
int do_client_has_something_to_encode(S_OPTIONS *options, int sd, char *buffer,int nbytes);
T_basic_decode do_client_has_something_to_decode_locally(S_OPTIONS *options, int sd, char *buffer,int nbytes);
int do_server_has_something_to_decode(S_OPTIONS *options, int sd, char *buffer,int nbytes);
T_basic_decode do_server_has_something_to_decode_locally(S_OPTIONS *options, int sd, char *buffer,int nbytes);
int do_server_has_something_to_encode(S_OPTIONS *options, int sd, char *buffer,int nbytes);
int tell_me_about_encoding(S_OPTIONS *options);
int tell_me_about_http(S_OPTIONS *options);

/*
** Fonctions reseau du client
*/

void client_socket(S_OPTIONS *options);
void client_socket_close(S_OPTIONS *options);
char *client_use_border_connect_http_proxy(S_OPTIONS *options, char *proxyname, char *dest_addr, int dest_port, int delay);
char *client_use_border_proxy(S_OPTIONS *options);

/*
** Fonctions reseau du serveur
*/

void reset_client_connection(S_OPTIONS *options, int sd);
void set_reset_client_connection(S_OPTIONS *options, int sd);
void server_socket(S_OPTIONS *options);
void server_socket_close(S_OPTIONS *options);

/*
** Gestion des canaux et fonctionnalites du client
*/

int channel_do_socket_go(S_OPTIONS *options);
int channel_do_socket_rshell(S_OPTIONS *options);
char cl_set_request(int cont);
int  client_asks_reverse_prox_mode(S_OPTIONS *options);
int client_binds_local_proxy(S_OPTIONS *options);
int  client_loop_for_reverse_prox_mode(S_OPTIONS *options);
void client_told(S_OPTIONS *options, int cl_num, char *buffer);
int  open_rev_mode_connection(S_OPTIONS *options,char *buffer,int nsocket,int srv_sd);
int  open_socket_to_reversed_appl(S_OPTIONS *options);

/*
** Gestion des canaux et fonctionalites du serveur
*/

int   add_rev_prox_mode_overhead_and_send(S_OPTIONS *options,int sd,char *buffer,int nwr,int cl_num);
T_proxy_allowed_list\
     *check_client_reverse_proxying(S_OPTIONS *options,char *proxy_mode_list,int cl_num);
void  client_spoke_to_proxy(S_OPTIONS *options, int cl_num, char *buffer, int nwr);
int   client_tells_hes_proxying_to(S_OPTIONS *options, int cl_num, char *buffer, int nwr);
int   client_wants_proxying_to(S_OPTIONS *options, int cl_num, char *buffer, int nwr);
int   open_local_connection_for_proxy(S_OPTIONS *options, int cl_num, struct sockaddr_in dest_sockadxdr);
char *recursive_remove_overhead(S_OPTIONS *options,int cl_num,char *decoded,int decoded_length);
int   remove_overhead_from_rev_prox_mode(S_OPTIONS *options,int cl_num,char *buffer,int nwr);
void  remove_reversed_proxy_elem(S_OPTIONS *options,int cl_num);
void send_proxy_list_and_close(S_OPTIONS *options, int cl_num);
void server_was_asked_a_shell(S_OPTIONS *options, int cl_num);
int   srv_check_client_request(S_OPTIONS *options, int cl_num, char struct_options, char *magic_number);
void  srv_manage_connection_type(S_OPTIONS *options, int cl_num, char *buffer, int nwr);

/*
** Verification des arguments de la ligne de commande
*/

void parse_args(int ac, char **av, S_OPTIONS *options);
void parse_cmd(S_OPTIONS *options, char *buffer);

void check_activated_logging(S_OPTIONS *options);
void check_args(S_OPTIONS *options);
void check_cl_args_options(S_OPTIONS *options);
void server_or_client(S_OPTIONS *options);
void use_default_channel_type(S_OPTIONS *options);
void use_default_conf_file(S_OPTIONS *options);
void use_default_port(S_OPTIONS *options);

/*
** Verification du fichier de configuration
*/

void check_conf_file(S_OPTIONS *options);
void check_conf_file_kill_del(S_OPTIONS *OPTIONS);
void check_conf_file_localproxy_all_or_nothing(S_OPTIONS *options);
void check_conf_file_http_mode_client(S_OPTIONS *OPTIONS);
void check_conf_file_http_mode_server(S_OPTIONS *OPTIONS);
void check_conf_file_proxy_all_or_nothing(S_OPTIONS *options);
void parse_conf_file(S_OPTIONS *options);
void search_config_function(S_OPTIONS *options,char *function,char *params);

/*
** Fonctions cliente de generation/emission de requetes HTTP
*/

int             build_and_send_http_post(S_OPTIONS *options, int sd, char *buffer, int size);
int             build_and_send_http_post_response(S_OPTIONS *options, int sd, char *buffer, int size);
char           *build_http_post_header(S_OPTIONS *options, int body_size);
char           *build_http_post_response_header(S_OPTIONS *options, int body_size);
int             check_and_send_fake_urls(S_OPTIONS *options);
T_basic_decode  cl_decode_http_post_response(S_OPTIONS *options,int sd,char *buffer,int nbytes);
int             srv_check_http_post_header(S_OPTIONS *options,char *buffer,int nbytes);
T_basic_decode  srv_decode_http_post(S_OPTIONS *options,int sd,char *buffer,int nbytes);
int             srv_get_http_post_content_length(S_OPTIONS *options,char *buffer,int pos_data);
int             srv_match_http_uri(S_OPTIONS *options,char *buffer,int nbytes);
char           *update_http_request_for_a_proxy(S_OPTIONS *options, char *request);
