/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: server.h,v 1.6 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
** Element de structure sur les connexions en cours.
** Chaque element correspond a une connexion
*/

typedef struct S_connect_sd_table
{
  /* Ce sd est il libre ? */
  char used;

  /* Informations sur le client */
  struct sockaddr_in sockaddr;

  /* client identifie ou en cours d'identification ? */
  char ident;

  /* Type de service demande par le client */
  char tos;
  
  /* Pid du Shell dans le cas d'une demande de shell */
  int pid_shell;

  /* Pipe de communication entre le serveur et le manager */
  int server_manager[2];

  /* Autorisation de proxying */
  char proxy_auth;

  /* Sockaddr du client proxyied */
  struct sockaddr_in cl_prox_sockaddr;

  /* Sockaddr pour la demande du client */
  struct sockaddr_in prox_to_sockaddr;

  /* SD vers la connection demandee par le client */
  int cl_req_sd;

  /* Structure de temps relative a la connexion */
  struct timeval tvu; /* Up since */
  struct timeval tvq; /* Quiet since */
  struct timeval tvr; /* Read since */

  /* Buffer temporaire de concatenation avant action */
  char tmpbuf[ACTION_BUFFER_SIZE];

  /* Pointeur sur le prochain octet libre du buffer temporaire */
  char *tmpbuf_ptr;

  /* Taille des donnees presentes dans le buffer temporaire */
  int tmpbuf_size;

  /* Position courante dans l'alphabet d'encodage pour les appels a basic_decode() */
  int basdec_alph_pos;

  /* Si le client est associe a une connexion etablie en mode proxy inverse */
  int rev_prox_mode;

  /* 
  ** Quel est le premier numero de client qui nous etait affecte et qui sert 
  ** d'index au niveau du client en mode proxy inverse
  ** Dans un mode comme http_post, il nous faut conserver ce client meme
  ** si le numero de client (sd) change.
  */
  int rev_prox_mode_sd;

  /* Magic number du client */
  int magic_number;

  /* Est ce que le client est en attente de fermeture */
  int close_flag;
} T_connect_sd_table;

#define SRV_CL_IDENT_NO 0
#define SRV_CL_IDENT_ON 1
#define SRV_CL_IDENT_YES 2

#define SRV_CL_IDENT(ident) (ident==SRV_CL_IDENT_NO?SRV_CL_IDENT_NO:(ident==SRV_CL_IDENT_ON?SRV_CL_IDENT_ON:SRV_CL_IDENT_YES))

/* 
** Note : Les types de tos sont definis dans includes/cl_identification.h 
*/

/* Est ce que le client est gere par un manager ? */
#define SRV_CL_MANAGED(cl_num) (options->srv_options->connect_infos->sd_table[cl_num].tos == CL_IDENT_ASKED_SHELL?1:0)

/* Est ce que le client demandant du proxying a ete valide ? */
#define SRV_CL_PROX_NO 0
#define SRV_CL_PROX_ON 1
#define SRV_CL_PROX_YES 2
#define SRV_CL_PROX_REV 3

#define SRV_CL_CLOSED_FLAG(cl_num) (options->srv_options->connect_infos->sd_table[cl_num].close_flag == 1 ?1:0)

/*****************************************************
** Structure principale sur les connexions en cours **
*****************************************************/

typedef struct S_connect_infos
{
  int num_sd_connected; /* Nombre de socket descriptors */
  int sdtablesize; /* Nombre maximal de sd pour le process */
  int select_var;

  /* Sets pour le select */
  fd_set sread; 
  fd_set swrite;
  fd_set serror;

  /* Liste d'info par sd */
  T_connect_sd_table *sd_table;

} T_connect_infos;

/* Un client est il connecte au serveur (src/server/do_server.c) */
#define SRV_SD_USED 1
#define SRV_SD_FREE 0

/* Nombre de connections dans la queue du listen */
#define SRV_LISTEN_NUM 10

/**************************************************************************
** Structure Element de liste d'autorisation pour le proxying du serveur **
**************************************************************************/

typedef struct S_proxy_allowed_list
{
  char *label;
  struct hostent *he;
  struct sockaddr_in sockaddr;
  void *next;
  void *first;
  int reverse;
} T_proxy_allowed_list;

/****************************************************************
** Structure d'options relatives a la configuration du serveur **
****************************************************************/

typedef struct {
  
  /* Socket descriptor */
  int sd;

  /* Structures socket systeme */
  struct hostent *he;
  struct sockaddr_in sockaddr;

  /* Protocole demande pour la socket */
  char protocol;

  /* Liste d'autorisations pour le proxying local du serveur */
  T_proxy_allowed_list *proxy_allowed_list;
  int proxy_allowed_list_elements_number;

  /* Information sur les connexions en cours */
  T_connect_infos *connect_infos;

  /* 
  ** Informations sur le shell que le serveur est susceptible
  ** de renvoyer aux clients
  */
  char *srv_shell_loc;
  char *srv_shell_cmd;

  /*
  ** Proxy Only ?
  */
  int proxy_only;

  /*
  ** Suppression des privileges
  */
  uid_t perm_uid;
  gid_t perm_gid;
  char *perm_chroot;

  /*
  ** Fausse banniere
  */
  char *fake_webserver;

  /*
  ** Directives de configuration en mode HTTP
  */

  char  *http_mode_uri;
  char  *http_mode_error_page;
  char **http_mode_fake_url_filename;
  int    http_mode_top_padding_size;
  int    http_mode_bot_padding_size;

  /* Delais de fermeture automatiques des connexions quiet */

  int kill_quiet_del;
  int kill_quiet_del_cf;

} S_SERVER;

/*
** Macros associees a la structure
*/

#define OPT_SRV_PROT_TCP 1
#define OPT_SRV_PROT_UDP 2

#define OPT_SRV_TCP() (options->srv_options->protocol == OPT_SRV_PROT_TCP ? 1 : 0)
#define OPT_SRV_UDP() (options->srv_options->protocol == OPT_SRV_PROT_UDP ? 1 : 0)

#define OPT_SRV_CONNECTED() (options->srv_options->sd > 0 ? 1 : 0)

#define OPT_SRV_PROXY_ONLY() (options->srv_options->proxy_only == 1 ? 1 : 0)
