/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: args.c,v 1.5 2003/04/23 07:06:10 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

void parse_args(int ac, char **av,S_OPTIONS *options)
{
  poptContext context;
  char *optarg = NULL;
  int c;

  /*
  ** Preparation de la structure PopT en fonction des options
  */

  struct poptOption options_table[] = {
    {"help",'h',POPT_ARG_NONE,NULL,'h'},
    {"version",'V',POPT_ARG_NONE,NULL,'V'},
    {"debug",'D',POPT_ARG_NONE,NULL,'D'},

    {"verbose",'v',POPT_ARG_NONE,NULL,'v'},
    {"logging",'l',POPT_ARG_NONE,NULL,'l'},
    {"logging-sylog",'L',POPT_ARG_NONE,NULL,'L'},
    {"rshell",'r',POPT_ARG_NONE,NULL,'r'},
    {"proxy_mode",'a',POPT_ARG_NONE,NULL,'a'},
    {"reverse_proxy_mode",'z',POPT_ARG_NONE,NULL,'z'},
    {"list_proxy_mode",'i',POPT_ARG_NONE,NULL,'i'},
    {"check_conf_file",'T',POPT_ARG_NONE,NULL,'T'},

    {"listen_address",'s',POPT_ARG_STRING,&optarg,'s'},
    {"listen_port",'p',POPT_ARG_STRING,&optarg,'p'},
    {"connect_address",'c',POPT_ARG_STRING,&optarg,'c'},
    {"connect_port",'d',POPT_ARG_STRING,&optarg,'d'},
    {"channel_type",'t',POPT_ARG_STRING,&optarg,'t'},
    {"config_file",'f',POPT_ARG_STRING,&optarg,'f'},

    {NULL, 0, 0, NULL, 0}
  };

  context = poptGetContext(NAME, ac, (const char **) av, options_table, 0);
  if ((c = poptGetNextOpt(context)) < 0)
    synopsis();

  /*
  ** Verification du premier argument
  */

  switch (c) {
  case 'h':
    synopsis();
  case 'V':
    version();
    exit (0);

  case 's':
    if (!(options->srv_options->he =  (struct hostent *) gethostbyname(optarg)))
      cctt_error_and_exit(stderr,"%s","[Options] : Bad IP Address with -s flag !\n");
    options->srv_options->sockaddr.sin_family = AF_INET;
    options->srv_options->sockaddr.sin_addr = *((struct in_addr *)options->srv_options->he->h_addr);
    bzero(&(options->srv_options->sockaddr.sin_zero), sizeof(options->srv_options->sockaddr.sin_zero));
    break;
  case 'p':
    if (!(options->srv_options->sockaddr.sin_port = htons(atoi(optarg))))
      cctt_error_and_exit(stderr,"%s","[Options] : Bad port with -p flag !\n");
    break;

  case 'c':
    if (!((options->cl_options)->he =  (struct hostent *) gethostbyname(optarg)))
      cctt_error_and_exit(stderr,"%s","[Options] : Bad IP Address with -c flag !\n");
    options->cl_options->sockaddr.sin_family = AF_INET;
    options->cl_options->sockaddr.sin_addr = *((struct in_addr *)options->cl_options->he->h_addr);
    bzero(&(options->cl_options->sockaddr.sin_zero), sizeof(options->cl_options->sockaddr.sin_zero));
    break;
  case 'd':
    if (!(options->cl_options->sockaddr.sin_port = htons(atoi(optarg))))
      cctt_error_and_exit(stderr,"%s","[Options] : Bad port with -d flag !\n");
    break;

  case 't':
    options->channel = (char *) malloc (strlen(optarg)+1);
    memset (options->channel,0,strlen(optarg)+1);
    strncpy(options->channel,optarg,strlen(optarg));
    break;

  case 'T':
    options->options |= (0x01 << 9);
    break;
  case 'D':
    options->options |= (0x01 << 8);
    break;
  case 'v':
    options->options |= (0x01 << 7);
    break;
  case 'f':
    options->options |= (0x01 << 6);
    options->conf_file = (char *) malloc (strlen(optarg)+1);
    memset (options->conf_file,0,strlen(optarg)+1);
    strncpy(options->conf_file,optarg,strlen(optarg));
    break;
  case 'l':
    options->options |= (0x01 << 5);
    break;
  case 'L':
    options->options |= (0x01 << 11);
    break;
  case 'r':
    options->options |= (0x01 << 4);
    break;
  case 'a':
    options->options |= (0x01 << 3);
    break;
  case 'z':
    options->options |= (0x01 << 12);
    break;
  case 'i':
    options->options |= (0x01 << 2);
    break;

  default:
    synopsis();
  }

  /*
  ** Verification des arguments suivants
  */

  while ((c = poptGetNextOpt(context)) >= 0) {
    switch (c) {
    case 'h':
      synopsis();
    case 'V':
      version();
      exit (0);

    case 's':
      if (!(options->srv_options->he =  (struct hostent *) gethostbyname(optarg)))
	cctt_error_and_exit(stderr,"%s","[Options] : Bad IP Address with -s flag !\n");
      options->srv_options->sockaddr.sin_family = AF_INET;
      options->srv_options->sockaddr.sin_addr = *((struct in_addr *)options->srv_options->he->h_addr);
      bzero(&(options->srv_options->sockaddr.sin_zero), sizeof(options->srv_options->sockaddr.sin_zero));
      break;
    case 'p':
      if (!(options->srv_options->sockaddr.sin_port = htons(atoi(optarg))))
	cctt_error_and_exit(stderr,"%s","[Options] : Bad port with -p flag !\n");
      break;

    case 'c':
      if (!(options->cl_options->he =  (struct hostent *) gethostbyname(optarg)))
	cctt_error_and_exit(stderr,"%s","[Options] : Bad IP Address with -c flag !\n");
      options->cl_options->sockaddr.sin_family = AF_INET;
      options->cl_options->sockaddr.sin_addr = *((struct in_addr *)options->cl_options->he->h_addr);
      bzero(&(options->cl_options->sockaddr.sin_zero), sizeof(options->cl_options->sockaddr.sin_zero));
      break;
    case 'd':
      if (!(options->cl_options->sockaddr.sin_port = htons(atoi(optarg))))
	cctt_error_and_exit(stderr,"%s","[Options] : Bad port with -d flag !\n");
      break;

    case 't':
      options->channel = (char *) malloc (strlen(optarg)+1);
      memset (options->channel,0,strlen(optarg)+1);
      strncpy(options->channel,optarg,strlen(optarg));
      break;
      
    case 'T':
      options->options |= (0x01 << 9);
      break;
    case 'D':
      options->options |= (0x01 << 8);
      break;
    case 'v':
      options->options |= (0x01 << 7);
      break;
    case 'f':
      options->options |= (0x01 << 6);
      options->conf_file = (char *) malloc (strlen(optarg)+1);
      memset (options->conf_file,0,strlen(optarg)+1);
      strncpy(options->conf_file,optarg,strlen(optarg));
      break;
    case 'l':
      options->options |= (0x01 << 5);
      break;
    case 'L':
      options->options |= (0x01 << 11);
      break;
    case 'r':
      options->options |= (0x01 << 4);
      break;
    case 'a':
      options->options |= (0x01 << 3);
      break;
    case 'z':
      options->options |= (0x01 << 12);
      break;
    case 'i':
      options->options |= (0x01 << 2);
      break;

    default:
      synopsis();
    }
  }

  /*
  ** Si un argument ne correspond pas
  */

  if ((optarg = (char *) poptGetArg(context))) 
    synopsis();

  /*
  ** Liberation memoire
  */

  poptFreeContext(context);
}

void server_or_client(S_OPTIONS *options)
{
  /*
  ** Pas de flag client NI de flag serveur
  ** puis un flag client ET un flag serveur
  */
  if ((!options->srv_options->he && !options->cl_options->he)
      || (options->srv_options->he && options->cl_options->he))
    cctt_error_and_exit(stderr,"%s","[Options] : Client and/or server ?!?!?\n");

  /* 
  ** On flag les options pour un client,
  ** pour le serveur, la macro renverra ok
  */
  if (options->cl_options->he)
    options->options |= (0x01 << 0);    
}

void use_default_port(S_OPTIONS *options)
{

  /*
  ** On positionne le port par defaut si 
  ** celui n'a pas ete precise en parametres -
  ** et on verifie que l'utilisateur ne s'est pas
  ** trompe de flag.
  */
  
  if (OPT_CL_OR_SRV())
    {
      if (options->srv_options->sockaddr.sin_port)
	cctt_error_and_exit(stderr,"%s","[Options] : Server port set with -c Client flag ?!?!?\n");
      if (!options->cl_options->sockaddr.sin_port)
	options->cl_options->sockaddr.sin_port = htons(DEFAULT_CL_CONNECT_PORT);
    }
  else
    {
      if (options->cl_options->sockaddr.sin_port)
	cctt_error_and_exit(stderr,"%s","[Options] : Client port set with -s Server flag ?!?!?\n");
      if (!options->srv_options->sockaddr.sin_port)
	options->srv_options->sockaddr.sin_port = htons(DEFAULT_SRV_LISTEN_PORT);
    }
}

void check_cl_args_options(S_OPTIONS *options)
{

  if (OPT_CL_OR_SRV())
    {
      /* Si plusieurs flags d'option client sont indiques */
      if ((OPT_RSHELL() && (OPT_CL_PROX_MOD() || OPT_CL_LIST_MOD() || OPT_CL_REV_PROX_MOD())) ||
	  (OPT_CL_PROX_MOD() && (OPT_CL_LIST_MOD() || OPT_CL_REV_PROX_MOD())) ||
	  (OPT_CL_LIST_MOD() && OPT_CL_REV_PROX_MOD()))
	cctt_error_and_exit(stderr,"%s","[Options] : Client must have ONE of the flags -r , -a, -z or -i !\n");
      /* Si aucune option n'est indique, on active le mode normal */
      if (!OPT_RSHELL() && !OPT_CL_PROX_MOD() && !OPT_CL_LIST_MOD() && !OPT_CL_REV_PROX_MOD())
	options->options |= (0x01 << 1);
    }
  else
    {
      /* Si des flags d'option client sont indiques et que
	 c'est le serveur qui est active */
      if (OPT_RSHELL() || OPT_CL_PROX_MOD() || OPT_CL_LIST_MOD() || OPT_CL_REV_PROX_MOD())
	cctt_error_and_exit(stderr,"%s","[Options] : Using Client flag (-r,-a,-z,-i) in server mode ?!?!?\n");
    }
}

void use_default_conf_file(S_OPTIONS *options)
{
  if (OPT_CL_OR_SRV())
    {
      /* Si le fichier de conf par defaut du client doit etre positionne */
      if (!OPT_CONFFILE())
	{
	  options->options |= (0x01 << 6);
	  options->conf_file = (char *) malloc (strlen(DEFAULT_CL_CONF_FILE)+1);
	  memset (options->conf_file,0,strlen(DEFAULT_CL_CONF_FILE)+1);
	  strncpy(options->conf_file,DEFAULT_CL_CONF_FILE,strlen(DEFAULT_CL_CONF_FILE));
	}
    }  
  else
    {
      /* Si le fichier de conf par defaut du serveur doit etre positionne */
      if (!OPT_CONFFILE())
	{
	  options->options |= (0x01 << 6);
	  options->conf_file = (char *) malloc (strlen(DEFAULT_SRV_CONF_FILE)+1);
	  memset (options->conf_file,0,strlen(DEFAULT_SRV_CONF_FILE)+1);
	  strncpy(options->conf_file,DEFAULT_SRV_CONF_FILE,strlen(DEFAULT_SRV_CONF_FILE));
	}
    }  
}

void  use_default_channel_type(S_OPTIONS *options)
{
  if (!options->channel)
    {
      options->channel = (char *) malloc (strlen(DEFAULT_CHANNEL_TYPE)+1);
      memset (options->channel,0,strlen(DEFAULT_CHANNEL_TYPE)+1);
      strncpy(options->channel,DEFAULT_CHANNEL_TYPE,strlen(DEFAULT_CHANNEL_TYPE));
    }
}

void check_activated_logging(S_OPTIONS *options)
{
  int test_fd;

  /* Suremment pas en mode client */
  if (OPT_CL_OR_SRV())
    {
      if (OPT_LOGGING() || OPT_LOGGING_SYSLOG())
	cctt_error_and_exit(stderr,"%s","[Options] : Using Server flag (-l, -L) in client mode ?!?!?\n");
    }
  else
    {
      if (OPT_LOGGING_SYSLOG()) /* Si syslog demande */
	{
	  /* On verifie que la facilite et la priorite ne sont pas nulles */
	  if (!DEFAULT_SYSLOG_FACILITY || !DEFAULT_SYSLOG_PRIORITY)
	    cctt_error_and_exit(stderr,"%s","[Options] : Error in syslog parameters (includes/definitions.h) !\n");
	  cctt_debug(stderr,"%s","Init : Activation logging sur syslog : OK");
	}
      if (OPT_LOGGING()) /* Si logging demande */
	{
	  if (!DEFAULT_LOG_FILE)
	    cctt_error_and_exit(stderr,"%s","[Options] : Error in logfile parameters (include/defintions.h) !\n");
	  /* Verifie que le fichier de log pourra etre ouvert en append */
	  if ((test_fd = open(DEFAULT_LOG_FILE,O_CREAT|O_APPEND,0666)) < 1)
	    cctt_error_and_exit(stderr,"%s","[Options] : Cannot open logfile (include/defintions.h) !\n");
	  close (test_fd);
	  cctt_debug(stderr,"%s","Init : Activation logging dans logfile : OK");
	}
    }
}

void check_args(S_OPTIONS *options)
{

  /* Client ou serveur ? */
  server_or_client(options);

  /* Positionne le port par default ?  */
  use_default_port(options);

  /* Verifie si les options -r, -a ou -i du client sont correctes 
     et si ces options ne sont pas passees pour un serveur */
  check_cl_args_options(options);

  /* Verifie si le fichier de config est positionne */
  use_default_conf_file(options);

  /* Verifie si le type de canal a ete precise */
  use_default_channel_type(options);

  /* Verifie si le logging etant active les parametres sont corrects */
  check_activated_logging(options);
}
