/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: channel_do_socket.c,v 1.6 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Fonction principale executee par le pointeur sur fonction
** gerant les types de canaux.
*/

void channel_do_socket(S_OPTIONS *options)
{
  int ret_val;

  /* 
  ** On verifie que le fichier de configuration ne contient pas 
  ** de coordonnees de proxy 
  */
  if (OPT_CL_CHAN_PROXY())
    cctt_error_and_exit(stderr,"%s","[Conf] : CHANNEL_PROXY_ configuration directives used ?!?!?");

  if (OPT_CL_PROX_MOD()) /* Client en mode proxy */
    ret_val = client_binds_local_proxy(options);
  else
    {
      /*
      ** Cet appel cree la socket et stocke le socket descriptor
      ** dans la struct options a l'endroit approprie.
      */
      client_socket(options);
      
      /*
      ** On commence par identifier le client
      */
      if (OPT_CL_CONNECTED())
	cl_identification(options,0);
      
      /*
      ** Si le client n'a pas ete identifie,
      ** la connexion a ete resetee par le serveur.
      */
      
      if (OPT_CL_CONNECTED()) /* On teste si la socket est up */
	{
	  if (OPT_RSHELL()) /* Si le client est un reverse shell */
	    {
	      ret_val = channel_do_socket_rshell(options);
	      if (ret_val == -1) cctt_debug(stderr,"%s","[TMP] : Une erreur est survenue : channel_do_socket.c\n");
	    }
	  if (OPT_CL_NORMAL_MOD() || OPT_CL_LIST_MOD())
	    {
	      /* Shell normal ou demande de la liste de proxy */
	      ret_val = channel_do_socket_go(options);
	      if (ret_val == -1) cctt_debug(stderr,"%s","[TMP] : Une erreur est survenue : channel_do_socket.c\n");
	    }
	  if (OPT_CL_REV_PROX_MOD())
	    {
	      ret_val = client_asks_reverse_prox_mode(options);
	      if (ret_val == -1) cctt_debug(stderr,"%s","[TMP] : Une erreur est survenue : channel_do_socket.c\n");
	    }
	}
      /* On ferme la socket */
      if (OPT_CL_CONNECTED()) /* On teste si la socket est up */
	client_socket_close(options);
    }
}
