/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: channel_do_socket_go.c,v 1.5 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Cette fonction implemente la partie cliente utilisee quand le serveur
** offre un shell sur une socket.
*/

int channel_do_socket_go(S_OPTIONS *options)
{
  int socket_up=1;   /* Condition de sortie pour la boucle */
  fd_set fdset_read; /* Set de Fds pour le select */
  fd_set fdset_exp;  /* Set de Fds pour le select */
  struct timeval tv; /* Structure de temps pour le select */
  int nrshell=0;     /* Nombre d'octets lus sur le shell courant */
  int nrsocket=0;    /* Nombre d'octets lus sur la socket */
  int sel_ret;       /* Valeur de retour du select */
  int nw=0;          /* Nombre d'octets ecrits */
  int http_turn=0;

  /* Le buffer utilise pour le transfert (precise dans includes/definitions.h )*/
  char buffer[SOCKET_BUFFER_SIZE+1];
  
  /* Initialisation du set pour le select */
  FD_ZERO(&fdset_read);
  FD_SET(0,&fdset_read);
  FD_SET(options->cl_options->sd,&fdset_read);
  FD_ZERO(&fdset_exp);
  FD_SET(options->cl_options->sd,&fdset_exp);

  /* Tant que notre socket vers le serveur est UP */
  while (socket_up == 1)
    {
      /* On met a jour la structure de temps (0,1 sec) */
      tv.tv_sec = 0;
      tv.tv_usec = 100;
      
      /* 
      ** Si une erreur survient, la fonction retourne avec erreur 
      ** Sinon, verifie si un evenement socket ou stdin survient
      */
      if ((sel_ret = select(options->cl_options->sd+1,&fdset_read,0,&fdset_exp,&tv)) < 0)
	return(-1);
      else
	{
	  if (sel_ret != 0)
	    {
	      /* 
	      ** Si quelquechose se presente sur stdin,
	      ** on lit et on ecrit dans la socket
	      **
	      ** Si on est en mode HTTP, on verifie qu'on a eu une reponse du serveur avant d'envoyer
	      ** une nouvelle requete
	      */
	      if (tell_me_about_encoding(options) == 2)
		{
		  if (options->cl_options->http_turn == 0 && FD_ISSET(0,&fdset_read))
		    {
		      memset(buffer,0,options->cl_options->srv_socket_buffer_size+1);
		      nrshell = read(0,buffer,options->cl_options->srv_socket_buffer_size);
		      if (nrshell > 0)
			nw = do_client_has_something_to_encode(options,options->cl_options->sd,buffer,nrshell);
		      options->cl_options->http_turn = 1;
		    }
		}
	      else
	      if (FD_ISSET(0,&fdset_read))
		{
		    memset(buffer,0,options->cl_options->srv_socket_buffer_size+1);
		    nrshell = read(0,buffer,options->cl_options->srv_socket_buffer_size);
		  if (nrshell > 0)
		    nw = do_client_has_something_to_encode(options,options->cl_options->sd,buffer,nrshell);
		}
	      /* 
	      ** Si quelquechose se presente sur la socket, on lit
	      ** et on envoie a stdout.
	      ** Si on lit 0 sur la socket ici (dans le select),
	      ** c'est que la socket est down
	      */
	      if (FD_ISSET(options->cl_options->sd,&fdset_read))
		{
		  memset(buffer,0,SOCKET_BUFFER_SIZE+1);
		  nrsocket = read(options->cl_options->sd,buffer,SOCKET_BUFFER_SIZE);
		  if (nrsocket > 0)
		    do_client_has_something_to_decode(options,1,buffer,nrsocket);
		  else
		    socket_up=0;
		  /* On met a jour le turn client/serveur au cas ou on est en mode HTTP */
		  options->cl_options->http_turn = 0;
		}
	    }
	}
      /*
      ** Ceci est specifique au mode HTTP
      ** Il est possible que le serveur veuille nous envoyer des donnees alors qu'on ne peut
      ** lui envoyer de requete avec des donnees arbitraires puisqu'au tour precedent, le
      ** serveur ne nous a pas repondu...
      ** J'ai choisi un pop de cette facon la : chaque select fait 100 msecs.
      ** le nombre de msecs entre chaque requetes est dans options->cl_options->http_mode_del_bet_con.
      ** => Donc a chaque tour de select on incremente une variable de 100 et quand cette variable est
      ** superieure a 2*options->cl_options->http_mode_del_bet_con, on la met a zero et on remet le
      ** http_turn global a 0.
      */
      if (tell_me_about_encoding(options) == 2)
	{
	  http_turn += 100;
	  if (http_turn > options->cl_options->http_mode_del_bet_con * 2 &&
	      options->cl_options->http_turn == 1)
	    {
	      http_turn = 0;
	      options->cl_options->http_turn = 0;
	    }
	}

      /* 
      ** On reinitialise le set du select sinon,
      ** au prochain coup l'appel select sera bloquant.
      */
      FD_ZERO(&fdset_read);
      FD_SET(0,&fdset_read);
      FD_SET(options->cl_options->sd,&fdset_read);
      FD_ZERO(&fdset_exp);
      FD_SET(options->cl_options->sd,&fdset_exp);
    }
  return (0);
}
