/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: channels.c,v 1.9 2003/06/11 13:14:19 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Pour chaque type de canal susceptible d'etre passe en parametre
** dans la ligne de commande, nous avons le type dans un champ de
** de cette structure, suivi d'un pointeur sur la fonction
** correspondante.
** Ce pointeur sur fonction est soit pour le client, soit pour le serveur.
** Pointeur sur fonction ? le prototypage est realise dans 
** includes/channels.h et non dans les .h de chaque fonction.
** Les fonctions clientes sont dans src/channels et les fonctions serveur
** dans src/server/
*/

T_channel_types channel_types_array[] = {
  {"socket",channel_do_socket,server_do_socket},
  {"socket_encode",channel_do_socket,server_do_socket},
  {"socket_http_proxy",channel_do_socket_http_proxy,server_do_socket},
  {"socket_http_proxy_encode",channel_do_socket_http_proxy,server_do_socket},
  {"client_only_with_http_proxy",channel_do_client_only_with_http_proxy,server_do_socket},
  {"http_post",channel_do_http,server_do_socket},
  {"http_post_proxy",channel_do_http,server_do_socket},
  {"http_post_proxy2",channel_do_http,server_do_socket},
  {"test",channel_do_test,server_do_test},
  {NULL,NULL,NULL}
};

/*
** Verifie si le type de canal demande existe a partir 
** de la structure precedente.
*/

void check_channel(S_OPTIONS *options)
{
  int i,exit;
  
  for (i=0,exit=0;channel_types_array[i].channel_name != NULL;i++)
    if ((strcmp(options->channel,channel_types_array[i].channel_name) == 0))
      exit=1;
  if (exit == 0)
    cctt_error_and_exit(stderr,"%s","[Options] : Channel asked doesn't exist ! ");

  cctt_debug(stderr,"Init : Type de canal demande (%s) : OK",options->channel);
}

/*
** on lance le traitement associe au canal demande.
** Ce lancement est effectue par l'appel a la fonction
** correspondante a qui est passee la structure options
** qui est desormais correctement renseignee.
*/

void do_channel(S_OPTIONS *options, int sd, char *buffer, int nwr)
{
  int i,exit;
  
  for (i=0,exit=0;channel_types_array[i].channel_name != NULL;i++)
    if ((strcmp(options->channel,channel_types_array[i].channel_name) == 0))
      {
	if (OPT_CL_OR_SRV())
	  channel_types_array[i].channel_type_client(options);
	else
	  channel_types_array[i].channel_type_server(options,sd,buffer, nwr);
      }
}
