/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: channels_functions.c,v 1.6 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Sont regroupees ici les fonctions reseau relatives
** a l etablissement des canaux
*/

/*
** Etablit la socket standard 
** Socket descriptor : options->cl_options->sd
** Structure SockAddr : options->cl_options->sockaddr_in
** Protocole : OPT_CL_TCP() ou OPT_CL_UDP()
*/

void client_socket(S_OPTIONS *options)
{
  int keepalive;

  if (OPT_CL_TCP()) /* TCP */
    {
      /* 
      ** On affiche l'erreur mais on ne quitte pas
      ** => Le choix est laisse a la fonction de gestion du canal 
      */
      if ((options->cl_options->sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	cctt_error(stderr,"%s","[Client] : Socket creation !");

      /*
      ** En KeepAlive
      */
      setsockopt(options->cl_options->sd,SOL_SOCKET,SO_KEEPALIVE, (int *)&keepalive, sizeof(keepalive));

      /*
      ** Meme raisonnement que precedemment
      */
      if (connect(options->cl_options->sd, 
		  (struct sockaddr *)&options->cl_options->sockaddr,
		  sizeof(struct sockaddr)) == -1)
	{
	  options->cl_options->sd = -1;
	  cctt_error(stderr,"%s","[Client] : Socket connection !");
	}
      
      if (OPT_CL_CONNECTED()) cctt_debug(stderr,"Conn : Connexion etablie sur le sd %d",options->cl_options->sd);
    }
  else /* UDP */
    {
      /* 
      ** On affiche l'erreur mais on ne quitte pas
      ** => Le choix est laisse a la fonction de gestion du canal 
      */
      if ((options->cl_options->sd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
	cctt_error(stderr,"%s","[Client] : Socket creation !");

      /*
      ** Meme raisonnement que precedemment
      */
      if (connect(options->cl_options->sd, 
		  (struct sockaddr *)&options->cl_options->sockaddr,
		  sizeof(struct sockaddr)) == -1)
	{
	  options->cl_options->sd = -1;
	  cctt_error(stderr,"%s","[Client] : Socket connection !");
	}
      
      if (OPT_CL_CONNECTED()) cctt_debug(stderr,"Conn : Connexion etablie sur le sd %d",options->cl_options->sd);
    }
}

void client_socket_close(S_OPTIONS *options)
{
  if (OPT_CL_TCP())
    {
      if (OPT_CL_CONNECTED())
	{
	  close(options->cl_options->sd);
	  cctt_debug(stderr,"Conn : Connexion ferme pour le sd %d",options->cl_options->sd);
	  options->cl_options->sd = -1;
	}
    }
  else
    {
      if (OPT_CL_CONNECTED())
	{
	  close(options->cl_options->sd);
	  cctt_debug(stderr,"Conn : Connexion ferme pour le sd %d",options->cl_options->sd);
	  options->cl_options->sd = -1;
	}
    }
}
