/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: check_and_send_fake_urls.c,v 1.2 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Les fonctions qui suivent sont chargee d'envoyer les fakes urls au serveur.
*/

/*
** Fonction chargee de creer le fake header
** Retourne NULL en cas de probleme dans la fake url
*/

char *build_fake_http_header(S_OPTIONS *options, char *fake_url)
{
  char *header,*ptr;
  int tmp_size;

  if (tell_me_about_http(options) == 2)
    header = update_http_request_for_a_proxy(options,fake_url);
  else
    {
      /* Setting the fake request header field */
      tmp_size = strlen(fake_url)+2;
      header = (char *) malloc (tmp_size+1);
      ptr = header;
      memset(header,0,tmp_size+1);
      strncpy(ptr,fake_url,strlen(fake_url));
      ptr = ptr+strlen(fake_url);
      *ptr = '\n';
    }

  /* Setting the host header field */
  tmp_size = strlen(HTTP_STD_HOST)+strlen(options->cl_options->http_mode_hostname)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_HOST,strlen(HTTP_STD_HOST));
  ptr = ptr+strlen(HTTP_STD_HOST);
  strncpy(ptr,options->cl_options->http_mode_hostname,strlen(options->cl_options->http_mode_hostname));
  ptr = ptr+strlen(options->cl_options->http_mode_hostname);
  *ptr = '\n';

  /* Don't forget the last '\n' */
  *(++ptr) = '\n';

  return (header);
}

/*
** Fonction chargee d'envoyer une fake url, de lire ce qui est renvoyee et de
** tout jeter a la poubelle.
** On forke car ca permet d'utiliser la structure cliente sans la modifier...
** Ne renvoie rien de particulier...
*/

int fork_and_send_fake_url(S_OPTIONS *options,char *fake_url)
{
  int fils,pid,test;
  int nr;
  char *request;
  char buffer[1024];

  cctt_debug(stderr,"[HTTP] -> Out : Sending the '%s' fake url for %d",fake_url,options->cl_options->sd);

  if ((fils = fork()) == 0)
    {
      /* On est dans le fils */

      /* On attend un leger delai relatif au delai configure entre chaque requete HTTP */
      usleep(options->cl_options->http_mode_del_bet_con);
      client_socket(options);
      
      /*
      ** Si on est dans un mode http avec proxy
      */
      if (OPT_CL_CONNECTED() && OPT_CL_CHAN_PROXY())
	client_use_border_proxy(options);

      if (!OPT_CL_CONNECTED())
	_exit (-1);

      /* On balance la requete... */
      if ((request = build_fake_http_header(options,fake_url)) == NULL)
	_exit(-1);
      if ((nr = write(options->cl_options->sd,request,strlen(request))) <= 0)
	_exit(-1);

      /* On lit ce que l'extremite renvoie */
      memset(buffer,0,1024);
      nr = read(options->cl_options->sd,buffer,1023);

      if (nr <= 0)
	_exit(-1);
      
      if (OPT_CL_CONNECTED()) /* On teste si la socket est up */
	client_socket_close(options);

      exit (0);
    }
  else
    {
      /* On est dans le pere */
      if (fils == -1)
	{
	  cctt_debug(stderr,"[HTTP] -> Out : Error fork while trying to send '%s' for %d",fake_url,options->cl_options->sd);
	  return (0);
	}
      /* Tout fonctionne, on attend que le fils quitte ... */
      cctt_debug(stderr,"[HTTP] -> Out : Waiting for fake_urls sending child (%d) to exit...",fils);
      while ((pid = wait3(&test,WNOHANG,NULL)) != fils)
	;
      if (test == 0)
	cctt_debug(stderr,"[HTTP] -> Out : fake '%s' for %d was sended",fake_url,options->cl_options->sd);
      else
	cctt_debug(stderr,"[HTTP] -> Out : fake '%s' for %d failed",fake_url,options->cl_options->sd);
    }
  return (0);
}

/*
** Parse la liste de fake urls et appelle la fonction chargee de les envoyer
*/

int parse_fake_urls_list(S_OPTIONS *options)
{
  int i=0;

  while (options->cl_options->http_mode_fake_urls[i] != NULL)
    {
      fork_and_send_fake_url(options,options->cl_options->http_mode_fake_urls[i]);
      i++;
    }
  return (0);
}

/*
** Cette fonction est chargee de generer les fakes urls configurees
** Ne renvoie rien de particulier pour la fonction appellante...
** Les return sont la pour faciliter la lecture
*/

int check_and_send_fake_urls(S_OPTIONS *options)
{
  unsigned int random;
  unsigned int max;
  int cpt=0;

  if (options->cl_options->http_mode_fake_urls_freq == -1)
    {
      cctt_debug(stderr,"[HTTP] -> Out : Random Fake urls sending for %d.",options->cl_options->sd);

      /* Tire un chiffre random a partir de la fonction de generation des magic numbers */
      random = give_me_random_client_number(options);
      /* Ce nombre a x chiffres (voir la fonction precedente) */
      for (max=10;cpt<nb_char_in_int(random)-1;cpt++)
	max *= 10;
      if (random < (max/2))
	{
	  parse_fake_urls_list(options);
	  cctt_debug(stderr,"[HTTP] -> Out : Random Fake urls sending for %d : Ok.",options->cl_options->sd);
	}
      else
	cctt_debug(stderr,"[HTTP] -> Out : Random Fake urls sending for %d : Not this time.",options->cl_options->sd);
      return (0);
    }
  if (options->cl_options->http_mode_fake_urls_freq == 1)
    {
      /* Envoie a chaque coup */
      cctt_debug(stderr,"[HTTP] -> Out : 1/1 Fake urls sending for %d.",options->cl_options->sd);
      parse_fake_urls_list(options);
      cctt_debug(stderr,"[HTTP] -> Out : 1/1 Fake urls sending for %d : Done.",options->cl_options->sd);
      return (0);
    }
  if (options->cl_options->http_mode_fake_urls_freq > 1)
    {
      /* N'envoie que chaque x requete */
      cctt_debug(stderr,"[HTTP] -> Out : 1/x Fake urls sending for %d (current nb_req is %d).",options->cl_options->sd,
		 options->cl_options->http_mode_req_nb);
      if (options->cl_options->http_mode_req_nb % options->cl_options->http_mode_fake_urls_freq == 0 &&
	  options->cl_options->http_mode_req_nb != 0)
	{
	  parse_fake_urls_list(options);
	  cctt_debug(stderr,"[HTTP] -> Out : 1/x Fake urls sending for %d (current nb_req is %d) : Done.",options->cl_options->sd,
		     options->cl_options->http_mode_req_nb);
	}
      else
	cctt_debug(stderr,"[HTTP] -> Out : 1/x Fake urls sending for %d (current nb_req is %d) : Waiting.",options->cl_options->sd,
		   options->cl_options->http_mode_req_nb);
      return (0);
    }
  return (-1);
}
