/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: cl_build_and_send_http_functions.c,v 1.3 2003/06/11 13:14:19 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Toutes les fonctions presentes ici permettent de creer et d'emettre des
** requetes HTTP pour les clients CCTT.
**
** Les defines sont dans includes/configuration.h
*/

/*
** Prend une requete en parametre et renvoie la requete construite
** pour un proxy http
*/

char *update_http_request_for_a_proxy(S_OPTIONS *options, char *request)
{
  char *header,*ptr;
  int tmp_size;
  int space,cpt;

  /* Setting the fake request header field */
  tmp_size = strlen(request)+strlen(options->cl_options->http_mode_hostname)+
    nb_char_in_int(options->cl_options->dest_server_port)+8+3;
  header = (char *) malloc (tmp_size+1);
  ptr = header;
  memset(header,0,tmp_size+1);
  
  for (cpt=0,space=0;space == 0 && cpt<strlen(request);cpt++)
    if (request[cpt] == ' ')
      space = cpt;
  if (space == 0)
    return(NULL);
  
  strncpy(ptr,request,space+1);
  ptr = ptr+space+1;
  strncpy(ptr,"http://",7);
  ptr = ptr+7;
  strncpy(ptr,options->cl_options->http_mode_hostname,strlen(options->cl_options->http_mode_hostname));
  ptr = ptr+strlen(options->cl_options->http_mode_hostname);
  *ptr = ':';
  ptr = ptr+1;
  strncpy(ptr,get_str_from_int(options->cl_options->dest_server_port),
	  nb_char_in_int(options->cl_options->dest_server_port));
  ptr = ptr+nb_char_in_int(options->cl_options->dest_server_port);
  strncpy(ptr,request+space+1,strlen(request)-(space+1));
  ptr = ptr+(strlen(request)-(space+1));
  *ptr = '\n';
  *(ptr+1) = '\0';

  return(header);
}

/*
** Cree le header d'une requete POST
*/

char *build_http_post_header(S_OPTIONS *options, int body_size)
{
  char *header, *ptr;
  int tmp_size;

  /* Setting the request header field */
  tmp_size = strlen(HTTP_STD_POST)+strlen(HTTP_STD_HTTPV)
    +strlen(options->cl_options->http_mode_uri)+2;
  header = (char *) malloc (tmp_size+2);
  ptr = header;
  memset(header,0,tmp_size+2);
  strncpy(ptr,HTTP_STD_POST,strlen(HTTP_STD_POST));
  ptr = ptr+strlen(HTTP_STD_POST);
  strncpy(ptr,options->cl_options->http_mode_uri,strlen(options->cl_options->http_mode_uri));
  ptr = ptr+strlen(options->cl_options->http_mode_uri);
  strncpy(ptr,HTTP_STD_HTTPV,strlen(HTTP_STD_HTTPV));
  ptr = ptr+strlen(HTTP_STD_HTTPV);

  if (tell_me_about_http(options) == 2) /* Si mode http avec proxy, on construit la bonne requete */
    header = update_http_request_for_a_proxy(options,header);
  else
    *ptr = '\n';

  /* Setting the host header field */
  tmp_size = strlen(HTTP_STD_HOST)+strlen(options->cl_options->http_mode_hostname)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_HOST,strlen(HTTP_STD_HOST));
  ptr = ptr+strlen(HTTP_STD_HOST);
  strncpy(ptr,options->cl_options->http_mode_hostname,strlen(options->cl_options->http_mode_hostname));
  ptr = ptr+strlen(options->cl_options->http_mode_hostname);
  *ptr = '\n';

  /* Setting the content-length */
  tmp_size = strlen(HTTP_STD_CLENGTH)+nb_char_in_int(body_size)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_CLENGTH,strlen(HTTP_STD_CLENGTH));
  ptr = ptr+strlen(HTTP_STD_CLENGTH);
  strncpy(ptr,get_str_from_int(body_size),nb_char_in_int(body_size));
  ptr = ptr+nb_char_in_int(body_size);
  *ptr = '\n';

  /* Setting the content-type */
  tmp_size = strlen(HTTP_STD_CTYPE)+strlen(options->cl_options->http_mode_contenttype)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_CTYPE,strlen(HTTP_STD_CTYPE));
  ptr = ptr+strlen(HTTP_STD_CTYPE);
  strncpy(ptr,options->cl_options->http_mode_contenttype,strlen(options->cl_options->http_mode_contenttype));
  ptr = ptr+strlen(options->cl_options->http_mode_contenttype);
  *ptr = '\n';

  /* Setting the Proxy-Connection: Keep-Alive field */
  tmp_size = strlen("Proxy-Connection: Keep-Alive")+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,"Proxy-Connection: Keep-Alive",strlen("Proxy-Connection: Keep-Alive"));
  ptr = ptr+strlen("Proxy-Connection: Keep-Alive");
  *ptr = '\n';

  /* Setting the Pragma: nocache field */
  tmp_size = strlen("Pragma: nocache")+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,"Pragma: nocache",strlen("Pragma: nocache"));
  ptr = ptr+strlen("Pragma: nocache");
  *ptr = '\n';

  /* Don't forget the last '\n' */
  *(++ptr) = '\n';

  return (header);
}

/*
** Cree une requete POST correcte et envoie la en gerant les options du mode HTTP
*/

int build_and_send_http_post(S_OPTIONS *options, int sd, char *buffer, int size)
{
  char *header,*request;
  int body_size,request_size;
  int nwrite=-1;
  int cpt,cl_num=0;

  char *rebuilded,*rebuilded2;
  int rebuilded_size=0,rebuilded_size2=0;

  /*
  ** Est-ce qu'un Top padding doit etre ajoute ?
  */
  if (options->cl_options->http_mode_top_padding)
    {
      rebuilded_size = size+strlen(options->cl_options->http_mode_top_padding);
      rebuilded = (char *) malloc(rebuilded_size);
      memcpy(rebuilded,options->cl_options->http_mode_top_padding,
	     strlen(options->cl_options->http_mode_top_padding));
      memcpy(rebuilded+strlen(options->cl_options->http_mode_top_padding),buffer,size);
    }
  else
    {
      rebuilded_size = size;
      rebuilded = buffer;
    }

  /*
  ** Est-ce qu'un Bottom padding doit etre ajoute ?
  */
  if (options->cl_options->http_mode_bottom_padding)
    {
      rebuilded_size2 = rebuilded_size + strlen(options->cl_options->http_mode_bottom_padding);
      rebuilded2 = (char *) malloc(rebuilded_size2);
      memcpy (rebuilded2,rebuilded,rebuilded_size);
      memcpy(rebuilded2+rebuilded_size,options->cl_options->http_mode_bottom_padding,
	     strlen(options->cl_options->http_mode_bottom_padding));
      free (rebuilded);
      rebuilded = rebuilded2;
      rebuilded_size = rebuilded_size2;
    }

  body_size = rebuilded_size;
  buffer = rebuilded;

  /* Construit le header */
  header = build_http_post_header(options,body_size);
  request_size = strlen(header)+body_size;

  /* Construit la requete */
  request = (char *) malloc(request_size+1);
  memset(request,0,request_size+1);
  strncpy(request,header,strlen(header));
  memcpy(request+strlen(header),buffer,body_size);

  /* 
  ** Pour gerer a la fois un client en mode proxy et un client en mode non-proxy,
  ** on utilise le sd options->cl_options->sd auquel on affecte le bon socket descriptor
  ** selon le mode utilise.
  */

  if (OPT_CL_PROX_MOD()) /* Client en mode proxy */
    {
      options->cl_options->sd = sd;

      /* On cherche le numero de client auquel est affecte ce sd pour recuperer les infos */
      for (cl_num=0,cpt=0;cpt<options->cl_options->client_sd_table_max&&cl_num==0;cpt++)
	if (options->cl_options->client_sd_table[cpt].remote_sd == sd)
	  {
	    /* On recupere le compteur de requete */
	    options->cl_options->http_mode_req_per_con_ctr = 
	      options->cl_options->client_sd_table[cpt].http_mode_req_per_con_ctr;
	    options->cl_options->http_mode_req_nb =
	      options->cl_options->client_sd_table[cpt].http_mode_req_nb;
	    cl_num = cpt;
	  }
    }

  cctt_debug(stderr,"[HTTP] -> Out : A %d bytes (cl : %d) HTTP POST (num %d) request is about to be sent on sd %d.",request_size,body_size,options->cl_options->http_mode_req_per_con_ctr,sd);

  /*
  ** On verifie si on doit generer des fakes urls.
  */

  if (options->cl_options->http_mode_fake_urls_freq)
    check_and_send_fake_urls(options);
  
  if (!OPT_CL_CONNECTED() || options->cl_options->http_mode_req_per_con_ctr >= options->cl_options->http_mode_req_per_con)
    {
      /* 
      ** Si on est pas connecte ou que le nombre de requetes emises est > a la borne max 
      **  => On deconnecte et reconnecte
      */
      if (OPT_CL_CONNECTED()) /* On teste si la socket est up */
	client_socket_close(options);
      options->cl_options->http_mode_req_per_con_ctr = 0;
      client_socket(options);

      /*
      ** Si on est dans un mode http avec proxy
      */
      if (OPT_CL_CONNECTED() && OPT_CL_CHAN_PROXY())
	client_use_border_proxy(options);

      /*
      ** Si on est en mode proxy, il nous faut mettre a jour l'element relatif au client
      */
      if (OPT_CL_PROX_MOD() && cl_num != 0)
	{
	  options->cl_options->client_sd_table[cl_num].remote_sd = options->cl_options->sd;
	  options->cl_options->client_sd_table[cl_num].http_mode_req_per_con_ctr =
	    options->cl_options->http_mode_req_per_con_ctr;
	  options->cl_options->client_sd_table[cl_num].magic_number = 
	    options->cl_options->magic_number;
	}

      if (!OPT_CL_CONNECTED())
	{
	  if (OPT_CL_PROX_MOD())
	    {
	      /* On ne quitte pas en mode proxy ! */
	      return (-1);
	    }
	  else
	    {
	      /* La connexion a plante, on pourrait ici preciser qu'il faut tester a nouveau x fois */
	      cctt_error_and_exit(stderr,"%s","[HTTP] : build_and_send_http_post() : Couldn't connect to server !!!");
	    }
	}
      else
	{
	  /* Et ben c'est ok, on se reidentifie mais en disant qu'on etait deja la avant */

	  cl_identification(options,1);
	  
	  if (!OPT_CL_CONNECTED())
	    {
	      if (OPT_CL_PROX_MOD())
		{
		  /* On ne quitte pas en mode proxy ! */
		  return (-1);
		}
	      else
		{
		  /* L'identification a plante, on pourrait ici preciser qu'il faut tester a nouveau x fois */
		  cctt_error_and_exit(stderr,"%s","[HTTP] : build_and_send_http_post() : server didn't ident us!!!");
		}
	    }
	}
    }
  
  /*
  ** La on est connecte : On attend comme le compteur de temps nous l'indique et on envoie
  */
  cctt_debug(stderr,"[HTTP] -> Out : Sleeping %d msec between each request.",options->cl_options->http_mode_del_bet_con);
  usleep(options->cl_options->http_mode_del_bet_con);
  options->cl_options->http_mode_req_per_con_ctr +=1;
  options->cl_options->http_mode_req_nb +=1;

  if (OPT_CL_PROX_MOD() && cl_num != 0)
    {
      options->cl_options->client_sd_table[cl_num].http_mode_req_per_con_ctr =
	options->cl_options->http_mode_req_per_con_ctr;
      options->cl_options->client_sd_table[cl_num].http_mode_req_nb =
	options->cl_options->http_mode_req_nb;
    }
  
  /*
  ** Attention, ici on n'ecrit pas sur le sd qu'envoie la function appelante
  ** car il est possible que le sd ait change...
  */
  nwrite = write(options->cl_options->sd,request,request_size);
  cctt_debug(stderr,"[HTTP] -> Out : Wrote %d bytes for a %d bytes request.",nwrite,request_size);

  /* Fait le menage */
  if (*request)
    free (request);
  
  /*
  ** Ici on pourrait choisir de retourner une valeur de test plutot que de faire des
  ** cctt_error_and_exit() en cas de probleme. La fonction appellante serait chargee
  ** de faire ce qui est necessaire.
  */

  return (nwrite);
}

/*
** Dans le cas ou on utilise un mode proxy, on renvoie des infos a l'utilisateur quand ca plante...
*/

int check_proxy_http_response(S_OPTIONS *options,int sd,char *buffer,int nbytes)
{
  char *tmp;

  if (nbytes > 0)
    {
      tmp = (char *) malloc (nbytes+1);
      memset(tmp,0,nbytes+1);
      memcpy(tmp,buffer,nbytes);
      cctt_verbose(stderr,"%s","[HTTP] -> In : It seems the proxy didn't accept our proxying request...\n");
      cctt_debug(stderr,"[HTTP] -> In : Proxy response was :\n###\n%s\n###\n",tmp);
      if (*tmp)
	free (tmp);
    }
  else
    cctt_verbose(stderr,"%s","[HTTP] -> In : It seems the proxy didn't accept our proxying request because I've got 0 bytes...\n");
  return (0);
}

/*
** Cette fonction gere les "erreurs" et renvoie NULL en structure quand une erreur
**  a ete trouvee !!!
** On utilise les deux fonctions de recherche de header et de content-length codees pour le serveur
*/

T_basic_decode cl_decode_http_post_response(S_OPTIONS *options,int sd,char *buffer,int nbytes)
{
  T_basic_decode basdec;
  int res,pos_data,cl;

  basdec.decoded = NULL;
  basdec.size = 0;

  if (nbytes == 0)
    {
      cctt_debug(stderr,"HTTP : Wont decode a 0 bytes HTTP response from %d\n",sd);
      return (basdec);
    }

  /* Ensuite on verifie qu'on a bien un header ET des donnees */
  if ((res = srv_check_http_post_header(options,buffer,nbytes)) <= 0)
    {
      cctt_debug(stderr,"HTTP POST : Couldn't find any correct header for client '%d'.",sd);
      if (res <= 0 && OPT_CL_CHAN_PROXY())
	check_proxy_http_response(options,sd,buffer,nbytes);
      return (basdec);
    }
  pos_data = res;
  /* On va maintenant chercher le content-length pour nos donnees dans le header seulement */
  if ((res = srv_get_http_post_content_length(options,buffer,pos_data)) <= 0)
    {
      cctt_debug(stderr,"HTTP POST : No content-length for client '%d'.",sd);
      if (res <= 0 && OPT_CL_CHAN_PROXY())
	check_proxy_http_response(options,sd,buffer,nbytes);
      return (basdec);
    }
  cl = res;

  /* 
  ** On sait ou commence les donnees et ou elles se terminent. 
  */

  if (OPT_DEBUG2()) cctt_debug(stderr,"[HTTP] -> In : Client %d : Total_size(%d) / Pos_data(%d) / CL(%d).",sd,nbytes,pos_data,cl);
  
  basdec.decoded = (char *) malloc(cl+1);
  memset(basdec.decoded,0,cl+1);
  memcpy(basdec.decoded,buffer+pos_data,cl);
  basdec.size = cl;
  
  return (basdec);
}



