/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: cl_identification.c,v 1.6 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Ces fonctions permettent d'appeler l'identification du client.
** On a un tableau de pointeur sur fonctions dans src/conf_file_IDENT.c
**   qui contient un mot cle present dans le fichier de configuration auquel
**   est associe un pointeur sur fonction.
** La definition de structure est presente dans includes/srv_ident_client.h
** La fonction definie ici doit etre (bien entendu) complementaire de la fonction
**  presente au sein du serveur qui est chargee de verifie que l'identification
**  est convenable (voir src/server/srv_ident_client.c).
*/

/*
** Renvoie un numero unique (?) pour chacun des clients
*/

unsigned int give_me_random_client_number(S_OPTIONS *options)
{
  unsigned int ok;
  unsigned int c_est_nul;
  unsigned int align;

  srand((unsigned int) time(NULL));
  ok = rand();
  c_est_nul = (unsigned int) getpid() + ok;
  srand((unsigned int) c_est_nul);
  align = rand();

  /* 
  ** La suite est debile mais il me faut une longueur fixe qui est utilisee
  ** pour la lecture dans les fonctions serveurs presentes dans srv_ident_client.c
  ** La valeur 7 est bien sur arbitraire ici comme dans les fonctions serveur.
  */

  if (nb_char_in_int(align) <= 7)
    while (nb_char_in_int(align) < 7)
      align *= 10;
  else
    while (nb_char_in_int(align) > 7)
      align /= 10;

  return (align);
}

/*
** Renvoie la chaine d'identification dans un format cleartext
*/

char *build_ident_string_clear(S_OPTIONS *options, int cont)
{
  char *ident_string;
  int magic;

  if (cont != 1)
    {
      magic = (int) give_me_random_client_number(options);
      options->cl_options->magic_number = magic;
      cctt_debug(stderr,"Ident : Setting the '%d' magic number.",magic);
    }
  else
    {
      magic = options->cl_options->magic_number;
      cctt_debug(stderr,"Ident : Keeping the '%d' magic number.",magic);
    }
  
  ident_string = (char *) malloc(strlen(options->ident_key)+1+nb_char_in_int(magic)+1);
  memset(ident_string,0,strlen(options->ident_key)+1+nb_char_in_int(magic)+1);
  ident_string[0] = cl_set_request(cont);
  strncpy(ident_string+1,options->ident_key,strlen(options->ident_key));
  strncpy(ident_string+1+strlen(options->ident_key),get_str_from_int(magic),nb_char_in_int(magic));
  return (ident_string);
}

/*
** Renvoie la chaine d'identification dans un format basic_encoding
*/

char *build_ident_string_basic(S_OPTIONS *options, int cont)
{
  char *ident_string;
  char *tmp_encoding;
  int magic;
  char *to_encode; /* La cle + le magic number */

  if (cont != 1)
    {
      magic = (int) give_me_random_client_number(options);
      options->cl_options->magic_number = magic;
      cctt_debug(stderr,"Ident : Setting the '%d' magic number.",magic);
    }
  else
    {
      magic = options->cl_options->magic_number;
      cctt_debug(stderr,"Ident : Keeping the '%d' magic number.",magic);
    }

  to_encode = (char *) malloc (strlen(options->ident_key)+nb_char_in_int(magic)+1);
  memset(to_encode,0,strlen(options->ident_key)+nb_char_in_int(magic)+1);
  strncpy(to_encode,options->ident_key,strlen(options->ident_key));
  strncpy(to_encode+strlen(options->ident_key),get_str_from_int(magic),nb_char_in_int(magic));

  tmp_encoding = basic_encode(to_encode,
			      be_get_key_length(options->ident_key),
			      CCTT_BASIC_ENCODING_ALPHABET,
			      strlen(to_encode));

  ident_string = (char *) malloc(strlen(tmp_encoding)+1+1);
  memset(ident_string,0,strlen(tmp_encoding)+1+1);
  ident_string[0] = cl_set_request(cont);
  strncpy(ident_string+1,tmp_encoding,strlen(tmp_encoding));

  if (*tmp_encoding)
    free (tmp_encoding);

  return (ident_string);
}

/*
** Fonction d'identification minimale : 
** Elle doit contenir un octet de demande de service (cl_set_request()) + la cle.
** Note : Le serveur fait un read de la meme taille de son cote (1 octet + la longueur de sa propre cle
**  => voir src/server/srv_ident_client.c
*/ 

int cl_identification_clear(S_OPTIONS *options,int cont)
{
  char *ident_string;
  int ret;

  ident_string = build_ident_string_clear(options,cont);
  cctt_verbose(stderr,"Client is sending IDENT_KEY in clear mode.\n");
  ret = do_client_has_something_to_encode(options,options->cl_options->sd,ident_string,strlen(ident_string));

  if (*ident_string)
    free(ident_string);
  return (ret);
}

/*
** Fonction d'identification avec codage succinct
** Identique sur le principe a la fonction cl_identification_clear
** voir basic_encoding.c
**
** !!! ATTENTION !!!
** La cle est d'abord codee et le flux peut ensuite etre code egalement selon le type canal demande.
*/

int cl_identification_basic(S_OPTIONS *options,int cont)
{
  char *ident_string;
  int ret;

  ident_string = build_ident_string_basic(options, cont);
  cctt_verbose(stderr,"%s","Client is sending IDENT_KEY in basic mode.\n");
  ret = do_client_has_something_to_encode(options,options->cl_options->sd,ident_string,strlen(ident_string));
  if (*ident_string)
    free(ident_string);
  G_basdec_alph_pos = 0;
  return (ret);
}

/*
** Fonction de recherche du pointeur sur fonction a executer
*/

void cl_identification(S_OPTIONS *options,int cont)
{
  int i=0,ret=0;
    
      /* On va chercher la fonction a executer */
      for (i=0;ident_type_array[i].ident_name != NULL;i++)
	if ((strcmp(options->identification,ident_type_array[i].ident_name) == 0))
      ret = ident_type_array[i].ident_type_client(options,cont);

  if (ret <= 0)
    {
      /* 
      ** N'a pas reussi a envoyer de donnees, et la fonction d'envoi n'a rien ferme...
      ** C'est ici qu'on ferme le programme...
      */
      cctt_error_and_exit(stderr,"%s","Couldn't send IDENT_STRING to server !!!");
    }
}
