/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: client_asks_reverse_prox_mode.c,v 1.2 2003/06/10 15:21:16 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Ouvre la socket vers le service demande
*/

int open_socket_to_reversed_appl(S_OPTIONS *options)
{
  int keepalive,sd;

  struct sockaddr_in sockaddr;
  struct hostent *he;
  
  if (OPT_CL_TCP()) /* TCP */
    {
      if ((sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
	  if (OPT_DEBUG2()) cctt_debug(stderr,"[RPMod] : Couldn' open socket!\n");
	  return (-1);
	}
      setsockopt(sd,SOL_SOCKET,SO_KEEPALIVE, (int *)&keepalive, sizeof(keepalive));
      if (!(he = (struct hostent *) gethostbyname(options->cl_options->dest_server_proxy_mode_address)))
	{
	  if (OPT_DEBUG2()) cctt_debug(stderr,"[RPMod] : Couldn' get hostent from %s!\n",options->cl_options->dest_server_proxy_mode_address);
	  return (-1);
	}
      sockaddr.sin_family = AF_INET;
      sockaddr.sin_addr = *((struct in_addr *)he->h_addr);
      bzero(&(sockaddr.sin_zero),sizeof(sockaddr.sin_zero));
      if (!(sockaddr.sin_port = htons(options->cl_options->dest_server_proxy_mode_port)))
	{
	  if (OPT_DEBUG2()) cctt_debug(stderr,"[RPMod] : Couldn' set sin_port from %d!\n",options->cl_options->dest_server_proxy_mode_port);
	  return (-1);
	}
      if (connect(sd,(struct sockaddr *)&sockaddr,sizeof(struct sockaddr)) == -1)
	{
	  if (OPT_DEBUG2()) cctt_debug(stderr,"[RPMod] : Couldn' connect!\n");
	  return (-1);
	}
      return(sd);
    }
  return (-1);
}

/*
** Ouvre une connexion vers le service demande
*/

int open_rev_mode_connection(S_OPTIONS *options,char *buffer,int nsocket,int srv_sd)
{
  int ctr,pos,pos2;
  char *label;
  int client,size,sd;
  char *datas;
  char *taille;
  int data_length;
  int ret;
  T_basic_decode basdec;

  if (nsocket <= 0)
    {
      cctt_debug(stderr,"%s","[IPMod] : Won't decode a 0 bytes buffer.");
      return (0);
    }

  /*
  ** On decode si besoin est
  */

  basdec = do_client_has_something_to_decode_locally(options,srv_sd,buffer,nsocket);
      buffer = (char *) basdec.decoded;
      nsocket = basdec.size;

  for (ctr=0,pos=0,pos2=0;pos2 == 0 && ctr < nsocket;ctr++)
    {
      if (buffer[ctr] == '/' && pos != 0)
	pos2 = ctr;
      if (buffer[ctr] == '/' && pos == 0)
	pos = ctr;
    }
  if (pos != 0 && pos2 != 0)
    {
      label = (char *) malloc(pos+1);
      memset(label,0,pos+1);
      strncpy(label,buffer,pos);
      client = atoi(label);

      taille = (char *) malloc(pos2+1-pos);
      memset(taille,0,pos2+1-pos);
      strncpy(taille,buffer+pos+1,pos2+1-pos);
      size = atoi(taille);

      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Will use %d as index and data size is %d",client,size);

      if (!(options->cl_options->client_sd_table[client].used & SRV_SD_USED))
	{
	  if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Will open the socket to use %d index",client);
	  /* Le client est libre */
	  options->cl_options->client_sd_table[client].used = SRV_SD_USED;
	  sd = open_socket_to_reversed_appl(options);
	  if (sd > 0)
	    {
	      /* La socket a pu etre ouverte */
	      options->cl_options->client_sd_table[client].sd = sd;
	      datas = buffer+pos2+1;
	      data_length = nsocket - pos2 -1;
	      if (data_length > 0)
		{
		  /* On a effectivement des donnees a envoyer au client */
		  ret = write(sd,datas,data_length);
		  if (ret < 1)
		    {
		      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Index %d is bad ... Reseting ...",client);
		      options->cl_options->client_sd_table[client].used = SRV_SD_FREE;
		      options->cl_options->client_sd_table[client].sd = -1;
		      return (-1);
		    }
		  else
		    {
		      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Wrote %d bytes to application",ret);
		    }
		}
	      else
		{
		  if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Opened %d sd to application but no datas...",sd);
		  return (0);
		}
	      return (ret);
	    }
	  else
	    options->cl_options->client_sd_table[client].used = SRV_SD_FREE;
	}
      else
	{
	  if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Index %d already opened ... Using it ...",client);
	  datas = buffer+pos2+1;
	  data_length = nsocket - pos2 -1;
	  ret = write(options->cl_options->client_sd_table[client].sd,datas,data_length);
	  if (ret < 1)
	    {
	      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Index %d is bad ... Reseting ...",client);
	      options->cl_options->client_sd_table[client].used = SRV_SD_FREE;
	      options->cl_options->client_sd_table[client].sd = -1;
	      return (-1);
	    }
	  else
	    {
	      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Wrote %d bytes to application",ret);
	    }
	  return (ret);
	}
    }
  return (-1);
}


/*
** La boucle principale pour le mode proxy inverse
*/

int client_loop_for_reverse_prox_mode(S_OPTIONS *options)
{
  int ret_sel; /* valeur de retour du select */
  struct timeval tv; /* Structure de temps pour le select */
  fd_set fdset_read;
  int highest_sd;
  char buffer[SRV_SOCKET_BUFFER_SIZE+1];
  int nsocket,nwrite;
  int max,cpt;

  max = getdtablesize();
  options->cl_options->client_sd_table = (T_client_sd_table *) malloc (sizeof(T_client_sd_table)* (max));
  for (cpt=0;cpt<max;cpt++)
    {
      options->cl_options->client_sd_table[cpt].used = SRV_SD_FREE;
      options->cl_options->client_sd_table[cpt].basdec_alph_pos = 0;
    }

  while (1)
    {
      FD_ZERO(&(fdset_read));
      FD_SET(options->cl_options->sd,&(fdset_read)); /* Socket en Listen local */
      highest_sd = options->cl_options->sd + 1;
      tv.tv_sec = 0;
      tv.tv_usec = 100;

      for (cpt=0;cpt<max;cpt++)
	if (options->cl_options->client_sd_table[cpt].used & SRV_SD_USED)
	  {		
	    FD_SET(options->cl_options->client_sd_table[cpt].sd,&(fdset_read));
	    highest_sd = options->cl_options->client_sd_table[cpt].sd + 1;
	  }
  
      if ((ret_sel = select(highest_sd,&(fdset_read),NULL,NULL,&tv)) < 0)
	perror ("select");
      else
	{
	  if (FD_ISSET(options->cl_options->sd,&(fdset_read)))
	    {
	      /* 
	      ** Des donnees sont en provenance du serveur
	      */
	      memset(buffer,0,SRV_SOCKET_BUFFER_SIZE+1);
	      nsocket = read(options->cl_options->sd,buffer,SRV_SOCKET_BUFFER_SIZE);
	      if (nsocket < 1)
		{
		  cctt_debug(stderr,"[IPMod] : Server closed the main socket !!!");
		  return (0);
		}
	      if (open_rev_mode_connection(options,buffer,nsocket,options->cl_options->sd) == -1)
		cctt_debug(stderr,"[IPMod] : Couldn't open connection to %s:%d !!!",options->cl_options->dest_server_proxy_mode_address,options->cl_options->dest_server_proxy_mode_port);
	    }
	  for (cpt=0;cpt<max;cpt++)
	    {
	      /*
	      ** Verifie si les clients parlent
	      */
	      if ((options->cl_options->client_sd_table[cpt].used & SRV_SD_USED) && FD_ISSET(options->cl_options->client_sd_table[cpt].sd,&(fdset_read)))
		{
		  memset(buffer,0,options->cl_options->srv_socket_buffer_size+1);

		  /*
		  ** Attention ici : la fonction qui envoie les donnees lues du client ajoute un overhead.
		  ** => On ne doit donc lire les donnees en provenance du client que pour une taille qui sera toujours inferieure au
		  ** buffer que nous allouons et qui est positionnee dans le serveur...
		  */
		  nsocket = read(options->cl_options->client_sd_table[cpt].sd,buffer,options->cl_options->srv_socket_buffer_size-(nb_char_in_int(sizeof(int)*256)*2+3));
		  if (nsocket > 0) /* On envoie au serveur via la connexion qui va bien */
		    {
		      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Readed %d bytes from application.",nsocket);
		      nwrite = add_rev_prox_mode_overhead_and_send(options,options->cl_options->sd,buffer,nsocket,cpt);
		      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Wrote %d bytes on the reverse connection.",nwrite);
		    }
		  else
		    {
		      cctt_verbose(stderr,"Client %d closed connection.\n",cpt);
		      options->cl_options->client_sd_table[cpt].used = SRV_SD_FREE;
		      options->cl_options->client_sd_table[cpt].sd = -1;
		    }
		}
	    }
	}
    }
}

/*
** Le client fonctionne en mode proxy
** Envoi de la demande au serveur
*/

int client_asks_reverse_prox_mode(S_OPTIONS *options)
{
  int nw;
  char *proxy_mode_list;
  int str_length;

  /*
  ** On envoie au serveur la chaine correspondant au serveur que l'on accepte de 'proxyer'.
  ** Elle est au meme format que la directive PROXY_MODE_LIST du fichier de configuration serveur
  ** moins le label qui sera positionner par le serveur.
  */
  
  str_length = strlen(options->cl_options->dest_server_proxy_mode_address)+
    nb_char_in_int(options->cl_options->dest_server_proxy_mode_port);
  proxy_mode_list = malloc(str_length+2);
  memset(proxy_mode_list,0,str_length+2);
  strncpy(proxy_mode_list,options->cl_options->dest_server_proxy_mode_address,
	  strlen(options->cl_options->dest_server_proxy_mode_address));
  proxy_mode_list[strlen(options->cl_options->dest_server_proxy_mode_address)] = ':';
  strncpy(proxy_mode_list+strlen(options->cl_options->dest_server_proxy_mode_address)+1,
	  get_str_from_int(options->cl_options->dest_server_proxy_mode_port),
	  nb_char_in_int(options->cl_options->dest_server_proxy_mode_port));

  cctt_debug(stderr,"[PMod] : Telling the server we can proxy to %s",proxy_mode_list);
  nw = do_client_has_something_to_encode(options,options->cl_options->sd,proxy_mode_list,strlen(proxy_mode_list));
  if (OPT_DEBUG2()) cctt_debug(stderr,"[PMod] : Sended a string of %d bytes",nw);
  client_loop_for_reverse_prox_mode(options);

  return (0);
}
