/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: client_binds_local_proxy.c,v 1.7 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Le client fonctionne en mode proxy
*/

int client_return_local_proxy_sd(S_OPTIONS *options)
{
  int local_sd=-1;
  int reuse=1; /* Reusaddr en TCP */

  if (OPT_CL_PMODE_TCP()) /* On veut une destination TCP => le proxy local est aussi en tcp */
    {
      if ((local_sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	cctt_error(stderr,"%s","[Client] : Client cannot create local socket in proxy mode!\n");
      if (local_sd)
	setsockopt(local_sd,SOL_SOCKET,SO_REUSEADDR,(int *)&reuse,sizeof(reuse));

      cctt_debug(stderr,"PMod : Client en proxy local pour le sd %d",local_sd);

      if (bind(local_sd,
	       (struct sockaddr *)&(options->cl_options->local_sockaddr),
	       sizeof(struct sockaddr))<0)
	{
	  cctt_error(stderr,"%s","[Client] : Client cannot bind local socket in proxy mode!\n");
	  local_sd = -1;
	}
      if ((listen(local_sd,SRV_LISTEN_NUM)) != 0)
	{
	  cctt_error(stderr,"%s","[Client] : Client cannot listen on local socket in proxy mode!\n");
	  local_sd = -1;
	}
    }
  return (local_sd);
}


int client_binds_local_proxy(S_OPTIONS *options)
{
  /* Parametrage socket */
  int local_sd;

  /* Parametrage select */
  int ret_sel; /* valeur de retour du select */
  struct timeval tv; /* Structure de temps pour le select */
  fd_set fdset_read;
  int highest_sd;
  int max,cpt;

  /* Parametrage socket des client */
  int new_sd; /* nouveau sd pour la socket */
  struct sockaddr_in sockaddr;
  int sock_size;

  /* Buffer entre les clients et le proxy local */
  char buffer[SRV_SOCKET_BUFFER_SIZE+1];
  int nsocket,nwrite;

  /* Celles-la ont ete lues (eventuellement) par la fonction de connexion au(x) proxy(s) */
  char *oob_datas=NULL;

  local_sd = client_return_local_proxy_sd(options);
  sock_size = sizeof(struct sockaddr_in);

  /*
  ** Notre client en mode proxy est pret...
  ** On boucle en attente de connexions
  */
  if (local_sd > 0)
    {
      /* Prepare les infos pour les futurs clients */
      max = getdtablesize();
      options->cl_options->client_sd_table = (T_client_sd_table *) malloc (sizeof(T_client_sd_table)* (max));
      options->cl_options->client_sd_table_max = max;
      for (cpt=0;cpt<max;cpt++)
	{
	  options->cl_options->client_sd_table[cpt].used = SRV_SD_FREE;
	  options->cl_options->client_sd_table[cpt].basdec_alph_pos = 0;
	}
      while (1)
	{
	  /* Initialisations pour le select */
	  FD_ZERO(&(fdset_read));
	  FD_SET(local_sd,&(fdset_read)); /* Socket en Listen local */
	  highest_sd = local_sd + 1;
	  for (cpt=0;cpt<max;cpt++)
	    if (options->cl_options->client_sd_table[cpt].used & SRV_SD_USED) /* Ajout d'un client connecte */
	      {		
		FD_SET(cpt,&(fdset_read));
		FD_SET(options->cl_options->client_sd_table[cpt].remote_sd,&(fdset_read));
		highest_sd = (cpt > options->cl_options->client_sd_table[cpt].remote_sd ? cpt + 1 : options->cl_options->client_sd_table[cpt].remote_sd + 1);
	      }
	  /* Structure de temps */
	  tv.tv_sec = 0;
	  tv.tv_usec = 100;
	  
	  if ((ret_sel = select(highest_sd,&(fdset_read),NULL,NULL,&tv)) < 0)
	    perror ("select");
	  else
	    {
	      if (FD_ISSET(local_sd,&(fdset_read)))
		{
		  /* Un client tente de se connecter a notre proxy local */
		  if ((new_sd = accept(local_sd,(struct sockaddr *)&sockaddr,&sock_size)) != -1)
		    {
		      options->cl_options->client_sd_table[new_sd].used = SRV_SD_USED;
		      options->cl_options->client_sd_table[new_sd].sd = new_sd;

		      /* Tente de se connecter au serveur */
		      client_socket(options);
		      if (OPT_CL_CONNECTED()) /* Si connecte, on stocke les infos */
			options->cl_options->client_sd_table[new_sd].remote_sd = options->cl_options->sd;
		      /* Si nous sommes en mode proxy */
		      if (OPT_CL_CONNECTED() && OPT_CL_CHAN_PROXY())
			oob_datas = client_use_border_proxy(options);
		      /* 
		      ** Check si le proxy a renvoye des donnees, auquel cas on les balance direct.
		      ** Par contre, s'il y a un \0 dans le buffer, ca plante...
		      */
		      if (oob_datas != NULL)
			{
			  nwrite = do_client_has_something_to_decode(options,new_sd,oob_datas,strlen(oob_datas));
			  if (OPT_DEBUG2()) cctt_debug(stderr,"PMod : Client en mode proxy a ecrit %d octets au client local",nwrite);
			  oob_datas = NULL;
			}

		      /* Si on est pas en mode client_only */
		      if ((strlen(options->channel) != strlen("client_only_with_http_proxy")) || 
			  (strcmp(options->channel,"client_only_with_http_proxy") != 0))
			{
			  if (OPT_CL_CONNECTED()) /* Si connecte, on s'identifie */
			    cl_identification(options,0);
			  if (OPT_CL_CONNECTED()) /* Si tjrs connecte, on envoie la requete */
			    {
			      options->cl_options->client_sd_table[new_sd].magic_number = options->cl_options->magic_number;
			    send_cl_proxying_request(options, sockaddr);
			}
			}
		      if (!OPT_CL_CONNECTED()) /* Ca a plante qqpart, on annule */
			{
			  options->cl_options->client_sd_table[new_sd].used = SRV_SD_FREE;
			  options->cl_options->client_sd_table[new_sd].sd = -1;
			  close (new_sd);
			}
		    }
		}
	      for (cpt=0;cpt<max;cpt++)
		{
		  /* 
		  ** Check si les clients parlent 
		  ** et si oui, envoie sur la socket
		  */
		  if (options->cl_options->client_sd_table[cpt].used & SRV_SD_USED && FD_ISSET(cpt,&(fdset_read)))
		    {
		      /*
		      ** En mode HTTP, le client envoie une requete puis attend la reponse du serveur
		      ** avant d'envoyer a nouveau...
		      */
		      if (tell_me_about_encoding(options) == 2)
			{
			  if (options->cl_options->client_sd_table[cpt].http_turn == 0)
			    {
			      memset(buffer,0,options->cl_options->srv_socket_buffer_size+1);
			      nsocket = read(cpt,buffer,options->cl_options->srv_socket_buffer_size);
			      if (OPT_DEBUG2()) cctt_debug(stderr,"PMod : Client en mode proxy a lu %d octets du client local",nsocket);
			      if (nsocket > 0) /* On envoie au serveur via la connexion qui va bien */
				{
				  nwrite = do_client_has_something_to_encode(options,options->cl_options->client_sd_table[cpt].remote_sd,buffer,nsocket);
				  if (OPT_DEBUG2()) cctt_debug(stderr,"PMod : Client en mode proxy a ecrit %d octets au serveur",nwrite);
				  options->cl_options->client_sd_table[cpt].http_turn = 100;
				}
			      else
				{
				  cctt_verbose(stderr,"Client %d closed connection.\n",cpt);
				  options->cl_options->client_sd_table[cpt].used = SRV_SD_FREE;
				  options->cl_options->client_sd_table[cpt].sd = -1;
				  shutdown (options->cl_options->client_sd_table[cpt].remote_sd,2);
				  options->cl_options->client_sd_table[cpt].remote_sd = -1;
				  shutdown (cpt,2);
				}
			    }
			}
		      else
			{
			  memset(buffer,0,options->cl_options->srv_socket_buffer_size+1);
			  nsocket = read(cpt,buffer,options->cl_options->srv_socket_buffer_size);
		      if (OPT_DEBUG2()) cctt_debug(stderr,"PMod : Client en mode proxy a lu %d octets du client local",nsocket);
		      if (nsocket > 0) /* On envoie au serveur via la connexion qui va bien */
			{
			  nwrite = do_client_has_something_to_encode(options,options->cl_options->client_sd_table[cpt].remote_sd,buffer,nsocket);
			  if (OPT_DEBUG2()) cctt_debug(stderr,"PMod : Client en mode proxy a ecrit %d octets au serveur",nwrite);
			}
		      else
			{
			  cctt_verbose(stderr,"Client %d closed connection.\n",cpt);
			  options->cl_options->client_sd_table[cpt].used = SRV_SD_FREE;
			  options->cl_options->client_sd_table[cpt].sd = -1;
			  shutdown (options->cl_options->client_sd_table[cpt].remote_sd,2);
			  options->cl_options->client_sd_table[cpt].remote_sd = -1;
			  shutdown (cpt,2);
			}
		    }
		    }
		  /* Check si le serveur renvoie des donnees a l'un des clients */
		  if (options->cl_options->client_sd_table[cpt].used & SRV_SD_USED && FD_ISSET(options->cl_options->client_sd_table[cpt].remote_sd,&(fdset_read)))
		    {
		      memset(buffer,0,SRV_SOCKET_BUFFER_SIZE+1);
		      nsocket = read(options->cl_options->client_sd_table[cpt].remote_sd,buffer,SRV_SOCKET_BUFFER_SIZE);
		      if (OPT_DEBUG2()) cctt_debug(stderr,"PMod : Client en mode proxy a lu %d octets du serveur",nsocket);
		      if (nsocket > 0)
			{
			  nwrite = do_client_has_something_to_decode(options,cpt,buffer,nsocket);
			  /* On specifie que le serveur vient de repondre pour un mode HTTP */
			  options->cl_options->client_sd_table[cpt].http_turn = 0;
			  if (OPT_DEBUG2()) cctt_debug(stderr,"PMod : Client en mode proxy a ecrit %d octets au client local",nwrite);
			}
		      else
			{
			  cctt_verbose(stderr,"Server %d closed connection for client %d.\n",options->cl_options->client_sd_table[cpt].remote_sd,cpt);
			  options->cl_options->client_sd_table[cpt].used = SRV_SD_FREE;
			  options->cl_options->client_sd_table[cpt].sd = -1;
			  shutdown (options->cl_options->client_sd_table[cpt].remote_sd,2);
			  options->cl_options->client_sd_table[cpt].remote_sd = -1;
			  shutdown (cpt,2);
			}
		    }
		}
	    }
	  /*
	  ** Ceci est specifique au mode HTTP
	  ** Voir les explications dans channel_do_socket_go.c
	  ** La meme chose sauf qu'on se sert de la variable http_turn de chaque client
	  ** comme compteur
	  */
	  if (tell_me_about_encoding(options) == 2)
	    {
	      for (cpt=0;cpt<max;cpt++)
		if (options->cl_options->client_sd_table[cpt].used & SRV_SD_USED &&
		    options->cl_options->client_sd_table[cpt].http_turn != 0)
		  {
		    options->cl_options->client_sd_table[cpt].http_turn += 100;
		    if (options->cl_options->client_sd_table[cpt].http_turn >
			options->cl_options->http_mode_del_bet_con * 2)
		      options->cl_options->client_sd_table[cpt].http_turn = 0;
		  }
	    }
	}
    }
  
  /* On ferme la socket locale */
  if (local_sd > 0)
    close (local_sd);
  return (0);
}



