/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: client_tells_hes_proxying_to.c,v 1.2 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Supprime un element reverse proxy_mode a partir du numero de client
*/

void remove_reversed_proxy_elem(S_OPTIONS *options,int cl_num)
{
  T_proxy_allowed_list *tmp_list,*tmp_list2;
  T_proxy_allowed_list *tmp_list_p=NULL;
  T_proxy_allowed_list *collage;

  /* On se positionne sur le premier */
  tmp_list = options->srv_options->proxy_allowed_list;
  if (tmp_list)
    {
      tmp_list = tmp_list->first;
      tmp_list2 = tmp_list->first;
    }

  while (tmp_list && tmp_list->next != NULL)
    {
      if (tmp_list->reverse == cl_num)
	{
	  if (tmp_list->first == tmp_list)
	    {
	      /* Premier et premier et le next n'est pas nul */
	      collage = tmp_list->next;
	      collage->first = collage;
	      options->srv_options->proxy_allowed_list = collage;
	      free (tmp_list);
	      tmp_list = collage;
	      options->srv_options->proxy_allowed_list_elements_number -= 1;
	    }
	  else
	    {
	      /* Au milieu et le next n'est pas nul */
	      collage = tmp_list_p;
	      collage->next = tmp_list->next;
	      free (tmp_list);
	      tmp_list = collage;
	      options->srv_options->proxy_allowed_list_elements_number -= 1;
	    }
	}
      tmp_list_p = tmp_list;
      tmp_list = tmp_list->next;
    }

  /* Le dernier */
  if (tmp_list)
    {
      if (tmp_list->reverse == cl_num)
	{
	  if (!tmp_list_p)
	    {
	      /* Le dernier et le premier */
	      free (tmp_list);
	      options->srv_options->proxy_allowed_list = NULL;
	      options->srv_options->proxy_allowed_list_elements_number -= 1;
	    }
	  else
	    {
	      /* Le dernier mais pas le premier */
	      free (tmp_list);
	      while (tmp_list2->next && tmp_list2 != tmp_list_p)
		tmp_list2 = tmp_list2->next;
	      if (tmp_list2->next && tmp_list2 != tmp_list_p)
		tmp_list2 = tmp_list2->next;
	      tmp_list2->next = NULL;
	      options->srv_options->proxy_allowed_list_elements_number -= 1;
	    }
	}
    }
}

/*
** Verifie que la chaine de reverse_proxy que le client nous envoie va pouvoir etre ajoutee
*/

T_proxy_allowed_list *check_client_reverse_proxying(S_OPTIONS *options,char *proxy_mode_list,int cl_num)
{
  char *mode_name,*mode_ip,*mode_port;
  T_proxy_allowed_list *tmp_list;

  /* Verification de la syntaxe */
  if (!(mode_name = strsep(&proxy_mode_list,":")) || strlen(mode_name) == 0)
    return (NULL);
  if (!(mode_ip = strsep(&proxy_mode_list,":")) || strlen(mode_ip) == 0)
    return (NULL);
  if (!(mode_port = strsep(&proxy_mode_list,":")) || strlen(mode_port) == 0)
    return (NULL);

  /* 
  ** Verification de la pertinence des informations entrees 
  ** et sauvegarde des informations dans la structure 
  */

  /* On cree un nouvel element dans la structure temporaire */
  tmp_list = (T_proxy_allowed_list *) malloc (sizeof(T_proxy_allowed_list));
  memset(tmp_list,0,sizeof(T_proxy_allowed_list));

  /* On positionne le label */
  tmp_list->label = (char *) malloc (strlen(mode_name)+1);
  memset(tmp_list->label,0,strlen(mode_name)+1);
  strncpy(tmp_list->label,mode_name,strlen(mode_name));

  /* On verifie l'adresse IP et on l'enregistre */
  if (!(tmp_list->he =  (struct hostent *) gethostbyname(mode_ip)))
    return (NULL);
  tmp_list->sockaddr.sin_family = AF_INET;
  tmp_list->sockaddr.sin_addr = *((struct in_addr *)tmp_list->he->h_addr);
  bzero(&(tmp_list->sockaddr.sin_zero), sizeof(tmp_list->sockaddr.sin_zero));

  /* On verifie le port et on l'enregistre */
  if (!(tmp_list->sockaddr.sin_port = htons(atoi(mode_port))))
    return (NULL);

  /* 
  ** L'element est relatif a un proxying inverse => On stocke le socket descriptor
  */
  tmp_list->reverse=cl_num;

  return(tmp_list);
}

/*
** Le client nous indique qu'il peut proxyier vers cette adresse.
*/

int client_tells_hes_proxying_to(S_OPTIONS *options, int cl_num, char *buffer, int nwr)
{
  T_basic_decode basdec;
  char *proxy_mode_list=NULL,*tmp_proxy_mode_list;
  struct sockaddr_in sockaddr;
  int received_length=0,tmp_length=0;

  T_proxy_allowed_list *tmp_list;

  /*
  ** Recupere la chaine proxy_mode_list du client
  */
  basdec = do_server_has_something_to_decode_locally(options,cl_num,buffer,nwr);
      proxy_mode_list = basdec.decoded;
      received_length = basdec.size;

  /*
  ** Formate cette chaine en ajoutant un label (@IP-port) relatif a l'extremite distante.
  */

  sockaddr = options->srv_options->connect_infos->sd_table[cl_num].sockaddr;
  tmp_length = strlen(inet_ntoa(sockaddr.sin_addr))+nb_char_in_int(ntohs(sockaddr.sin_port))+2+received_length+1;

  tmp_proxy_mode_list = malloc(tmp_length+1);
  memset(tmp_proxy_mode_list,0,tmp_length+1);
  snprintf(tmp_proxy_mode_list,tmp_length,"%s-%d:%s",
	   inet_ntoa(sockaddr.sin_addr),
	   ntohs(sockaddr.sin_port),
	   proxy_mode_list);

  cctt_verbose(stderr,"Server is checking if it can add '%s' client %d proxying request.\n",tmp_proxy_mode_list,cl_num);
  if ((tmp_list = check_client_reverse_proxying(options,tmp_proxy_mode_list,cl_num)) != NULL)
    {
      add_auth_to_chained_list(options,tmp_list);
      cctt_verbose(stderr,"Server added '%s' in the proxy mode list.\n",tmp_proxy_mode_list);
      free (tmp_proxy_mode_list);
      return (1);
    }
  else
    {
      cctt_verbose(stderr,"Couldn't add '%s' in the proxy mode list.\n",tmp_proxy_mode_list);
      free (tmp_proxy_mode_list);
      return(0);
    }
  return (0);
}

