/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: client_told.c,v 1.7 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Ces quelques fonctions permettent au serveur d'envoyer des donnees aux clients
** ayant binde un reverse shell
*/

void client_told(S_OPTIONS *options, int cl_num, char *buffer)
{
  T_basic_decode basdec;

  basdec = do_server_has_something_to_decode_locally(options,cl_num,buffer,strlen(buffer));
      log_reverse_session(cl_num,basdec.decoded,1);
      fprintf(stdout,"Reverse-Shell / Client %d told :\n%s\n",cl_num,basdec.decoded);
}

/*
** Appele par srv_manage_connection_type pour les clients en mode proxy inverse deja identifies.
*/

char *recursive_remove_overhead(S_OPTIONS *options,int cl_num,char *decoded,int decoded_length)
{
  int ctr,pos,pos2,client,size,nwrite;
  char *new,*label,*taille;
  int reinject_size;
  int cpt,fst_sd=0;

  /*
  ** Supprime le premier overhead
  */

  if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Recursive remove overhead for %d on a %d bytes buffer.",cl_num,decoded_length);

  for (ctr=0,pos2=0,pos=0;pos2 == 0 && ctr < decoded_length;ctr++)
    {
      if (decoded[ctr] == '/' && pos != 0)
	pos2 = ctr;
      if (decoded[ctr] == '/' && pos == 0)
	pos = ctr;
    }

  if (pos != 0 && pos2 != 0)
    {
      label = (char *) malloc(pos+1);
      memset(label,0,pos+1);
      memcpy(label,decoded,pos);
      client = atoi(label);

      taille = (char *) malloc(pos2+1-pos+1);
      memset(taille,0,pos2+1-pos+1);
      memcpy(taille,decoded+pos+1,pos2+1-pos);
      size = atoi(taille);

      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Recursive remove overhead found a %d bytes buffer for %d client.",size,client);

      /* 
      ** Va chercher le bon numero de client : Si la connexion a ete coupee (http_post)
      ** on a ici un index qui correspond au premier client et qui n'est plus egal
      ** a l'actuel sd
      */
      for (cpt=0;fst_sd == 0 && cpt<options->srv_options->connect_infos->sdtablesize;cpt++)
	if (options->srv_options->connect_infos->sd_table[cpt].rev_prox_mode_sd == client)
	  fst_sd = cpt;
      if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : %d bytes Overhead for %d is for %d.",size,client,fst_sd);
      if (fst_sd != 0)
	client = fst_sd;

      if (size <= decoded_length-pos2-1)
	{
	  /*
	  ** On a plusieurs overhead dans ce buffer ... On la joue en recursif ?
	  */
	  new = (char *) malloc (size);
	  memcpy(new,decoded+pos2+1,size);
	  nwrite = do_server_has_something_to_encode(options,client,new,size);
	  if (OPT_DEBUG2()) cctt_debug(stderr,"Srv  : Ecriture de %d octets au client %d",nwrite,client);
	  return(recursive_remove_overhead(options,cl_num,decoded+size+pos2+1,decoded_length-(size+pos2+1)));
	}
      else
	{
	  /*
	  ** L'overhead indique qu'il reste size, mais on n'a pas la totalite dans ce buffer.
	  ** On envoie donc ce qu'on a et on previent la fonction do_server que des donnees sont manquantes
	  ** en creant un buffer que la fonction do_server replacera dans le buffer temporaire.
	  ** Ce buffer est bien entendu un nouvel overhead corrige avec la taille des donnees manquantes.
	  */
	  new = decoded+pos2+1;
	  nwrite = do_server_has_something_to_encode(options,client,new,decoded_length-pos2-1);
	  if (OPT_DEBUG2()) cctt_debug(stderr,"Srv  : Ecriture de %d octets au client %d",nwrite,client);
	  
	  /* On cree le nouvel overhead */
	  reinject_size = nb_char_in_int(client) + nb_char_in_int(size-(decoded_length-pos2-1));
	  options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr = (char *) malloc (reinject_size+3);
	  memset(options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr,0,reinject_size+3);
	  memcpy(options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr,
		  get_str_from_int(client),
		  nb_char_in_int(client));
	  options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr[nb_char_in_int(client)] = '/';
	  memcpy(options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr+nb_char_in_int(client)+1,
		  get_str_from_int(size-(decoded_length-pos2-1)),
		  nb_char_in_int(size-(decoded_length-pos2-1)));
	  options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr[nb_char_in_int(client)+nb_char_in_int(size-(decoded_length-pos2-1))+1] = '/';
	  options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr[nb_char_in_int(client)+nb_char_in_int(size-(decoded_length-pos2-1))+2] = '\0';
	  options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_size = strlen(options->srv_options->connect_infos->sd_table[cl_num].tmpbuf_ptr);

	  if (OPT_DEBUG2()) cctt_debug(stderr,"Srv  : Reinjection overhead dans le buffer client %d.",cl_num);
	}
    }
  else
    {
      /*
      ** Ca doit planter si on arrive la ... Mais comme je ne sais pas comment faire pour arriver ici ...
      */
      if (OPT_DEBUG2()) cctt_debug(stderr,"%s","[IPMod] : Recursive remove failed in recursive_remove_overhead().");
    }
  return (NULL);
}

int remove_overhead_from_rev_prox_mode(S_OPTIONS *options,int cl_num,char *buffer,int nwr)
{
  T_basic_decode basdec;
  char *decoded=NULL;
  int decoded_length=0,nwrite=0;

  /*
  ** On decode d'abord le buffer si jamais besoin est ...
  ** => on enleve le codage basic, les eventuels headers HTTP, etc...
  */

  basdec = do_server_has_something_to_decode_locally(options,cl_num,buffer,nwr);
      decoded = (char *) basdec.decoded;
      decoded_length = basdec.size;

  /*
  ** Supprime l'overhead et envoie ... Attention, la fonction est recursive.
  */
  recursive_remove_overhead(options,cl_num,decoded,decoded_length);
  if (OPT_DEBUG2()) cctt_debug(stderr,"[IPMod] : Recursive remove overhead for %d is done.",cl_num);
  return (nwrite);
}

/*
** Appelle par la fonction suivante...
*/

int add_rev_prox_mode_overhead_and_send(S_OPTIONS *options,int sd,char *buffer,int nwr,int cl_num)
{
  int nwrite=0,done=0;
  T_basic_decode basdec;
  char *return_to_signed;
  char *overhead;
  char *this_is_to_send;
  int this_is_length;
  int overhead_size;

  /* On commence a creer l'overhead en ajoutant le numero de client */
  overhead_size = nb_char_in_int(cl_num) + nb_char_in_int(nwr);
  overhead = (char *) malloc (overhead_size+1);
  memset(overhead,0,overhead_size+1);
  memcpy(overhead,get_str_from_int(cl_num),nb_char_in_int(cl_num));
  overhead[nb_char_in_int(cl_num)] = '/';

  if (buffer && tell_me_about_encoding(options) > 1)
    {
      if (OPT_CL_OR_SRV())
	{
	  /* Si on est en mode client, les donnees proviennent de l'application */
	  return_to_signed = buffer;
	  basdec.size = nwr;
	}
      else
	{
	  /* Si on est en mode serveur, les donnees proviennent d'un client cctt */
	  basdec = do_server_has_something_to_decode_locally(options,sd,buffer,nwr);
	  return_to_signed = (char *) basdec.decoded;
	}

      /* On finit d'ajouter l'overhead avec la taille des donnees */
      memcpy(overhead+nb_char_in_int(cl_num)+1,get_str_from_int(basdec.size),nb_char_in_int(basdec.size));
      overhead[nb_char_in_int(cl_num)+nb_char_in_int(basdec.size)+1] = '/';
      overhead[nb_char_in_int(cl_num)+nb_char_in_int(basdec.size)+2] = '\0';

      /* On a decode, on ajoute l'overhead */
      if (overhead)
	{
	  this_is_length = strlen(overhead)+basdec.size;
	  this_is_to_send = (char *) malloc (this_is_length+1);
	  memset(this_is_to_send,0,this_is_length+1);
	  memcpy(this_is_to_send,overhead,strlen(overhead));
	  memcpy(this_is_to_send+strlen(overhead),return_to_signed,basdec.size);
	  if (OPT_CL_OR_SRV())
	    {
	      /* On est appelle par un client en mode proxy inverse */
	      nwrite = do_client_has_something_to_encode(options,sd,this_is_to_send,this_is_length);
	    }
	  else
	    {
	      /* On est appele en mode serveur */
	      nwrite = do_server_has_something_to_encode(options,sd,this_is_to_send,this_is_length);
	    }
	}
      else
	cctt_debug(stderr,"[Tmp] : Erreur dans client_told.c\n");

      done = 1;
    }

  if (done == 0)
    {
      /* On finit d'ajouter l'overhead avec la taille des donnees */
      memcpy(overhead+nb_char_in_int(cl_num)+1,get_str_from_int(nwr),nb_char_in_int(nwr));
      overhead[nb_char_in_int(cl_num)+nb_char_in_int(nwr)+1] = '/';
      overhead[nb_char_in_int(cl_num)+nb_char_in_int(nwr)+2] = '\0';

      /* On ajoute l'overhead */
      if (overhead)
	{
	  this_is_length = strlen(overhead)+nwr;
	  this_is_to_send = (char *) malloc (this_is_length);
	  memcpy(this_is_to_send,overhead,strlen(overhead));
	  memcpy(this_is_to_send+strlen(overhead),buffer,nwr);
	  nwrite = write(sd,this_is_to_send,this_is_length);
	}
      else
	cctt_debug(stderr,"[Tmp] : Erreur dans client_told.c\n");
    }

  return (nwrite);
}

void client_spoke_to_proxy(S_OPTIONS *options, int cl_num, char *buffer,int nwr)
{
  int nw;

  if (options->srv_options->connect_infos->sd_table[cl_num].rev_prox_mode == 0)
    {
  /* 
      ** Client Applicatif -1-> Client CCTT -2-> Serveur CCTT -3-> Serveur Applicatif
      **                            <----- Codage ----->        
      **
      ** On est en mode proxy normal et le client a demande a parler a une application precise...
      ** L'encodage des donnees s'arrete donc ici et nous envoyons les donnees decodees a l'application.
  */
  nw = do_server_has_something_to_decode(options,options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd,buffer,nwr);
      if (OPT_DEBUG2()) cctt_debug(stderr,"Srv  : Ecriture de %d octets sur l'application sd %d",nw,options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd);
    }
  else
    {
      /*
      ** Ici, on est en mode proxy inverse :
      ** Client Applicatif -1-> Client CCTT -2-> Serveur CCTT <-1- Client CCTT Inverse -3-> Serveur Applicatif
      **                             <--------------- Codage -------------------->
      **
      ** On doit donc decoder, ajouter l'overhead et renvoyer en reencodant.
      */
      nw = add_rev_prox_mode_overhead_and_send(options,options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd,buffer,nwr,options->srv_options->connect_infos->sd_table[cl_num].rev_prox_mode_sd);
      if (OPT_DEBUG2()) cctt_debug(stderr,"Srv  : Ecriture de %d octets vers la connexion en rev_prox_mode %d",nw,options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd);
    }
}


