/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: client_use_border_proxy.c,v 1.7 2003/06/11 13:14:19 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Ces fonctions gere l'utilisation de proxy intermediaire
** entre notre client principal et le serveur distant.
*/

/* Fonction de gestion d'un proxy HTTP supportant la methode CONNECT */

#define CONNECT_METHOD "CONNECT : HTTP/1.0\n\n"
#define CONNECT_RESP_SIZE 128 /* Arbitraire */
#define CONNECT_RESP "HTTP/1.0 200 Connection established"

char *client_use_border_connect_http_proxy(S_OPTIONS *options, char *proxyname, char *dest_addr, int dest_port, int delay)
{
  char *connect_string,*prox_resp;
  int buf_size,nr,ptr,ok,wait_ms;
  char resp_buf[CONNECT_RESP_SIZE*4];
  struct timeval tv;
  fd_set prx_rd;
  char *datas_are=NULL;

  /* On prepare la requete CONNECT */
  buf_size = strlen(CONNECT_METHOD)+strlen(dest_addr)+5;
  connect_string = (char *) malloc(buf_size+1);
  memset(connect_string,0,buf_size+1);
  snprintf(connect_string,buf_size,"CONNECT %s:%d HTTP/1.0\n\n",
	   dest_addr,dest_port);

  /* On envoie la requete */
  cctt_verbose(stderr,"Client sends the Connect string to proxy and waits %d msecs...\n",delay);
  ok = 0;
  if (write(options->cl_options->sd,connect_string,strlen(connect_string)) != strlen(connect_string))
    ok = -1;

  cctt_debug(stderr,"Cl : Client connection to proxy '%s'",proxyname);
  cctt_debug(stderr,"Cl : Client sended '%s'",connect_string);

  /* On prepare les buffers de reception */
  prox_resp = (char *) malloc(CONNECT_RESP_SIZE);
  memset(prox_resp,0,CONNECT_RESP_SIZE);
  memset(resp_buf,0,CONNECT_RESP_SIZE*4);

  /* 
  ** Boucle de lecture des informations renvoyees par le proxy :
  **   On recoit une chaine du proxy :
  **     * Si on a la place de la mettre dans le buffer temporaire, on ajoute, sinon on quitte.
  **   On incremente un compteur de temps :
  **     * Si cet increment est inferieur a la valeur du delai defini, on reboucle
  **       sinon on quitte.
  */

  for (nr=0,ptr=0,wait_ms=0 ; ok==0 && wait_ms < delay ; nr=0)
    {
      /* On initialise notre compteur de temps et le select */
      tv.tv_usec = 100;
      tv.tv_sec = 0;
      FD_ZERO(&(prx_rd));
      FD_SET(options->cl_options->sd,&(prx_rd));

      if (select(options->cl_options->sd+1,&(prx_rd),NULL,NULL,&tv) > 0)
	{
	  nr = read(options->cl_options->sd,prox_resp,CONNECT_RESP_SIZE-1);
	  if (OPT_DEBUG2()) cctt_debug(stderr,"Cl : Read %d bytes : '%s'\n",nr,prox_resp);
	  if ((nr < 1) || (ptr + nr > (CONNECT_RESP_SIZE*4)-1))
	    ok = -2;
	  else
	    {
	      strncpy(resp_buf+ptr,prox_resp,nr);
	      ptr += nr;
	      memset(prox_resp,0,CONNECT_RESP_SIZE);
	    }
	}
      wait_ms += 100;
    }


  /*
  ** Verification de l'etat des I/O
  */

  if (ok == -1 || ok == -2) /* Probleme lors du write OU probleme lors du read */
    {
      if (resp_buf) cctt_debug(stderr,"Cl : Proxy response was %s",resp_buf);
      cctt_error(stderr,"[Client] : Proxy %s closed the socket !\n",proxyname);
      /* Le retour a la fonction appellante effectue un check de la socket */
      client_socket_close(options);
    }

  /*
  ** Verification de la chaine envoyee par le proxy
  */

  if (ok >= 0 && memchr(resp_buf,'\n',ptr) != NULL)
    {
      /* 
      ** On au au moins un '\n' dans le buffer, donc le proxy a du nous envoye une reponse...
      ** On verifie que la connexion a eu lieu en effectuant une comparaison de la premiere ligne
      */
      if (strncmp(CONNECT_RESP,resp_buf,strlen(CONNECT_RESP)) != 0)
	{
	  /* Le proxy a refuse notre requete */
	  cctt_error(stderr,"[Client] : Proxy %s refused the connect string and closed the socket !\n",proxyname);
	  client_socket_close(options);
	}
      else
	{
	  cctt_verbose(stderr,"Border proxy %s accepted the Connect string...\n",proxyname);
	  /* Il s'agit maintenant de recuperer les donnees s'il y en a et de les renvoyer */
	  if ((datas_are = strstr(resp_buf,"\r\n\r\n")) != NULL)
	    if (datas_are+4 == NULL)
	      datas_are = NULL;
	}
    }
  else
    {
      if (ptr == 0)
	cctt_error(stderr,"[Client] : Proxy %s did not send anything !\n",proxyname);
      else
	cctt_error(stderr,"[Client] : Proxy %s sended something crazy ?!?!?\n",proxyname);
      /* Le retour a la fonction appellante effectue un check de la socket */
      client_socket_close(options);
    }
  free (connect_string);
  free (prox_resp);
  if (datas_are != NULL)
    return (datas_are+4);
  return (NULL);
}

/*
** Permet d'envoyer vers le bon type de support proxy
** Voir channels.c
*/

char *client_use_border_proxy(S_OPTIONS *options)
{
  int i=0;
  char *proxy_sended_datas=NULL;

  if (OPT_CL_CHAN_PROXY())
    {
      if (options->cl_options->http_proxy_chain_addrs[i] != NULL)
	{
	  /* Si on est en mode HttpProxyChain */
	  while (options->cl_options->http_proxy_chain_addrs[i] != NULL && OPT_CL_CONNECTED())
	    {
	      /* Si i == 0, on utilise les 'coordonnees' du proxy principal avant d'utiliser celles de la chaine */
	      if (OPT_CL_CONNECTED())
		client_use_border_connect_http_proxy(options,
						     (i==0?
						      inet_ntoa(options->cl_options->sockaddr.sin_addr):
						      options->cl_options->http_proxy_chain_addrs[i-1]),
						     options->cl_options->http_proxy_chain_addrs[i],
						     atoi(options->cl_options->http_proxy_chain_ports[i]),
						     (i==0?
						      options->cl_options->proxy_delay:
						      options->cl_options->http_proxy_chain_delays[i]));
	      i++;
	    }
	  if (OPT_CL_CONNECTED())
	    proxy_sended_datas = client_use_border_connect_http_proxy(options,
						 options->cl_options->http_proxy_chain_addrs[i-1],
						 options->cl_options->dest_server_address,
						 options->cl_options->dest_server_port,
						 options->cl_options->http_proxy_chain_delays[i-1]);
	}
      else
	{
	  /* Sinon, on est en mode normal */
	  if (tell_me_about_http(options) != 2)
	  proxy_sended_datas = client_use_border_connect_http_proxy(options,
					       inet_ntoa(options->cl_options->sockaddr.sin_addr),
					       options->cl_options->dest_server_address,
					       options->cl_options->dest_server_port,
					       options->cl_options->proxy_delay);
	  else
	    return (NULL);
	}
    }
  else
    { 
      cctt_debug(stderr,"%s","[TMP] : Erreur client_use_border_proxy.c\n");
      exit (-1); 
    }

  if (proxy_sended_datas != NULL)
    return (proxy_sended_datas);
  return (NULL);
}







