/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: client_wants_proxying_to.c,v 1.6 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

int open_local_connection_for_proxy(S_OPTIONS *options, int cl_num, struct sockaddr_in dest_sockaddr)
{
  if ((options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
    {
      cctt_debug(stderr,"PMod : Serveur : Probleme de socket locale (client %d)",cl_num);
      options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd = 0;
      return (0);
    }
  if (connect(options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd,
	      (struct sockaddr *)&dest_sockaddr,
	      sizeof(struct sockaddr)) == -1)
    {
      cctt_debug(stderr,"PMod : Serveur : Probleme de socket locale (client %d)",cl_num);
      options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd = 0;
      return (0);
    }

  cctt_debug(stderr,"PMod : Serveur : Ouverture de connexion local sur le sd %d",options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd);

  return (1);
}

/* 
** Le client a demande a fonctionner en mode proxy... on verifie sa demande.
** D'autres informations presentes dans src/channels/send_cl_proxying_request.c 
*/

int client_wants_proxying_to(S_OPTIONS *options, int cl_num, char *buffer, int nwr)
{
  T_cl_proxying_request struct_received;
  T_proxy_allowed_list *tmp_list;
  int auth=0;
  T_basic_decode basdec;

  /* On prepare la structure */
  memset(&struct_received,0,sizeof(T_cl_proxying_request));

  /*
  ** On decode d'abord le buffer si jamais besoin est ...
  ** => on enleve le codage basic, les eventuels headers HTTP, etc...
  */
  basdec = do_server_has_something_to_decode_locally(options,cl_num,buffer,nwr);
  if (!(basdec.decoded) || basdec.size <= 0)
    return (0);

      struct_received.protocol = *basdec.decoded;
      memcpy(&struct_received.sockaddr,basdec.decoded+1,sizeof(struct sockaddr_in));
      memcpy(&struct_received.cl_prox_sockaddr,basdec.decoded+2+sizeof(struct sockaddr_in),sizeof(struct sockaddr_in));

  cctt_debug(stderr,"PMod : Serveur : La requete proxy du client contient %s",inet_ntoa(struct_received.sockaddr.sin_addr));
  cctt_debug(stderr,"PMod : Serveur : La requete proxy du client contient %d",ntohs(struct_received.sockaddr.sin_port));
  cctt_verbose(stderr,"Server is checking client %d proxying request.\n",cl_num);

  if (options->srv_options->proxy_allowed_list_elements_number > 0)
    {
      tmp_list = options->srv_options->proxy_allowed_list->first;
      while (tmp_list->next != NULL)
	{
	  if (memcmp(&(tmp_list->sockaddr.sin_addr),&(struct_received.sockaddr.sin_addr),sizeof(tmp_list->sockaddr.sin_addr)) == 0 &&
	      memcmp(&(tmp_list->sockaddr.sin_port),&(struct_received.sockaddr.sin_port),sizeof(tmp_list->sockaddr.sin_port)) == 0)
	    {
	      cctt_verbose(stderr,"Client %d proxying request was configured.\n",cl_num);

	      /* 
	      ** On met en place la connexion demandee par le client
	      */
	      if (tmp_list->reverse == 0) /* Proxy normal */
		{
	      if (open_local_connection_for_proxy(options, cl_num, tmp_list->sockaddr) == 0)
		return (0);
	      options->srv_options->connect_infos->sd_table[cl_num].proxy_auth = SRV_CL_PROX_YES;
		}
	      else /* Proxy reverse */
		{ 
		  /* N'y va pas si le close flag est positionne ! */
		  if (options->srv_options->connect_infos->sd_table[tmp_list->reverse].close_flag == 1)
		    {
		      cctt_verbose(stderr,"Client %d proxying request was configured but close flag is set on %d.\n",cl_num,tmp_list->reverse);
		      return (0);
		    }

		  options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd = tmp_list->reverse;
		  options->srv_options->connect_infos->sd_table[cl_num].rev_prox_mode = tmp_list->reverse;
		  cctt_verbose(stderr,"Client %d proxying request is relayed in reverse proxy mode on sd %d.\n",cl_num,tmp_list->reverse);
		  options->srv_options->connect_infos->sd_table[cl_num].proxy_auth = SRV_CL_PROX_REV;
		}

	      auth = 1;
	      memcpy(&(options->srv_options->connect_infos->sd_table[cl_num].cl_prox_sockaddr),
		     &struct_received.cl_prox_sockaddr,
		     sizeof(struct_received.cl_prox_sockaddr));
	      memcpy(&(options->srv_options->connect_infos->sd_table[cl_num].prox_to_sockaddr),
		     &tmp_list->sockaddr,
		     sizeof(struct_received.cl_prox_sockaddr));

	      options->srv_options->connect_infos->sd_table[cl_num].tvu.tv_sec = 0;
	      options->srv_options->connect_infos->sd_table[cl_num].tvu.tv_usec = 0;
	      options->srv_options->connect_infos->sd_table[cl_num].tvq.tv_sec = 0;
	      options->srv_options->connect_infos->sd_table[cl_num].tvq.tv_usec = 0;
	      options->srv_options->connect_infos->sd_table[cl_num].tvr.tv_sec = 0;
	      options->srv_options->connect_infos->sd_table[cl_num].tvr.tv_usec = 0;

	      if (tmp_list->reverse != 0)
		add_rev_prox_mode_overhead_and_send(options,options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd,"",0,cl_num);
	    }
	  tmp_list = tmp_list->next;
	}
      if (memcmp(&(tmp_list->sockaddr.sin_addr),&(struct_received.sockaddr.sin_addr),sizeof(tmp_list->sockaddr.sin_addr)) == 0 &&
	  memcmp(&(tmp_list->sockaddr.sin_port),&(struct_received.sockaddr.sin_port),sizeof(tmp_list->sockaddr.sin_port)) == 0)
	{
	  cctt_verbose(stderr,"Client %d proxying request was configured.\n",cl_num);

	  /* 
	  ** On met en place la connexion demandee par le client
	  */
	  if (tmp_list->reverse == 0) /* Proxy normal */
	    {
	  if (open_local_connection_for_proxy(options, cl_num, tmp_list->sockaddr) == 0)
	    return (0);
	  options->srv_options->connect_infos->sd_table[cl_num].proxy_auth = SRV_CL_PROX_YES;
	    }
	  else /* Proxy reverse */
	    { 
	      /* N'y va pas si le close flag est positionne ! */
	      if (options->srv_options->connect_infos->sd_table[tmp_list->reverse].close_flag == 1)
		{
		  cctt_verbose(stderr,"Client %d proxying request was configured but close flag is set on %d.\n",cl_num,tmp_list->reverse);
		  return (0);
		}

	      options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd = tmp_list->reverse;
	      options->srv_options->connect_infos->sd_table[cl_num].rev_prox_mode = tmp_list->reverse;
	      cctt_verbose(stderr,"Client %d proxying request is relayed in reverse proxy mode on sd %d.\n",cl_num,tmp_list->reverse);
	      options->srv_options->connect_infos->sd_table[cl_num].proxy_auth = SRV_CL_PROX_REV;
	    }

	  auth = 1;
	  memcpy(&(options->srv_options->connect_infos->sd_table[cl_num].cl_prox_sockaddr),
		 &struct_received.cl_prox_sockaddr,
		 sizeof(struct_received.cl_prox_sockaddr));
	  memcpy(&(options->srv_options->connect_infos->sd_table[cl_num].prox_to_sockaddr),
		 &tmp_list->sockaddr,
		 sizeof(struct_received.cl_prox_sockaddr));
	  options->srv_options->connect_infos->sd_table[cl_num].tvu.tv_sec = 0;
	  options->srv_options->connect_infos->sd_table[cl_num].tvu.tv_usec = 0;
	  options->srv_options->connect_infos->sd_table[cl_num].tvq.tv_sec = 0;
	  options->srv_options->connect_infos->sd_table[cl_num].tvq.tv_usec = 0;

	  if (tmp_list->reverse != 0)
	    add_rev_prox_mode_overhead_and_send(options,options->srv_options->connect_infos->sd_table[cl_num].cl_req_sd,"",0,cl_num);
	}
    }
  if (auth == 0)
    cctt_verbose(stderr,"Client %d proxying request was not configured.\n",cl_num);
  /* Si la demande est fausse , la connexion est coupe dans la fonction appellante */
  return (auth);
}
