/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file.c,v 1.8 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

extern char *ConfigFileFunctionsArray[];
extern void *ConfigFileArrayPtr[];

/*
** Cette fonction cherche dans un tableau de pointeur sur fonctions
** si le label defini dans le fichier de configuration est bien
** associe a une fonction existante.
** Le cas echeant, elle appelle cette fonction.
*/

void search_config_function(S_OPTIONS *options,char *function,char *params)
{
  int i,exit;
  void (*ptr_fn)();

  for (i=0,exit=0;ConfigFileFunctionsArray[i] != NULL && exit == 0;i++)
    {
      if (strcmp(ConfigFileFunctionsArray[i],function)==0)
	{
	  ptr_fn = ConfigFileArrayPtr[i];
	  ptr_fn(options,function,params);
	  exit=1;
	}
    }
  if (exit == 0)
    cctt_error_and_exit(stderr,"[Conf] : %s is not valid !",function);
}

/*
** Cette fonction parse le fichier de configuration
** requis (serveur ou client) et appelle pour chaque ligne
** correcte la fonction au dessus qui se charge de chercher
** le bon pointeur sur fonction.
*/

void  parse_conf_file(S_OPTIONS *options)
{
  int  nb_oct_conf_file;
  char *conf_file, *conf_file_free;
  char *ptr,*function,*params;

  char test;

  /* Peut-on ouvrir le fichier de configuration ? */ 
  options->fd_conf_file = open(options->conf_file,O_RDONLY);
  if (options->fd_conf_file < 1)
    cctt_error_and_exit(stderr,"%s","[Options] : Cannot open configuration file !");
  cctt_debug(stderr,"Init : Ouverture fichier de config %s : OK",options->conf_file);

  /* Nombre d'octet du fichier de configuration + 1 */
  nb_oct_conf_file = lseek(options->fd_conf_file,0,SEEK_END);
  if (nb_oct_conf_file < 1)
    {
      close(options->fd_conf_file);
      cctt_error_and_exit(stderr,"%s","[Options] : Configuration file is empty !");
    }

  /* Fait attention a la derniere ligne du fichier (si elle est finie par un '\n' ou non */
  lseek(options->fd_conf_file,-1,SEEK_END);
  if ((read(options->fd_conf_file,&test,1) == 1) && test != '\n')
    nb_oct_conf_file += 1;

  /* Recuperation du contenu du fichier de configuration */
  conf_file = (char *) malloc (nb_oct_conf_file);
  memset (conf_file,0,nb_oct_conf_file);
  lseek(options->fd_conf_file,0,SEEK_SET);
  read(options->fd_conf_file,conf_file,nb_oct_conf_file-1);
  cctt_debug(stderr,"Init : Lecture de %d octets de config : OK",nb_oct_conf_file-1);

  /* Sauvegarde du pointeur pour le free et
     parsage du buffer ligne par ligne */
  conf_file_free = conf_file;
  while ((ptr = strsep(&conf_file,"\n")))
    {
      /* Si la ligne n'est pas vide et qu'il n'y a pas de caractere
	 indiquand un commentaire */
      if (strlen(ptr) > 1 && !index(ptr,'#'))
	{
	  function = strsep(&ptr,"=");
	  params = strsep(&ptr,"=");
	  search_config_function(options,function,params);
	}
    }
  
  free (conf_file_free);
  close(options->fd_conf_file);

  cctt_debug(stderr,"Init : Fermeture fichier de config %s : OK",options->conf_file);
}

/*
** Le fichier de configuration a ete parsee et les options
** correctes prises en compte.
** On verifie maintenant que la configuration demandee est viable
** et on positionne les flags en resultant dans les options.
*/

void check_conf_file_proxy_all_or_nothing(S_OPTIONS *options)
{
  /*
  ** Si le proxy a ete demande (CHANNEL_PROXY), alors les donnees de connexion
  ** relatives au serveur ont TOUTES dues etre copiees dans
  ** leurs champs de sauvegarde.
  ** Elles ne doivent donc plus etre nulles.
  */

  if (options->cl_options->dest_server_port != 0 &&
      options->cl_options->dest_server_prot != 0 &&
      options->cl_options->proxy_delay != 0 &&
      options->cl_options->dest_server_address != NULL)
    {
      options->options |= (0x01 << 10);
    }
  else
    {
      if (options->cl_options->dest_server_port ||
	  options->cl_options->dest_server_prot || 
	  options->cl_options->dest_server_address ||
	  options->cl_options->proxy_delay)
	cctt_error_and_exit(stderr,"%s","[Conf] : CHANNEL_PROXY directives allowing to tunnel trough a remote proxy are incomplete !");
    }
}

void check_conf_file_localproxy_all_or_nothing(S_OPTIONS *options)
{

  if (OPT_CL_PROX_MOD())
    {
      if (options->cl_options->local_he == NULL ||
	  options->cl_options->local_sockaddr.sin_port == 0 ||
	  options->cl_options->dest_server_proxy_mode_prot == 0 ||
	  options->cl_options->dest_server_proxy_mode_address == NULL ||
	  options->cl_options->dest_server_proxy_mode_port == 0)
	cctt_error_and_exit(stderr,"%s","[Conf] : PROXY_MODE_LOCAL directives to create a local proxy are incomplete !");
    }
  if (OPT_CL_REV_PROX_MOD())
    {
      if (options->cl_options->dest_server_proxy_mode_prot == 0 ||
	  options->cl_options->dest_server_proxy_mode_address == NULL ||
	  options->cl_options->dest_server_proxy_mode_port == 0)
	cctt_error_and_exit(stderr,"%s","[Conf] : PROXY_MODE_LOCAL directives to allow local proxying are incomplete !");
    }
}

void check_conf_file_http_mode_client(S_OPTIONS *OPTIONS)
{
  int http_mode=0,error=0;
  
  if (tell_me_about_encoding(options) == 2)
    http_mode = 1;

  if (!options->cl_options->http_mode_req_per_con || options->cl_options->http_mode_req_per_con <= 0 ||
      !options->cl_options->http_mode_del_bet_con || options->cl_options->http_mode_del_bet_con <= 0 ||
      !options->cl_options->http_mode_datasize || options->cl_options->http_mode_datasize <= 0 ||
      !options->cl_options->http_mode_uri || options->cl_options->http_mode_uri == NULL ||
      !options->cl_options->http_mode_hostname || options->cl_options->http_mode_hostname == NULL ||
      !options->cl_options->http_mode_contenttype || options->cl_options->http_mode_contenttype == NULL)
    {
      if (http_mode == 1)
	error = 1;
    }
  else
    if (http_mode == 0)
      error = 1;

  if (http_mode == 0 && (options->cl_options->http_mode_top_padding != NULL ||
			 options->cl_options->http_mode_bottom_padding != NULL ||
			 options->srv_options->http_mode_top_padding_size ||
			 options->srv_options->http_mode_bot_padding_size ||
			 options->cl_options->http_mode_fake_urls != NULL ||
			 options->cl_options->http_mode_fake_urls_freq))
    error = 1;

  if (http_mode == 1 && 
      ((options->cl_options->http_mode_fake_urls != NULL &&
       !options->cl_options->http_mode_fake_urls_freq) ||
      (!options->cl_options->http_mode_fake_urls &&
       options->cl_options->http_mode_fake_urls_freq)))
    error = 1;
      
  
  if (error == 1)
    {
      if (http_mode == 0)
	cctt_error_and_exit(stderr,"%s","[Conf] : HTTP_MOD directives used with a non HTTP mode channel !");
      else
	cctt_error_and_exit(stderr,"%s","[Conf] : HTTP_MOD directives are incomplete !");
    }
}

void check_conf_file_http_mode_server(S_OPTIONS *OPTIONS)
{
  int http_mode=0,error=0;
  
  if (tell_me_about_encoding(options) == 2)
    http_mode = 1;

  if (!options->srv_options->http_mode_uri || options->srv_options->http_mode_uri == NULL ||
      !options->srv_options->http_mode_error_page || options->srv_options->http_mode_error_page == NULL)
    {
      if (http_mode == 1)
	error = 1;
    }
  else
    if (http_mode == 0)
      error = 1;
  
  if (http_mode == 0 && (options->cl_options->http_mode_top_padding != NULL ||
			 options->cl_options->http_mode_bottom_padding != NULL ||
			 options->srv_options->http_mode_top_padding_size ||
			 options->srv_options->http_mode_bot_padding_size ||
			 options->srv_options->http_mode_fake_url_filename))
    error = 1;
  
  if (error == 1)
    {
      if (http_mode == 0)
	cctt_error_and_exit(stderr,"%s","[Conf] : HTTP_MOD directives used with a non HTTP mode channel !");
      else
	cctt_error_and_exit(stderr,"%s","[Conf] : HTTP_MOD directives are incomplete !");
    }
}

void check_conf_file_kill_del(S_OPTIONS *OPTIONS)
{
  if (options->srv_options->kill_quiet_del == 0)
    options->srv_options->kill_quiet_del = DEF_KILL_QUIET_DEL;
  if (options->srv_options->kill_quiet_del_cf == 0)
    options->srv_options->kill_quiet_del_cf = DEF_KILL_QUIET_DEL_CF;
}

void check_conf_file(S_OPTIONS *options)
{

  if (OPT_CL_OR_SRV())
    {
      /* 
      ** On verifie que si un proxy a ete demande,
      ** l' @IP, le port et le protocole ont ete specifies
      */

      check_conf_file_proxy_all_or_nothing(options);

      /*
      ** On verifie que si proxy_mode a ete demande,
      ** les donnees (@IP,port,prot.) relatives a l'etablissement de la socket
      ** locale et les donnees (@IP,port,prot) de la socket que devra etablir
      ** le serveur sont positionnees.
      ** ET CE SEULEMENT SI LE FLAG -a ou -z AVAIT ETE PASSE EN PARAMETRES
      */
      check_conf_file_localproxy_all_or_nothing(options);

      /*
      ** On verifie les donnees presentes pour le mode HTTP et egalement
      ** leur absence quand ce mode n'est pas HTTP.
      */ 
      check_conf_file_http_mode_client(options);

      /*
      ** On positionne la taille des donnees a placer dans une socket vers le serveur
      */
      if (tell_me_about_encoding(options) == 2)
	options->cl_options->srv_socket_buffer_size = options->cl_options->http_mode_datasize;
      else
	options->cl_options->srv_socket_buffer_size = SRV_SOCKET_BUFFER_SIZE;
      
      cctt_debug(stderr,"%s","Init : Verification du fichier de config  : OK");

    }
  else
    {
      check_conf_file_http_mode_server(options);
      check_conf_file_kill_del(options);
      cctt_debug(stderr,"%s","Init : Verification du fichier de config  : OK");
    }
}
