/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file_CHANNEL_PROXY.c,v 1.5 2003/04/23 07:06:10 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Les 3 fonctions qui suivent sont reservees au client et permettent
** de modifier la structure sockaddr (dans client.h) qui sera utilisee
** pour etablir la socket de communication.
** Cette socket reliera le client au proxy et les anciens parametres 
** concernant le serveur de destination sont stockes dans d'autres
** champs present dans la structure client.
**
*/

void conf_file_CHANNEL_PROXY_IP(S_OPTIONS *options,char *function, char *params)
{
  char *tmp_addr;

  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);

  /*
  ** On deplace l'adresse IP du serveur de destination dans un autre champ
  */
  tmp_addr = inet_ntoa(options->cl_options->sockaddr.sin_addr);
  options->cl_options->dest_server_address = (char *) malloc (strlen(tmp_addr)+1);
  memset (options->cl_options->dest_server_address,0,strlen(tmp_addr)+1);
  strncpy(options->cl_options->dest_server_address,tmp_addr,strlen(tmp_addr));
  
  cctt_debug(stderr,"Init : ConfFile(C) - CHANNEL_PROXY_IP, %s Dest_Srv avec proxy",tmp_addr);

  /*
  ** On stocke l'adresse du proxy dans la structure sockaddr
  */
  if (!((options->cl_options)->he =  (struct hostent *) gethostbyname(params)))
    cctt_error_and_exit(stderr,"[Conf] : CHANNEL_PROXY_IP=%s is not valid !\n",params);
  options->cl_options->sockaddr.sin_family = AF_INET;
  options->cl_options->sockaddr.sin_addr = *((struct in_addr *)options->cl_options->he->h_addr);
  bzero(&(options->cl_options->sockaddr.sin_zero), sizeof(options->cl_options->sockaddr.sin_zero));

  cctt_debug(stderr,"Init : ConfFile(C) - CHANNEL_PROXY_IP, %s positionnee en proxy",params);
}

void conf_file_CHANNEL_PROXY_PORT(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);

  /*
  ** On deplace le port du serveur de destination dans un autre champ
  */
  options->cl_options->dest_server_port = ntohs(options->cl_options->sockaddr.sin_port);

  cctt_debug(stderr,"Init : ConfFile(C) - CHANNEL_PROXY_PORT, %d Dst_Srv avec proxy",options->cl_options->dest_server_port);

  /*
  ** On stocke le port du proxy dans la structure sockaddr
  */
  if (!(options->cl_options->sockaddr.sin_port = htons(atoi(params))))
    cctt_error_and_exit(stderr,"[Conf] : CHANNEL_PROXY_PORT=%s is not valid !\n",params);

  cctt_debug(stderr,"Init : ConfFile(C) - CHANNEL_PROXY_PORT, %s positionne en proxy:port",params);
}

void conf_file_CHANNEL_PROXY_PROT(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);
  
  /*
  ** On deplace le protocole du serveur de destination dans un autre champ
  */
  options->cl_options->dest_server_prot = options->cl_options->protocol;

  cctt_debug(stderr,"Init : ConfFile(C) - CHANNEL_PROXY_PROT, protocole %d du Dst_Srv avec proxy",options->cl_options->dest_server_prot);

  /*
  ** On stocke le prot pour le proxy dans le champ approprie
  */
  options->cl_options->protocol = 0;
  if ((strcmp(params,"tcp")) == 0)
    options->cl_options->protocol = OPT_CL_PROT_TCP;
  if ((strcmp(params,"udp")) == 0)
    options->cl_options->protocol = OPT_CL_PROT_UDP;
  if (options->cl_options->protocol == 0)
    cctt_error_and_exit(stderr,"[Conf] : CHANNEL_PROXY_PROT=%s is not valid !\n",params);

  cctt_debug(stderr,"Init : ConfFile(C) - CHANNEL_PROXY_PROT, protocole %d positionne pour proxy:port",options->cl_options->protocol);
}

void conf_file_CHANNEL_PROXY_DEL(S_OPTIONS *options,char *function, char *params)
{
  int i;
  
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);

  /*
  ** On verifie qu'on a bien un delai
  */
  for (i=0;i<strlen(params);i++)
    if (!isdigit(params[i]))
      cctt_error_and_exit(stderr,"[Conf] : CHANNEL_PROXY_DEL=%s is not valid !\n",params);

  /*
  ** On stocke le delai
  */
  options->cl_options->proxy_delay = atoi(params);
  cctt_debug(stderr,"Init : ConfFile(C) - CHANNEL_PROXY_DEL, %s msecs positionnees",params);
}
