
/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file_HTTP_MOD.c,v 1.2 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Toutes les fonctions de verification des parametres des fichiers de configuration
** pour une utilisation du mode HTTP.
*/

/********************************************************
** Les fonctions qui suivent sont reservees au client. **
********************************************************/

/* Le nombre de requete prevues par connexion TCP */

void conf_file_HTTP_MOD_CL_REQ_PER_CON(S_OPTIONS *options,char *function, char *params)
{
  int i;

  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_REQ_PER_CON %s directive is incomplete !",params);

  /* Verifie que notre appel atoi() va fonctionner */
  for (i=0;i<strlen(params);i++)
    if (!isdigit(params[i]))
      cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_REQ_PER_CON (%s) directive is invalid !",params);

  options->cl_options->http_mode_req_per_con = (int) atoi(params);
  cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_CL_REQ_PER_CON positionne a %d",options->cl_options->http_mode_req_per_con);
}

/* Le delay a respecter entre chaque connexion TCP */

void conf_file_HTTP_MOD_CL_DELAY_BET_CON(S_OPTIONS *options,char *function, char *params)
{
  int i;

  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_DELAY_BET_CON %s directive is incomplete !",params);

  /* Verifie que notre appel atoi() va fonctionner */
  for (i=0;i<strlen(params);i++)
    if (!isdigit(params[i]))
      cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_DELAY_BET_CON (%s) directive is invalid !",params);

  options->cl_options->http_mode_del_bet_con = (int) atoi(params);
  cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_CL_DELAY_BET_CON positionne a %d msecs",options->cl_options->http_mode_del_bet_con);
}

/* Fake Urls list */

void conf_file_HTTP_MOD_CL_FAKE_URLS(S_OPTIONS *options,char *function, char *params)
{
  int i=0;

  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_FAKE_URLS %s directive is incomplete !",params);

  /* Si la liste n'existe pas encore */
  if (options->cl_options->http_mode_fake_urls == NULL)
    {
      /* On reserve la memoire pour le tableau */
      options->cl_options->http_mode_fake_urls = (char **) malloc (sizeof(char **));
      /* On ajoute le premier element */
      options->cl_options->http_mode_fake_urls[0] = (char *) malloc(strlen(params)+1);
      memset(options->cl_options->http_mode_fake_urls[0],0,strlen(params)+1);
      strncpy(options->cl_options->http_mode_fake_urls[0],params,strlen(params));
      cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_CL_FAKE_URLS (0) positionne a %s",params);
      /* On initialise l'element suivant */
      options->cl_options->http_mode_fake_urls[1] = NULL;
    }
  else
    {
      /* Un element a deja ete ajoute... on avance ... */
      while (options->cl_options->http_mode_fake_urls[i] != NULL)
	i++;
      /* On ajoute cet element */
      options->cl_options->http_mode_fake_urls[i] = (char *) malloc(strlen(params)+1);
      memset(options->cl_options->http_mode_fake_urls[i],0,strlen(params)+1);
      strncpy(options->cl_options->http_mode_fake_urls[i],params,strlen(params));
      cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_CL_FAKE_URLS (%d) positionne a %s",i,params);
      /* On initialise l'element suivant */
      options->cl_options->http_mode_fake_urls[i+1] = NULL;
    }
}

/* Frequence d'envoi des fausses requetes (0 = random, 1 = une par vrai requete, x = toutes les x requetes */

void conf_file_HTTP_MOD_CL_FAKE_URLS_FREQ(S_OPTIONS *options,char *function, char *params)
{
  int i;

  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_FAKE_URLS_FREQ %s directive is incomplete !",params);

  /* Verifie que notre appel atoi() va fonctionner */
  for (i=0;i<strlen(params);i++)
    if (!isdigit(params[i]) && params[i] != '-')
      cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_FAKE_URLS_FREQ (%s) directive is invalid !",params);

  options->cl_options->http_mode_fake_urls_freq = (int) atoi(params);

  if (options->cl_options->http_mode_fake_urls_freq == 0 ||
      options->cl_options->http_mode_fake_urls_freq < -1)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_FAKE_URLS_FREQ (%s) directive is invalid !",params);

  cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_CL_FAKE_URLS_FREQ positionne a %d",options->cl_options->http_mode_fake_urls_freq);
}

/* L'Hostname a ajouter dans la requete */

void conf_file_HTTP_MOD_CL_HOSTNAME(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_HOSTNAME %s directive is incomplete !",params);

  options->cl_options->http_mode_hostname = (char *) malloc(strlen(params)+1);
  memset(options->cl_options->http_mode_hostname,0,strlen(params)+1);
  strncpy(options->cl_options->http_mode_hostname,params,strlen(params));
  cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_HOSTNAME positionne a %s",params);
}

/* Le Content-Type a ajouter dans la requete */

void conf_file_HTTP_MOD_CL_CONTENTTYPE(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CONTENTTYPE %s directive is incomplete !",params);

  options->cl_options->http_mode_contenttype = (char *) malloc(strlen(params)+1);
  memset(options->cl_options->http_mode_contenttype,0,strlen(params)+1);
  strncpy(options->cl_options->http_mode_contenttype,params,strlen(params));
  cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_CONTENTTYPE positionne a %s",params);
}

/* La taille maximum des donnees a placer en donnees HTTP */

void conf_file_HTTP_MOD_CL_DATASIZE(S_OPTIONS *options,char *function, char *params)
{
  int i;

  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_DATASIZE %s directive is incomplete !",params);

  /* Verifie que notre appel atoi() va fonctionner */
  for (i=0;i<strlen(params);i++)
    if (!isdigit(params[i]))
      cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_DATASIZE (%s) directive is invalid !",params);

  options->cl_options->http_mode_datasize = (int) atoi(params);
  cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_CL_DATASIZE positionne a %d",options->cl_options->http_mode_datasize);
}


/********************************************************
** Ces fonctions sont communes pour clients et serveur **
********************************************************/

/* L'URI a ajouter dans la requete */

void conf_file_HTTP_MOD_URI(S_OPTIONS *options,char *function, char *params)
{
  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_URI %s directive is incomplete !",params);
  
  if (OPT_CL_OR_SRV())
    {
      options->cl_options->http_mode_uri = (char *) malloc(strlen(params)+1);
      memset(options->cl_options->http_mode_uri,0,strlen(params)+1);
      strncpy(options->cl_options->http_mode_uri,params,strlen(params));
      cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_URI positionne a %s",params);
    }
  else
    {
      options->srv_options->http_mode_uri = (char *) malloc(strlen(params)+1);
      memset(options->srv_options->http_mode_uri,0,strlen(params)+1);
      strncpy(options->srv_options->http_mode_uri,params,strlen(params));
      cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_URI positionne a %s",params);
    }
}

/* Nom de fichier a utiliser pour le top padding d'un message HTTP (requete ou reponse) */

void conf_file_HTTP_MOD_CL_TOP_PAD(S_OPTIONS *options,char *function, char *params)
{
  FILE *top_file;
  int nb_oct;

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_TOP_PAD %s directive is incomplete !",params);

  if ((top_file = fopen(params,"r")) == NULL)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_TOP_PAD : %s file is not valid !",params);
  else
    {
      fseek(top_file,0,SEEK_END);
      nb_oct = ftell(top_file);
      if (nb_oct < 1)
	{
	  fclose(top_file);
	  cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_TOP_PAD : %s file is not valid !",params);
	}
      fseek(top_file,0,SEEK_SET);
      options->cl_options->http_mode_top_padding = (char *) malloc (nb_oct);
      memset(options->cl_options->http_mode_top_padding,0,nb_oct);
      fread(options->cl_options->http_mode_top_padding,nb_oct-1,1,top_file);
      cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_CL_TOP_PAD, %d octets lus",nb_oct-1);
      fclose(top_file);
    }
}

/* Nom de fichier a utiliser pour le bottom padding d'un message HTTP (requete ou reponse) */

void conf_file_HTTP_MOD_CL_BOT_PAD(S_OPTIONS *options,char *function, char *params)
{
  FILE *bot_file;
  int nb_oct;

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_BOT_PAD %s directive is incomplete !",params);

  if ((bot_file = fopen(params,"r")) == NULL)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_BOT_PAD : %s file is not valid !",params);
  else
    {
      fseek(bot_file,0,SEEK_END);
      nb_oct = ftell(bot_file);
      if (nb_oct < 1)
	{
	  fclose(bot_file);
	  cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_CL_BOT_PAD : %s file is not valid !",params);
	}
      fseek(bot_file,0,SEEK_SET);
      options->cl_options->http_mode_bottom_padding = (char *) malloc (nb_oct);
      memset(options->cl_options->http_mode_bottom_padding,0,nb_oct);
      fread(options->cl_options->http_mode_bottom_padding,nb_oct-1,1,bot_file);
      cctt_debug(stderr,"Init : ConfFile(C) - HTTP_MOD_CL_BOT_PAD, %d octets lus",nb_oct-1);
      fclose(bot_file);
    }
}

/* Taille en octet du top padding que l'autre cote ajoute a ses donnees */

void conf_file_HTTP_MOD_SRV_TOP_PAD(S_OPTIONS *options,char *function, char *params)
{
  int i;

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_TOP_PAD %s directive is incomplete !",params);

  /* Verifie que notre appel atoi() va fonctionner */
  for (i=0;i<strlen(params);i++)
    if (!isdigit(params[i]))
      cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_TOP_PAD (%s) directive is invalid !",params);

  options->srv_options->http_mode_top_padding_size = (int) atoi(params);
  cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_SRV_TOP_PAD positionne a %d",options->srv_options->http_mode_top_padding_size);
}

/* Taille en octet du bottom padding que l'autre cote ajoute a ses donnees */

void conf_file_HTTP_MOD_SRV_BOT_PAD(S_OPTIONS *options,char *function, char *params)
{
  int i;

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_BOT_PAD %s directive is incomplete !",params);

  /* Verifie que notre appel atoi() va fonctionner */
  for (i=0;i<strlen(params);i++)
    if (!isdigit(params[i]))
      cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_BOT_PAD (%s) directive is invalid !",params);

  options->srv_options->http_mode_bot_padding_size = (int) atoi(params);
  cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_SRV_BOT_PAD positionne a %d",options->srv_options->http_mode_bot_padding_size);
}

/********************************************************
** Les fonctions qui suivent sont reservees au serveur.**
********************************************************/

/*
** La liste des fichiers qu'on cherchera en recevant des fausses urls du client
** On ne stocke que le nom de fichier, charge a la fonction qui implementera le
** test de verifier si le fichier peut etre ouvert et d'envoyer son contenu.
*/

void conf_file_HTTP_MOD_SRV_FAKE_URLS(S_OPTIONS *options,char *function, char *params)
{
  int i=0;

  /* Suremment pas en mode client */
  if (OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in client mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_FAKE_URLS %s directive is incomplete !",params);

  /* Si la liste n'existe pas encore */
  if (options->srv_options->http_mode_fake_url_filename == NULL)
    {
      /* On reserve la memoire pour le tableau */
      options->srv_options->http_mode_fake_url_filename = (char **) malloc (sizeof(char **));
      /* On ajoute le premier element */
      options->srv_options->http_mode_fake_url_filename[0] = (char *) malloc(strlen(params)+1);
      memset(options->srv_options->http_mode_fake_url_filename[0],0,strlen(params)+1);
      strncpy(options->srv_options->http_mode_fake_url_filename[0],params,strlen(params));
      cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_SRV_FAKE_URLS (0) positionne a %s",params);
      /* On initialise l'element suivant */
      options->srv_options->http_mode_fake_url_filename[1] = NULL;
    }
  else
    {
      /* Un element a deja ete ajoute... on avance ... */
      while (options->srv_options->http_mode_fake_url_filename[i] != NULL)
	i++;
      /* On ajoute cet element */
      options->srv_options->http_mode_fake_url_filename[i] = (char *) malloc(strlen(params)+1);
      memset(options->srv_options->http_mode_fake_url_filename[i],0,strlen(params)+1);
      strncpy(options->srv_options->http_mode_fake_url_filename[i],params,strlen(params));
      cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_SRV_FAKE_URLS (%d) positionne a %s",i,params);
      /* On initialise l'element suivant */
      options->srv_options->http_mode_fake_url_filename[i+1] = NULL;
    }
}

/* La page d'erreur que le serveur envoie */

void conf_file_HTTP_MOD_SRV_ERROR_PAGE(S_OPTIONS *options,char *function, char *params)
{
  FILE *error_file;
  int nb_oct;

  /* Suremment pas en mode client */
  if (OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in client mode is not valid !",function);

  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_ERROR_PAGE %s directive is incomplete !",params);

  if ((error_file = fopen(params,"r")) == NULL)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_ERROR_PAGE : %s file is not valid !",params);
  else
    {
      fseek(error_file,0,SEEK_END);
      nb_oct = ftell(error_file);
      if (nb_oct < 1)
	{
	  fclose(error_file);
	  cctt_error_and_exit(stderr,"[Conf] : HTTP_MOD_SRV_ERROR_PAGE : %s file is not valid !",params);
	}
      fseek(error_file,0,SEEK_SET);
      options->srv_options->http_mode_error_page = (char *) malloc (nb_oct);
      memset(options->srv_options->http_mode_error_page,0,nb_oct);
      fread(options->srv_options->http_mode_error_page,nb_oct-1,1,error_file);
      cctt_debug(stderr,"Init : ConfFile(S) - HTTP_MOD_SRV_ERROR_PAGE, %d octets lus",nb_oct-1);
      fclose(error_file);
    }
}
