/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file_HTTP_PROXY_CHAIN.c,v 1.5 2003/04/23 07:06:10 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Cette fonction permet de gerer une chaine de proxy en stockant
** dans une variable de la structure cliente les proxy intermediaires.
*/

void conf_file_HTTP_PROXY_CHAIN(S_OPTIONS *options,char *function, char *params)
{
  char *bk_params;
  char *tmproxy,*addr_proxy,*port_proxy, *delay_proxy;
  struct hostent *tmphe;
  int i,cpt=0;

  bk_params = params;
  
  if (!params || strlen(params) == 0)
    cctt_error_and_exit(stderr,"[Conf] : HTTP_PROXY_CHAIN %s directive is incomplete !\n",params);

  /*
  ** On parse =proxy:port;proxy:port etc...
  */

  options->cl_options->http_proxy_chain_addrs = (char **) malloc (sizeof(char **));
  options->cl_options->http_proxy_chain_ports = (char **) malloc (sizeof(char **));
  options->cl_options->http_proxy_chain_delays = (int *) malloc (sizeof(int *));

  while ((tmproxy = strsep(&params,";")) != NULL)
    {
      if (!tmproxy || strlen(tmproxy) == 0)
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",bk_params);

      /* On verifie la pertinence de la directive */
      if ((addr_proxy = strsep(&tmproxy,":")) == NULL || strlen(addr_proxy) == 0)
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",bk_params);
      if ((port_proxy = strsep(&tmproxy,":")) == NULL || strlen(port_proxy) == 0)
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",bk_params);
      if ((delay_proxy = strsep(&tmproxy,":")) == NULL || strlen(delay_proxy) == 0)
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",bk_params);

      /* On verifie que les donnees sont bien des adresses et des ports */
      if (!(tmphe = (struct hostent *) gethostbyname(addr_proxy)))
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",addr_proxy);
      for (i=0;i<strlen(port_proxy);i++)
	if (!isdigit(port_proxy[i]))
	  cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",port_proxy);
      if (!(i = atoi(port_proxy)) || i < 1 || i > 65535)
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",port_proxy);

      for (i=0;i<strlen(delay_proxy);i++)
	if (!isdigit(delay_proxy[i]))
	  cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",delay_proxy);
      if (!(i = atoi(delay_proxy)))
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",delay_proxy);
      
      /* On a ici une adresse IP, un port et un delai qu'on stocke */

      options->cl_options->http_proxy_chain_addrs[cpt] = (char *) malloc (strlen(addr_proxy)+1);
      memset(options->cl_options->http_proxy_chain_addrs[cpt],0,strlen(addr_proxy)+1);
      strncpy(options->cl_options->http_proxy_chain_addrs[cpt],addr_proxy,strlen(addr_proxy));
      
      options->cl_options->http_proxy_chain_ports[cpt] = (char *) malloc (strlen(port_proxy)+1);
      memset(options->cl_options->http_proxy_chain_ports[cpt],0,strlen(port_proxy)+1);
      strncpy(options->cl_options->http_proxy_chain_ports[cpt],port_proxy,strlen(port_proxy));

      options->cl_options->http_proxy_chain_delays[cpt] = (int) malloc (sizeof(int));
      options->cl_options->http_proxy_chain_delays[cpt] = i;

      cpt+=1;
      options->cl_options->http_proxy_chain_addrs[cpt] = NULL;
      options->cl_options->http_proxy_chain_ports[cpt] = NULL;
      options->cl_options->http_proxy_chain_delays[cpt] = 0;
    }
}
