/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file_IDENT.c,v 1.5 2003/04/23 07:06:10 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Pour chaque type d'identification susceptible d'etre parametree
** dans le fichier de configuration, nous avons un champ de
** de cette structure, suivi d'un pointeur sur la fonction
** correspondante.
** Ce pointeur sur fonction est soit pour le client, soit pour le serveur.
** Les fonctions serveurs sont dans :
** src/server/srv_ident_client.c et includes/srv_ident_client.h
** Les fonctions client sont dans :
** src/channels/cl_identification.c et includes/cl_identification.h
** La structure est dans includes/srv_ident_client.h
*/

T_ident_type ident_type_array[] = {
  {"clear_ident",cl_identification_clear,srv_ident_client_clear},
  {"basic_ident",cl_identification_basic,srv_ident_client_basic},
  {NULL,NULL,NULL}
};

void conf_file_IDENT(S_OPTIONS *options,char *function, char *params)
{
  int i,exit;
  
  /* Verifie si le type d'identification existe */
  for (i=0,exit=0;ident_type_array[i].ident_name != NULL;i++)
    if ((strcmp(params,ident_type_array[i].ident_name) == 0))
      exit=1;
  if (exit == 0)
    cctt_error_and_exit(stderr,"%s","[Options] : Identification asked doesn't exist !\n");
  
  options->identification = (char *) malloc (strlen(params)+1);
  memset (options->identification,0,strlen(params)+1);
  strncpy(options->identification,params,strlen(params));
}

void do_ident_exist(S_OPTIONS *options)
{
  if (options->identification == NULL)
    cctt_error_and_exit(stderr,"%s","[Options] : Identification asked doesn't exist !\n");
  if (options->ident_key == NULL)
    cctt_error_and_exit(stderr,"%s","[Options] : No IDENT_KEY defined in configuration file !\n");
}

void conf_file_IDENT_KEY(S_OPTIONS *options,char *function, char *params)
{
  options->ident_key = (char *) malloc (strlen(params)+1);
  memset (options->ident_key,0,strlen(params)+1);
  strncpy(options->ident_key,params,strlen(params));
}
