/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file_PERM.c,v 1.5 2003/04/23 07:06:10 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Cette fonction est reservee au serveur.
** Elle permet de positionner les variables de restriction de privileges quand
** c'est root qui execute le programme.
*/

void conf_file_PERM_USER_GROUP(S_OPTIONS *options,char *function, char *params)
{
  struct passwd *tmp_passwd;

  /* Suremment pas en mode client */
  if (OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in client mode is not valid !\n",function);

  if (geteuid() == 0)
    {
      if (strlen(params) == 0)
	cctt_error_and_exit(stderr,"[Conf] : PERM_USER_GROUP (%s) directive is incomplete !\n",params);
      /* Recupere uid en verifiant que user existe */
      if ((tmp_passwd = getpwnam(params)) == NULL)
	cctt_error_and_exit(stderr,"[Conf] : User %s doesn't exist (PERM_USER_GROUP directive) !!!\n",params);
      /* Stocke l'uid et le gid*/
      options->srv_options->perm_uid = tmp_passwd->pw_uid;
      cctt_debug(stderr,"Init : ConfFile(C) - PERM_USER_GROUP : uid %d positionee",(int) options->srv_options->perm_uid);
      options->srv_options->perm_gid = tmp_passwd->pw_gid;
      cctt_debug(stderr,"Init : ConfFile(C) - PERM_USER_GROUP : gid %d positione",(int) options->srv_options->perm_gid);
    }
  else
    cctt_error_and_exit(stderr,"%s","[Options] : Using PERM_USER_GROUP without uid 0 isn't allowed !\n");
}

void conf_file_PERM_CHROOT(S_OPTIONS *options,char *function, char *params)
{
  struct stat tmpstat;

  if (geteuid() == 0)
    {
      if (strlen(params) == 0) /* Repertoire positionne ? */
	cctt_error_and_exit(stderr,"[Conf] : PERM_CHROOT (%s) directive is incomplete !\n",params);

      if (lstat(params,&tmpstat) != 0) /* Repertoire existe ? */
	cctt_error_and_exit(stderr,"[Conf] : PERM_CHROOT : Directory %s not accessible !\n",params);

      /* Est bien un repertoire et n'est pas un lien ? */
      if (!S_ISDIR(tmpstat.st_mode) || S_ISLNK(tmpstat.st_mode))
	cctt_error_and_exit(stderr,"[Conf] : PERM_CHROOT : Directory %s not accessible !\n",params);

      options->srv_options->perm_chroot = (char *) malloc (strlen(params)+1);
      memset (options->srv_options->perm_chroot,0,strlen(params)+1);
      strncpy(options->srv_options->perm_chroot,params,strlen(params));
    }
  else
    cctt_error_and_exit(stderr,"%s","[Options] : Using PERM_CHROOT without uid 0 isn't allowed !\n");
}

