/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file_PROXY_MODE.c,v 1.5 2003/04/23 07:06:10 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Nous remplissons la structure local_sockaddr avec l'adresse IP fournie en argument.
** Cette structure permettra au client de se positionner en listen sur une socket
** local de facon a agir comme un proxy local si cette fonctionnalite est demandee.
*/

void conf_file_PROXY_MODE_LOCAL_IP(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);

  /* Uniquemment en proxy_mode */
  if (OPT_CL_PROX_MOD())
    {
  if (!((options->cl_options)->local_he =  (struct hostent *) gethostbyname(params)))
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LOCAL_IP=%s is not valid !\n",params);
  options->cl_options->local_sockaddr.sin_family = AF_INET;
  options->cl_options->local_sockaddr.sin_addr = *((struct in_addr *)options->cl_options->local_he->h_addr);
  bzero(&(options->cl_options->local_sockaddr.sin_zero), sizeof(options->cl_options->local_sockaddr.sin_zero));

      cctt_debug(stderr,"Init : ConfFile(C) - PROXY_MODE_LOCAL_IP, adresse %s positionnee pour proxy local",params);
    }
}

/*
** Nous remplissons la structure local_sockaddr avec le port fourni en argument.
** Cette structure permettra au client de se positionner en listen sur une socket
** local de facon a agir comme un proxy local si cette fonctionnalite est demandee.
*/

void conf_file_PROXY_MODE_LOCAL_PORT(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);

  /* Uniquemment en proxy_mode */
  if (OPT_CL_PROX_MOD())
    {
  if (!(options->cl_options->local_sockaddr.sin_port = htons(atoi(params))))
	cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LOCAL_PORT=%s is not valid !\n",params);

      cctt_debug(stderr,"Init : ConfFile(C) - PROXY_MODE_LOCAL_PORT, port %s positionne pour proxy local",params);
    }
}

/*
** Nous renseignons un champ protocole dans la structure d'information du client nous indiquant
** que le client se comportera comme un proxy local pour le protocole indique.
*/

void conf_file_PROXY_MODE_PROT(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);
  
  options->cl_options->dest_server_proxy_mode_prot = 0;
  if ((strcmp(params,"tcp")) == 0)
    options->cl_options->dest_server_proxy_mode_prot = OPT_CL_PROT_TCP;
  if ((strcmp(params,"udp")) == 0)
    options->cl_options->dest_server_proxy_mode_prot = OPT_CL_PROT_UDP;

  if (options->cl_options->dest_server_proxy_mode_prot == 0)
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_PROT=%s is not valid !\n",params);

  cctt_debug(stderr,"Init : ConfFile(C) - PROXY_MODE_PROT, protocole %d positionne pour proxy local",options->cl_options->dest_server_proxy_mode_prot);
}

/*
** Nous renseignons la structure d'information client pour le cas ou le mode proxy local a ete
** demande.
** Le client emettra un requete de connexion (dans les datas) a destination du serveur en lui 
** precisant l'adresse IP qui est indiquee ici.
*/

void conf_file_PROXY_MODE_REMOTE_IP(S_OPTIONS *options,char *function, char *params)
{
  struct hostent *tmp_he;

  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);
  
  if (!((tmp_he =  (struct hostent *) gethostbyname(params))))
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_REMOTE_IP=%s is not valid !\n",params);
  
  options->cl_options->dest_server_proxy_mode_address = (char *) malloc (strlen(params)+1);
  memset (options->cl_options->dest_server_proxy_mode_address,0,strlen(params)+1);
  strncpy(options->cl_options->dest_server_proxy_mode_address,params,strlen(params));
  
  cctt_debug(stderr,"Init : ConfFile(C) - PROXY_MODE_REMOTE_IP, adresse %s positionnee pour proxying via le serveur",params);
}

/*
** Nous renseignons la structure d'information client pour le cas ou le mode proxy local a ete
** demande.
** Le client emettra un requete de connexion (dans les datas) a destination du serveur en lui 
** precisant le port qui est indiquee ici.
*/

void conf_file_PROXY_MODE_REMOTE_PORT(S_OPTIONS *options,char *function, char *params)
{
  /* Suremment pas en mode serveur */
  if (! OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in server mode is not valid !\n",function);

  if (!(options->cl_options->dest_server_proxy_mode_port = atoi(params)))
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_REMOTE_PORT=%s is not valid !\n",params);

  cctt_debug(stderr,"Init : ConfFile(C) - PROXY_MODE_REMOTE_PORT, port %s positionne pour proxying via le serveur",params);
}
