/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: conf_file_PROXY_MODE_LIST.c,v 1.5 2003/04/23 07:06:10 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Pourquoi une ?#!. de liste chainee ?
** Tout simplement qu'au bout d'une heure de prise de tete
** sur ce !@#// de tableau, j'ai pense que j'irais plus vite
** comme ca :)
*/

void add_auth_to_chained_list(S_OPTIONS *options, T_proxy_allowed_list *tmp_list)
{
  T_proxy_allowed_list *browser;

  /*
  ** On ajoute dans la liste chainee
  */

  /* Si premier element de la liste */
  if (!options->srv_options->proxy_allowed_list)
    {
      /* Le premier est cet element */
      tmp_list->first = tmp_list;
      /* Le next est nul */
      tmp_list->next = NULL;
      /* On enregistre */
      options->srv_options->proxy_allowed_list = tmp_list;
    }
  else
    {
      /* Le premier est le premier de la liste */
      tmp_list->first = options->srv_options->proxy_allowed_list->first;
      /* Le next est NULL */
      tmp_list->next = NULL;

      /* On se deplace jusqu'au dernier */
      browser = options->srv_options->proxy_allowed_list;
      while (browser->next)
	browser = browser->next;

      /* Le next de la liste est cet element */
      browser->next = tmp_list;
      options->srv_options->proxy_allowed_list = browser;
    }

  /* On fait +1 pour le total d'elements */
  options->srv_options->proxy_allowed_list_elements_number += 1;

  options->srv_options->proxy_allowed_list = options->srv_options->proxy_allowed_list->first;
}

/*
** Cette fonction est reservee au serveur
** Elle permet de creer, a partir des informations du fichier de config
** la liste des autorisations de proxying au niveau du serveur.
*/

void conf_file_PROXY_MODE_LIST(S_OPTIONS *options,char *function, char *params)
{
  char *mode_name,*mode_ip,*mode_port;
  char *bk_params;
  T_proxy_allowed_list *tmp_list;

  /* Suremment pas en mode client */
  if (OPT_CL_OR_SRV())
    cctt_error_and_exit(stderr,"[Conf] : Using %s in client mode is not valid !\n",function);

  bk_params = params;

  /* Verification de la syntaxe de la ligne d'autorisation (label:ip:port) */

  if (!(mode_name = strsep(&params,":")) || strlen(mode_name) == 0)
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",bk_params);
  if (!(mode_ip = strsep(&params,":")) || strlen(mode_ip) == 0)
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",bk_params);
  if (!(mode_port = strsep(&params,":")) || strlen(mode_port) == 0)
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directives are incomplete !\n",bk_params);

  /* 
  ** Verification de la pertinence des informations entrees 
  ** et sauvegarde des informations dans la structure 
  */

  /* On cree un nouvel element dans la structure temporaire */
  tmp_list = (T_proxy_allowed_list *) malloc (sizeof(T_proxy_allowed_list));
  memset(tmp_list,0,sizeof(T_proxy_allowed_list));

  /* On positionne le label */
  tmp_list->label = (char *) malloc (strlen(mode_name)+1);
  memset(tmp_list->label,0,strlen(mode_name)+1);
  strncpy(tmp_list->label,mode_name,strlen(mode_name));

  /* On verifie l'adresse IP et on l'enregistre */
  if (!(tmp_list->he =  (struct hostent *) gethostbyname(mode_ip)))
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directive is not valid !\n",bk_params);
  tmp_list->sockaddr.sin_family = AF_INET;
  tmp_list->sockaddr.sin_addr = *((struct in_addr *)tmp_list->he->h_addr);
  bzero(&(tmp_list->sockaddr.sin_zero), sizeof(tmp_list->sockaddr.sin_zero));

  /* On verifie le port et on l'enregistre */
  if (!(tmp_list->sockaddr.sin_port = htons(atoi(mode_port))))
    cctt_error_and_exit(stderr,"[Conf] : PROXY_MODE_LIST (%s) directive is not valid !\n",bk_params);

  /* L'element est relatif a un proxying normal */
  tmp_list->reverse=0;

  /* On ajoute l'element dans la liste */
  add_auth_to_chained_list(options,tmp_list);

  cctt_debug(stderr,"Init : ConfFile(S) - PROXY_MODE_LIST %s directive syntax seems to be correct",bk_params);
}

