/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: do_server.c,v 1.10 2003/06/11 13:14:19 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Ces fonctions gerent le serveur :
** # attente des clients
** # enregistrement des clients
** # desenregistrement des clients
** # appel d'un pointeur sur fonction relatif au type de canal
** # commandes passees sur le prompt du serveur
*/

/*
** Cette fonction est appellee apres chaque retour de select.
** Elle verifie si le temp d'attente avant envoi du buffer temporaire est ecoule.
** Le cas echeant, elle transmet le buffer pour traitement.
*/

void srv_manage_client_check_and_go(S_OPTIONS *options, int client_number)
{
  /* 
  ** Si le temp d'attente de remplissage est ecoule et qu'on a qqc a envoyer
  */
  if ((options->srv_options->connect_infos->sd_table[client_number].tvr.tv_usec > WAITIME_BEFORE_ACTION)
      && options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size != 0)
    {
      /* on remet a zero le compteur de temps */
      options->srv_options->connect_infos->sd_table[client_number].tvr.tv_usec = 0;
      /* On traite le buffer par l'appel d'un pointeur sur fonction (Voir channels.c) */
      do_channel(options,client_number,
		 options->srv_options->connect_infos->sd_table[client_number].tmpbuf,
		 options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size);
      /* 
      ** Si ce pointeur est positionne, cela signifie qu'une fonction n'a pas pu traiter toutes les donnees
      ** du buffer (par exemple un client en mode proxy inverse a ete lu en plusieurs read).
      ** tmpbuf_ptr pointe vers les donnees a replacer dans le buffer
      ** tmpbuf_size contient la longueur de ces donnees
      */
      if (options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr != NULL)
	{
	  memcpy(options->srv_options->connect_infos->sd_table[client_number].tmpbuf,
		  options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr,
		  options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size);
	  memset(options->srv_options->connect_infos->sd_table[client_number].tmpbuf+options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size,0,ACTION_BUFFER_SIZE-options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size);
	  /* On n'initialise surtout pas tmpbuf_size !!! par contre on reset tmpbuf_ptr */
	  free (options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr);
	  options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr = NULL;
	}
      else
	{
	  /* Tout a ete correctement emis => On vide le buffer et on reinitialise */
      memset(options->srv_options->connect_infos->sd_table[client_number].tmpbuf,0,ACTION_BUFFER_SIZE);
      options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size = 0;
	}
    }
}

/*
** Cette fonction est appelle a chaque fois qu'un client TCP active un socket descriptor
** du serveur. Elle permet de stocker le buffer lu et de l'envoyer si la taille du nouveau
** buffer n'entre pas dans le buffer temporaire.
*/

void srv_manage_client(S_OPTIONS *options,int client_number)
{
  char buffer[SRV_SOCKET_BUFFER_SIZE+1];
  int nsocket;
  int sock_size;

  /* Initialise ce dont on aura besoin */
  sock_size = sizeof(struct sockaddr_in);
  memset(buffer,0,SRV_SOCKET_BUFFER_SIZE+1);

  /* Lit un paquet pour la taille SRV_SOCKET_BUFFER_SIZE */
  if ((nsocket = recvfrom(client_number,
			  buffer,SRV_SOCKET_BUFFER_SIZE,0,
			  (struct sockaddr *) &(options->srv_options->connect_infos->sd_table[client_number].sockaddr),
			  &sock_size)) < 0)
    {
      cctt_debug(stderr,"%s","srv_manage_client : read error\n");
      reset_client_connection(options,client_number);
    }

  if (nsocket > 0 && OPT_DEBUG2()) cctt_debug(stderr,"Srv  : Lecture de %d octets du client %d",nsocket,client_number);

  /* Si on a lu quelquechose */
  if (nsocket > 0)
    {
      /* 
      ** Si on a la place d'ajouter dans le buffer temporaire 
      */
      if (nsocket + options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size < ACTION_BUFFER_SIZE)
	{
	  /* On a la place d'ajouter dans le buffer */
	  memcpy(options->srv_options->connect_infos->sd_table[client_number].tmpbuf+options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size,buffer,nsocket);
	  options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size += nsocket;

	  if (OPT_DEBUG2()) cctt_debug (stderr,"Srv  :   Storing in temporary buffer (%d bytes on %d total) for %d",nsocket,options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size,client_number);
	}
      else
	{
	  /* Sinon, il faut qu'on traite tout de suite ce qu'il y a en attente */

	  /* on remet a zero le compteur de temps */
	  options->srv_options->connect_infos->sd_table[client_number].tvr.tv_usec = 0;
	  /* On traite le buffer */
	  if (OPT_DEBUG2()) cctt_debug (stderr,"Srv  :     Sending temporary buffer (%d bytes) for client %d",options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size,client_number);
	  do_channel(options,client_number,
		     options->srv_options->connect_infos->sd_table[client_number].tmpbuf,
		     options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size);

	  /* 
	  ** Si ce pointeur est positionne, cela signifie qu'une fonction n'a pas pu traiter toutes les donnees
	  ** du buffer (par exemple un client en mode proxy inverse a ete lu en plusieurs read).
	  ** tmpbuf_ptr pointe vers les donnees a replacer dans le buffer
	  ** tmpbuf_size contient la longueur de ces donnees
	  */
	  if (options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr != NULL)
	    {
	      memcpy(options->srv_options->connect_infos->sd_table[client_number].tmpbuf,
		      options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr,
		      options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size);
	      memset(options->srv_options->connect_infos->sd_table[client_number].tmpbuf+options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size,0,ACTION_BUFFER_SIZE-options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size);
	      /* On n'initialise surtout pas tmpbuf_size !!! par contre on reset tmpbuf_ptr */
	      free (options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr);
	      options->srv_options->connect_infos->sd_table[client_number].tmpbuf_ptr = NULL;
	    }
	  else
	    {
	      /* Tout a ete correctement emis => On vide le buffer et on reinitialise */
	  memset(options->srv_options->connect_infos->sd_table[client_number].tmpbuf,0,ACTION_BUFFER_SIZE);
	  options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size = 0;
	    }
	  
	  if (nsocket < ACTION_BUFFER_SIZE)
	    {
	      /* On a la place d'ajouter dans le buffer */

	      memcpy(options->srv_options->connect_infos->sd_table[client_number].tmpbuf+options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size,buffer,nsocket);
	      options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size += nsocket;

	      if (OPT_DEBUG2()) cctt_debug (stderr,"Srv  :   Storing in temporary buffer (%d bytes (total %d)) for client %d",nsocket,options->srv_options->connect_infos->sd_table[client_number].tmpbuf_size,client_number);
	    }
	  else
	    {
	      /* 
	      ** On a pas la place de stocker dans le buffer temporaire ?!?! 
	      ** On se casse...
	      */
	      cctt_error(stderr,"[Server] : (do_server.c) Client sended %d bytes and this doesn't fit in the tmp buffer\n",nsocket);
	      reset_client_connection(options,client_number);
	    }
	}
    }
  else /* Il y a eu un probleme sur la socket... On close */
    {
      if (nsocket == 0) /* Le client a ferme la socket, on le retire du pool */
	{
	  /* 
	  ** On a ici un probleme complique...
	  ** On ne peut pas retirer le client du pool des que la socket tombe car
	  ** si un autre client se presente (http_post par exemple) avec le magic number
	  ** correspondant et tente d'obtenir l'ancien contexte, il ne trouvera rien...
	  ** Donc a priori, on ne retire le client qu'au bout d'un certain temps...
	  */
	  if (tell_me_about_encoding(options) == 2)
	    set_reset_client_connection(options,client_number);
	  else
	reset_client_connection(options,client_number);
	}
      /* Affichage du prompt serveur */
      show_cctt_prompt_server();
    }
}

/*
** Cette fonction permet de manager les clients UDP
*/

void srv_manage_client_udp(S_OPTIONS *options)
{
  int cpt=0,max;
  char buffer[SRV_SOCKET_BUFFER_SIZE+1];
  int nsocket,sock_size;
  struct sockaddr_in sockaddr;
  int contin=0, error=0;

  max = options->srv_options->connect_infos->sdtablesize;
  memset(buffer,0,SRV_SOCKET_BUFFER_SIZE+1);

  sock_size = sizeof(struct sockaddr_in);
  memset(&sockaddr,0,sock_size);

  /*
  ** On lance les donnees dans la socket sur ce call.
  ** Les process geres par un manager font le read eux meme.
  */

  if ((nsocket = recvfrom(options->srv_options->sd,buffer,SRV_SOCKET_BUFFER_SIZE,MSG_OOB|MSG_PEEK,
			  (struct sockaddr *) &sockaddr,
			  &sock_size) < 0))
    {
      cctt_error(stderr,"%s","\n[Server] : Client does not respond. Kill the connection manually !!!\n");
      cctt_debug(stderr,"%s","srv_manage_client_udp function error");
      error = 1;
    }

  /*
  ** Recherche si le client est deja enregistre
  */
  
  if (error == 0)
    for (cpt=3;cpt<max;cpt++)
      if (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED)
	if (memcmp(&sockaddr,&options->srv_options->connect_infos->sd_table[cpt].sockaddr,
		   sizeof(options->srv_options->connect_infos->sd_table[cpt].sockaddr)) == 0)
	  {
	    options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec = 0;
	    contin = 1;
	    break;
	  }

  /*
  ** Sinon, enregistre le client
  */

  if (contin == 0 && error == 0)
    for (cpt=3;cpt<max;cpt++)
      {
	/* On cherche une case libre qui nous servira comme numero de client */
	if (!(options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED))
	  {
	    /* On enregistre la connexion */
	    options->srv_options->connect_infos->sd_table[cpt].used = SRV_SD_USED;
	    /* Client pas encore identifie */
	    options->srv_options->connect_infos->sd_table[cpt].ident = SRV_CL_IDENT_NO;
	    /* Client n'a pas encore fait sa demande de service */
	    options->srv_options->connect_infos->sd_table[cpt].tos = 0;
	    /* Client n'a pas encore de manager */
	    options->srv_options->connect_infos->sd_table[cpt].pid_shell = 0;
	    /* Pas d'autorisation proxy */
	    options->srv_options->connect_infos->sd_table[cpt].proxy_auth = SRV_CL_PROX_NO;
	    /* Compteur de client */
	    options->srv_options->connect_infos->num_sd_connected += 1;
	    /* Adresse client */
	    memcpy(&options->srv_options->connect_infos->sd_table[cpt].sockaddr,
		   &sockaddr,sizeof(sockaddr));
	    /* Structure de temps */
	    options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec = 0;
	    options->srv_options->connect_infos->sd_table[cpt].tvu.tv_usec = 0;
	    options->srv_options->connect_infos->sd_table[cpt].tvr.tv_usec = 0;
	    /* Initialisation position alphabet pour basic_decode() */
	    options->srv_options->connect_infos->sd_table[cpt].basdec_alph_pos = 0;
	    break;
	  }
      }

  /*
  ** Prend en compte le paquet
  */
  if (error == 0)
    {
      if (SRV_CL_MANAGED(cpt) == 0)
	{
	  /* Si non gere par un manager, sort les donnees de la socket */
	  nsocket = recvfrom(options->srv_options->sd,buffer,SRV_SOCKET_BUFFER_SIZE,MSG_OOB,
			     (struct sockaddr *) &sockaddr,
			     &sock_size);
	  do_channel(options,cpt,buffer,nsocket);
	}
    }
}

/*
** Fonction d'initialisation des set pour le select
*/

void init_srv_set(S_OPTIONS *options)
{
  int cpt,max;

  max = options->srv_options->connect_infos->sdtablesize;

  FD_ZERO(&(options->srv_options->connect_infos->sread));

  /* 
  ** On ajoute l'entree standard, mais on ne positionne pas le 
  ** flag SRV_SD_USED... stdin est gere autrement.
  ** Idem pour la socket principal
  */
  /*
  ** En mode proxy only, l'acces a STDIN est supprime
  */
  if (!OPT_SRV_PROXY_ONLY())
    FD_SET(0,&(options->srv_options->connect_infos->sread));
  FD_SET(options->srv_options->sd,&(options->srv_options->connect_infos->sread));

  /* Si on est en TCP, on ajoute tous les sd marques utilises */
  if (OPT_SRV_TCP())
    for (cpt=0;cpt<max;cpt++)
      if (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED)
	{
	  FD_SET(cpt,&(options->srv_options->connect_infos->sread));
	  /* Le fd le plus haut + 1 pour le select */
	  options->srv_options->connect_infos->select_var = cpt + 1;
	}
}

/*
** Fonction de recherche de sd actualises par le select
*/

#define SRV_CMD_BUFFER_SIZE 1024

void check_sd_and_exec(S_OPTIONS *options)
{
  int new_sd;
  struct sockaddr_in sockaddr;
  int sock_size;
  char buffer[SRV_CMD_BUFFER_SIZE+1];
  int nshell;
  int cpt,max;

  max = options->srv_options->connect_infos->sdtablesize;

  if (FD_ISSET(0,&(options->srv_options->connect_infos->sread)))
    {
      /* Stdin a ete activee */
      memset(buffer,0,SRV_CMD_BUFFER_SIZE+1);
      nshell = read(0,buffer,SRV_CMD_BUFFER_SIZE);
      if (nshell > 0)
	parse_cmd(options,buffer);
      /* Affichage du prompt serveur */
      show_cctt_prompt_server();
    }
  else
    {
      if (FD_ISSET(options->srv_options->sd,&(options->srv_options->connect_infos->sread)))
	{
	  /* Socket principale activee */
	  if (OPT_SRV_TCP()) /* TCP */
	    {
	      sock_size = sizeof(struct sockaddr);
	      if ((new_sd = accept(options->srv_options->sd,(struct sockaddr *)&sockaddr,&sock_size)) != -1)
		{
		  /* On enregistre la connexion */
		  options->srv_options->connect_infos->sd_table[new_sd].used = SRV_SD_USED;
		  memset(&(options->srv_options->connect_infos->sd_table[new_sd].sockaddr),0,sizeof(struct sockaddr_in));
		  memcpy(&(options->srv_options->connect_infos->sd_table[new_sd].sockaddr),
			 &(sockaddr),
			 sizeof(sockaddr));
		  
		  /* Client pas encore identifie */
		  options->srv_options->connect_infos->sd_table[new_sd].ident = SRV_CL_IDENT_NO;
		  /* Client n'a pas encore fait sa demande de service */
		  options->srv_options->connect_infos->sd_table[new_sd].tos = 0;
		  /* Client n'a pas encore de manager */
		  options->srv_options->connect_infos->sd_table[new_sd].pid_shell = 0;
		  /* Pas d'autorisation proxy */
		  options->srv_options->connect_infos->sd_table[new_sd].proxy_auth = SRV_CL_PROX_NO;
		  /* Donc pas de sd associe */
		  options->srv_options->connect_infos->sd_table[new_sd].cl_req_sd = 0;
		  /* Donc pas de reverse proxy mode */
		  options->srv_options->connect_infos->sd_table[new_sd].rev_prox_mode = 0;
		  /* Compteur de client */
		  options->srv_options->connect_infos->num_sd_connected += 1;
		  /* Structure de temps */
		  options->srv_options->connect_infos->sd_table[new_sd].tvq.tv_usec = 0;
		  options->srv_options->connect_infos->sd_table[new_sd].tvu.tv_usec = 0;
		  options->srv_options->connect_infos->sd_table[new_sd].tvr.tv_usec = 0;
		  /* Initialisation position alphabet pour basic_decode() */
		  options->srv_options->connect_infos->sd_table[new_sd].basdec_alph_pos = 0;
		  /* Vidage et initialisation buffer temporaire */
		  memset(options->srv_options->connect_infos->sd_table[new_sd].tmpbuf,0,ACTION_BUFFER_SIZE);
		  options->srv_options->connect_infos->sd_table[new_sd].tmpbuf_size = 0;
		  cctt_verbose(stderr,"Client connection from %s\n",inet_ntoa(sockaddr.sin_addr));
		      show_cctt_prompt_server();
		    }
		}
	  else /* UDP */
	    {
	      srv_manage_client_udp(options);
	    }
	}
	  /* 
      ** Peut etre que le select nous indique qu'un client a parle sur l'une des sockets supervisees
	  ** On parvient ici pour une socket persistante en TCP.
	  */
	  for (cpt=0;cpt<max;cpt++)
	    {
	      /*
	      ** On parse la liste des sockets supervises :
	      ** $ On verifie que le sd a ete indique en utilise
	      ** $ On verifie ensuite que ce sd n'est pas deja monitore par un manager
	      */
	      if ((options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED)
		  && FD_ISSET(cpt,&(options->srv_options->connect_infos->sread)))
		{
	      if (SRV_CL_CLOSED_FLAG(cpt) != 1)
		{
		  /* On en profite pour mettre a jour la structure de temps */
		  options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec = 0;
		  if (SRV_CL_MANAGED(cpt) == 0)
		    srv_manage_client(options,cpt);
		}
	    }
	}
    }
}

/*
** Station de supervision des managers (processus fils crees par le serveur )
*/

void check_managers(S_OPTIONS *options)
{
  int pid,test; /* Pour le wait3 */
  int cpt,max;

  max = options->srv_options->connect_infos->sdtablesize;

  if ((pid = wait3(&test,WNOHANG,NULL)) > 0)
    {
      if (test == 0) /* Un manager vient de quitter */
	{
	  for (cpt=0;cpt<max;cpt++) /* On parse la liste d'informations des clients */
	    if (options->srv_options->connect_infos->sd_table[cpt].pid_shell == pid)
	      {
		/* C'est le manager de ce client la qui a quitte => on kill */
		cctt_verbose(stderr,"Shell forked pid '%d' was killed .\n",pid);
		reset_client_connection(options,cpt);
	      }
	}
    }
}

/*
** Supervision des connexions proxyied
** On verifie ici si les extremites ont qqc a dire
** Attention, cette fonction n'est pas utilisee pour le proxy_mode inverse !!!
*/

void check_proxy_connections(S_OPTIONS *options)
{
  int ret_sel; /* valeur de retour du select */
  int cpt,max,higher=0;
  fd_set sread; 
  struct timeval tv; /* Structure de temps pour le select */

  char buffer[SRV_CMD_BUFFER_SIZE+1];
  int napplprox,nwrite;

  max = options->srv_options->connect_infos->sdtablesize;
  tv.tv_sec = 0;
  tv.tv_usec = 100;

  FD_ZERO(&(sread));
  for (cpt=0;cpt<max;cpt++)
    if (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED &&
	options->srv_options->connect_infos->sd_table[cpt].tos == CL_IDENT_PROX_MOD &&
	options->srv_options->connect_infos->sd_table[cpt].proxy_auth != SRV_CL_PROX_REV &&
	SRV_CL_CLOSED_FLAG(cpt) != 1)
      {
	FD_SET(options->srv_options->connect_infos->sd_table[cpt].cl_req_sd,&(sread));
	higher = options->srv_options->connect_infos->sd_table[cpt].cl_req_sd + 1;
      }
  if ((ret_sel = select(higher,&(sread),NULL,NULL,&tv)) < 0)
    perror("select");
  else
    {
      for (cpt=0;cpt<max;cpt++)
	if (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED && 
	    options->srv_options->connect_infos->sd_table[cpt].tos == CL_IDENT_PROX_MOD)
	  {
	    if (FD_ISSET(options->srv_options->connect_infos->sd_table[cpt].cl_req_sd,&(sread)))
	      {
		/* C'est ce client qui a parle */
		/* On met sa structure de temps a zero */
		options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec = 0;
		options->srv_options->connect_infos->sd_table[cpt].tvu.tv_usec += 100;
		options->srv_options->connect_infos->sd_table[cpt].tvr.tv_usec += 100;
		memset(buffer,0,SRV_CMD_BUFFER_SIZE+1);
		napplprox = read(options->srv_options->connect_infos->sd_table[cpt].cl_req_sd,buffer,SRV_CMD_BUFFER_SIZE);
		if (OPT_DEBUG2()) cctt_debug(stderr,"Srv  : A lu %d octets de la derniere application sur le sd %d pour le client %d",napplprox,options->srv_options->connect_infos->sd_table[cpt].cl_req_sd,cpt);
		if (napplprox > 0)
		  {
		    /* L'application parle ... On envoie sur le sd concerne */
		    nwrite = do_server_has_something_to_encode(options,cpt,buffer,napplprox);
		    if (OPT_DEBUG2()) cctt_debug(stderr,"Srv  : A ecrit %d octets au client %d",nwrite,cpt);

		  }
		else
		  {
		    /* L 'application est down, on kille tout */
		    cctt_debug(stderr,"PMod : Serveur : La connexion locale sur le sd %d a ete fermee",options->srv_options->connect_infos->sd_table[cpt].cl_req_sd);
		    reset_client_connection(options,cpt);
		  }
	      }
	    else
	      {
		/* Ce client existe mais n'a pas parle */
		/* On met sa structure de temps a jour */
		options->srv_options->connect_infos->sd_table[cpt].tvu.tv_usec += 100;
		options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec += 100;
		options->srv_options->connect_infos->sd_table[cpt].tvr.tv_usec += 100;
	      }
	  }
    }
}

/*
** Plusieurs fonctions de verification/maj des informations clientes...
*/

void check_if_clients_wait(S_OPTIONS *options)
{
  int cpt,max;

  if (OPT_SRV_TCP())
    {
      max = options->srv_options->connect_infos->sdtablesize;
      for (cpt=0;cpt<max;cpt++)
	if ((options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED))
	  {
	    /* 
	    ** Verifie si les clients ont ecoule leur temp d'attente avant emission du
	    ** buffer temporaire.
	    */
	    if (SRV_CL_CLOSED_FLAG(cpt) != 1)
	    srv_manage_client_check_and_go(options, cpt);
	    /*
	    ** Met a jour les structures de temps pour tous les sds utilises,
	    ** ssi ce n'est pas du mode proxy (gere dans check_proxy_connections())
	    */
	    if (options->srv_options->connect_infos->sd_table[cpt].tos != CL_IDENT_PROX_MOD)
	      {
		options->srv_options->connect_infos->sd_table[cpt].tvu.tv_usec += 100;
		options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec += 100;
		options->srv_options->connect_infos->sd_table[cpt].tvr.tv_usec += 100;
	      }
	  }
    }
}

void check_if_srv_kill_connections(S_OPTIONS *options)
{
  int cpt,max;

  max = options->srv_options->connect_infos->sdtablesize;
  for (cpt=0;cpt<max;cpt++)
    if ((options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED))
      {
	if (options->srv_options->connect_infos->sd_table[cpt].close_flag == 1 && 
	    options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec >
	    options->srv_options->kill_quiet_del_cf)
	  {
	    cctt_verbose(stderr,"[Srv] : Automatic close_flag quiet killing of %d",cpt);
	    reset_client_connection(options,cpt);
	  }
	if (options->srv_options->connect_infos->sd_table[cpt].close_flag == 0 && 
	    options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec >
	    options->srv_options->kill_quiet_del)
	  {
	    cctt_verbose(stderr,"[Srv] : Automatic quiet killing of %d",cpt);
	    reset_client_connection(options,cpt);
	  }
      }
}


void do_server(S_OPTIONS *options)
{
  int cpt;
  int sdtablesize;
  int ret_sel; /* valeur de retour du select */
  struct timeval tv; /* Structure de temps pour le select */

  /* Allocation memoire pour la structure d'infos */
  options->srv_options->connect_infos = 
    (T_connect_infos *) malloc (sizeof(T_connect_infos));

  /* Allocation memoire pour les elements et recuperation nb_max de sd */
  sdtablesize = getdtablesize();
  options->srv_options->connect_infos->sdtablesize = sdtablesize;
  options->srv_options->connect_infos->sd_table = 
    (T_connect_sd_table *) malloc (sizeof(T_connect_sd_table)* 
				   (sdtablesize));

  cctt_debug(stderr,"Srv  : %d socket descriptors sont prevus",sdtablesize);
  
  /* Initialisation des elements d'infos de connexion */
  for (cpt=0;cpt<sdtablesize;cpt++)
    options->srv_options->connect_infos->sd_table[cpt].used = SRV_SD_FREE;

  /* Mise a zero du nombre de client connectes */
  options->srv_options->connect_infos->num_sd_connected = 0;
  
  /* Preparation de la socket principale */
  server_socket(options);

  /* Suppression des privileges */
  drop_priviledges(options);

  /*
  ** Si l'appel a socket a reussi,
  ** sinon on boucle jusqu'a ce que ca marche ?
  */
  if (OPT_SRV_CONNECTED())
    {
      cctt_debug(stderr,"Srv  : Listen sur le sd %d",options->srv_options->sd);
      /* Le fd le plus haut + 1 pour le select */
      options->srv_options->connect_infos->select_var = options->srv_options->sd + 1;

      /* Affichage du prompt serveur */
      show_cctt_prompt_server();

      while (1)
	{
	  /* On initialise le set du select */
	  init_srv_set(options);

	  /* On met a jour la structure de temps (0,1 sec) */
	  tv.tv_sec = 0;
	  tv.tv_usec = 100;

	  if ((ret_sel = select(options->srv_options->connect_infos->select_var,&(options->srv_options->connect_infos->sread),NULL,NULL,&tv)) < 0)
	    {
	      perror ("select");
	    }
	  else
	    {
	      /* On verifie si des sd ont ete actualises par le select */
	      check_sd_and_exec(options);
	      
	      /* On verifie les managers */
	      check_managers(options);

	      /* On verifie les connexions de proxy */
	      check_proxy_connections(options);

	      /* On verifie si des clients doivent emettre le buffer temporaire */
	      check_if_clients_wait(options);

	      /* On verifie si des connections quiet depuis un certain temps doivent etre tuees */
	      check_if_srv_kill_connections(options);
	    }
	}
  server_socket_close(options);
    }
}
