/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: do_server_deencode.c,v 1.6 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Ces fonctions ont pour but de coder/decoder le flux entre serveur et client
** Si aucun codage n'est requis, on se contente d'ecrire le buffer
*/

int do_server_has_something_to_encode(S_OPTIONS *options, int sd, char *buffer,int nbytes)
{
  int nwrite=0,done=0;
  char *tmp_encoding;

  if (tell_me_about_encoding(options) == 1)
    {
      /* On est en mode basic_encoding */
      tmp_encoding = basic_encode(buffer,
				  be_get_key_length(options->ident_key),
				  CCTT_BASIC_ENCODING_ALPHABET,
				  nbytes);
      if (OPT_DEBUG2()) cctt_debug(stderr,"\n  Encoding 1 ==>Send %d bytes\n    ===>%s\n",strlen(tmp_encoding),tmp_encoding);

      nwrite = sendto ((OPT_SRV_UDP()?options->srv_options->sd:sd),
		       tmp_encoding,strlen(tmp_encoding),0,
		       (const struct sockaddr *) &options->srv_options->connect_infos->sd_table[sd].sockaddr,
		       sizeof(options->srv_options->connect_infos->sd_table[sd].sockaddr));
      free (tmp_encoding);
      done = 1;
    }

  if (tell_me_about_encoding(options) == 2)
    {
      /* On est dans un mode HTTP */
      nwrite = build_and_send_http_post_response(options,sd,buffer,nbytes);
      done = 1;
    }

  if (done == 0)
    {
      nwrite = sendto ((OPT_SRV_UDP()?options->srv_options->sd:sd),
		       buffer,nbytes,0,
		       (const struct sockaddr *) &options->srv_options->connect_infos->sd_table[sd].sockaddr,
		       sizeof(options->srv_options->connect_infos->sd_table[sd].sockaddr));
    }  
  return (nwrite);
}

int do_server_has_something_to_decode(S_OPTIONS *options, int sd, char *buffer,int nbytes)
{
  int nwrite=0,done=0;
  T_basic_decode basdec;
  char *return_to_signed;

  if (tell_me_about_encoding(options) == 1)
    {
      /* On est en mode basic encoding */
      if (OPT_DEBUG2()) cctt_debug(stderr,"\n  Encoding 1 ===>Received %d bytes\n    ===>%s\n",nbytes,buffer);
      basdec = basic_decode(buffer,
			    be_get_key_length(options->ident_key),
			    CCTT_BASIC_ENCODING_ALPHABET,
			    options,
			    sd);
      return_to_signed = (char *) basdec.decoded;
      nwrite = write(sd,return_to_signed,basdec.size);
      done = 1;
    }

  if (tell_me_about_encoding(options) == 2)
    {
      /* On est dans un mode HTTP */
      basdec = srv_decode_http_post(options,sd,buffer,nbytes);
      /* Si on doit supprimer un top padding */
      if (options->srv_options->http_mode_top_padding_size > 0)
	{
	  if (basdec.size - options->srv_options->http_mode_top_padding_size <= 0)
	    return (nwrite);
	  basdec.decoded = basdec.decoded+options->srv_options->http_mode_top_padding_size;
	  basdec.size -= options->srv_options->http_mode_top_padding_size;
	}
      /* Si on doit supprimer un bottom padding */
      if (options->srv_options->http_mode_bot_padding_size > 0)
	{
	  if (basdec.size - options->srv_options->http_mode_bot_padding_size <= 0)
	    return (nwrite);
	  basdec.size -= options->srv_options->http_mode_bot_padding_size;
	  basdec.decoded[basdec.size] = '\0';
	}
      return_to_signed = (char *) basdec.decoded;
      nwrite = write(sd,return_to_signed,basdec.size);
      done = 1;
    }

  if (done == 0)
    {
      nwrite = write(sd,buffer,nbytes);
    }

  return (nwrite);
}

T_basic_decode do_server_has_something_to_decode_locally(S_OPTIONS *options, int sd, char *buffer,int nbytes)
{
  T_basic_decode basdec;

  if (tell_me_about_encoding(options) == 1)
    {
      /* On est en mode basic encoding */
      if (OPT_DEBUG2()) cctt_debug(stderr,"\n  Encoding 1 ===>Received %d bytes\n    ===>%s\n",nbytes,buffer);
      basdec = basic_decode(buffer,
			    be_get_key_length(options->ident_key),
			    CCTT_BASIC_ENCODING_ALPHABET,
			    options,
			    sd);
      return (basdec);
    }

  if (tell_me_about_encoding(options) == 2)
    {
      /* On est en mode http_post */
      basdec = srv_decode_http_post(options,sd,buffer,nbytes);
      /* Si on doit supprimer un top padding */
      if (options->srv_options->http_mode_top_padding_size > 0)
	{
	  if (basdec.size - options->srv_options->http_mode_top_padding_size <= 0)
	    {
	      basdec.decoded = "";
	      basdec.size = 0;
	      return (basdec);
	    }
	  basdec.decoded = basdec.decoded+options->srv_options->http_mode_top_padding_size;
	  basdec.size -= options->srv_options->http_mode_top_padding_size;
	}
      /* Si on doit supprimer un bottom padding */
      if (options->srv_options->http_mode_bot_padding_size > 0)
	{
	  if (basdec.size - options->srv_options->http_mode_bot_padding_size <= 0)
	    {
	      basdec.decoded = "";
	      basdec.size = 0;
	      return (basdec);
	    }
	  basdec.size -= options->srv_options->http_mode_bot_padding_size;
	  basdec.decoded[basdec.size] = '\0';
	}
      return (basdec);
    }

  basdec.decoded = (char *) malloc(nbytes+1);
  memset(basdec.decoded,0,nbytes+1);
  memcpy(basdec.decoded,buffer,nbytes);
  basdec.size=nbytes;

  return (basdec);
}

