/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: functions.c,v 1.7 2003/06/11 13:14:19 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

void version(void)
{
  fprintf(stderr,VERSION);
}

void synopsis(void)
{
  fprintf(stderr,VERSION);
  fprintf (stderr,"%s","Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com\n");
  fprintf(stderr,
	  "\nSynopsis:\n---------\n"
	  "\n"
	  "  %s -s listen_address  [-p listen_port]  [-f config_file] [-t channel_type] [-l] [-L] [-v] [-T]\n"
	  "  %s -c connect_address [-d connect_port] [-f config_file] [-t channel_type] [-r | -a | -i | -z] [-v] [-T]\n"
	  "  %s -V\n"
	  "  %s -h\n\n",NAME,NAME,NAME,NAME);
  fprintf(stderr,
	  "Parameters:\n-----------\n"
	  "\n"
	  "  %s\t--version\t\t-V\tShow version.\n"
	  "  %s\t--help\t\t\t-h\tShow this help.\n"
	  "  %s\t--debug\t\t\t-D\tShow debug messages (for developpers).\n"
	  "\n  Server :\n"
	  "  %s\t--listen_address\t-s\tSet the listenning IP address.\n"
	  "  %s\t--listen_port\t\t-p\tSet the listenning port (default is tcp 4242).\n"
	  "  %s\t--logging\t\t-l\tLogging mode to file (default is cctt_srv.log).\n"
	  "  %s\t--logging-syslog\t-L\tLogging mode to syslog (default is daemon.notice).\n"
	  "\n  Client :\n"
	  "  %s\t--connect_address\t-c\tSet the server destination IP address.\n"
	  "  %s\t--connect_port\t\t-d\tSet the server destination port (default is tcp 4242).\n"
	  "  %s\t--rshell\t\t-r\tClient has to bind a reverse shell.\n"
	  "  %s\t--list_proxy_mode\t-i\tGet the list of server proxy_mode.\n"
	  "  %s\t--proxy_mode\t\t-a\tClient has to work in proxy_mode.\n"
	  "  %s\t--reverse_proxy_mode\t-z\tClient has to work in reverse_proxy_mode.\n"
	  "\n  General :\n"
	  "  %s\t--channel_type\t\t-t\tSet the channel type (default is socket).\n"
	  "  %s\t--config_file\t\t-f\tConfiguration file (default are cctt_srv.cf and cctt_cl.cf).\n"
	  "  %s\t--verbose\t\t-v\tRun in verbose mode.\n"
	  "  %s\t--check_conf_file\t-T\tCheck configuration file syntax and exit.\n"
	  "\n",
	  NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME,NAME);
  fprintf(stderr,
	  "Channel types:\n--------------\n"
	  "\n"
	  "  socket\t\t\tChannel is a standard socket from client to server.\n"
	  "  socket_encode\t\t\tChannel type is 'socket' and data flow is 'encoded'.\n"
	  "  socket_http_proxy\t\tChannel is a socket and use the HTTP CONNECT proxy method.\n"
	  "  socket_http_proxy_encode\tChannel type is 'socket_http_proxy' and data flow is 'encoded'.\n"
	  "  client_only_with_http_proxy\tChannel type uses 'socket_http_proxy' but doesn't need a Cctt server.\n"
	  "  http_post\t\t\tChannel type uses HTTP protocol with POST requests.\n"
	  "  http_post_proxy\t\tChannel type uses HTTP protocol with POST requests and next hop is an HTTP proxy.\n"
	  "  http_post_proxy2\t\tChannel type uses HTTP protocol with POST requests and next hop is an HTTP proxy (CONNECT).\n"
	  "  test\t\t\t\tChannel is something in development.\n"
	  "\n");
  fprintf(stderr,
	  "Licence:\n--------\n\n"
	  "This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.\n\n"
	  "This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.\n\n"
	  "You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n"
	  "\n");
  exit(0);
}

char *chomp(char *buffer)
{
  if (strlen(buffer) > 0)
    buffer[strlen(buffer)-1] = '\0';
  return (buffer);
}

/*
** Renvoie le nombre de caracteres present dans un int
*/

int nb_char_in_int(int number)
{
  int div=10,res=0;

  if (number == 0)
    return (1);
  while (number != 0)
    {
      number /= div;
      res +=1;
    }
  return (res);
}

/*
** Renvoie la chaine correspondant a un int.
*/

char *get_str_from_int(int number)
{
  char *res;
  int size;
  int div=10,cpt;

  if (!number && number != 0)
    return (NULL);
  size = nb_char_in_int(number);

  res = (char *) malloc (size+1);
  memset(res,0,size+1);
  
  if (number == 0)
    {
      *res = '0';
      return (res);
    }
  for (cpt=size-1;cpt>-1&&number!=0;cpt--)
    {
      res[cpt] = ((number%div) + 48);
      number /= div;
    }
  return (res);
}

/*
** Renvoie une priorite syslog a partir du nom lui correspondant
*/

int get_syslog_priority(char *name)
{
  if (strncmp(name,"alert",strlen(name)) == 0)
    return (LOG_ALERT);
  if (strncmp(name,"crit",strlen(name)) == 0)
    return (LOG_CRIT);
  if (strncmp(name,"LOG_debug",strlen(name)) == 0)
    return (LOG_DEBUG);
  if (strncmp(name,"emerg",strlen(name)) == 0)
    return (LOG_EMERG);
  if (strncmp(name,"err",strlen(name)) == 0)
    return (LOG_ERR);
  if (strncmp(name,"info",strlen(name)) == 0)
    return (LOG_INFO);
  if (strncmp(name,"notice",strlen(name)) == 0)
    return (LOG_NOTICE);
  if (strncmp(name,"warning",strlen(name)) == 0)
    return (LOG_WARNING);
  return (LOG_NOTICE);
}

/*
** Renvoie une facilite syslog a partir du nom lui correspondant
*/

int get_syslog_facility(char *name)
{
  if (strncmp(name,"auth",strlen(name)) == 0)
    return (LOG_AUTH);
  if (strncmp(name,"authpriv",strlen(name)) == 0)
    return (LOG_AUTHPRIV);
  if (strncmp(name,"cron",strlen(name)) == 0)
    return (LOG_CRON);
  if (strncmp(name,"daemon",strlen(name)) == 0)
    return (LOG_DAEMON);
  if (strncmp(name,"kern",strlen(name)) == 0)
    return (LOG_KERN);
  if (strncmp(name,"security",strlen(name)) == 0)
    return (LOG_AUTH);
  if (strncmp(name,"syslog",strlen(name)) == 0)
    return (LOG_SYSLOG);
  if (strncmp(name,"user",strlen(name)) == 0)
    return (LOG_USER);
  if (strncmp(name,"local0",strlen(name)) == 0)
    return (LOG_LOCAL0);
  if (strncmp(name,"local1",strlen(name)) == 0)
    return (LOG_LOCAL1);
  if (strncmp(name,"local2",strlen(name)) == 0)
    return (LOG_LOCAL2);
  if (strncmp(name,"local3",strlen(name)) == 0)
    return (LOG_LOCAL3);
  if (strncmp(name,"local4",strlen(name)) == 0)
    return (LOG_LOCAL4);
  if (strncmp(name,"local5",strlen(name)) == 0)
    return (LOG_LOCAL5);
  if (strncmp(name,"local6",strlen(name)) == 0)
    return (LOG_LOCAL6);
  if (strncmp(name,"local7",strlen(name)) == 0)
    return (LOG_LOCAL7);
  return(LOG_DAEMON);
}

/*
** Enregistre une session reverse-shell dans un fichier de log
*/

void log_reverse_session(int clnum, char *msg, int way)
{
  FILE *logstream;
  time_t ccttime;

  if (OPT_LOGGING())
    {
      if ((logstream = fopen(DEFAULT_LOG_FILE,"a+")) == NULL)
	cctt_error(stderr,"[Server] : Cannot access to %s logfile !!!\n",DEFAULT_LOG_FILE);
      else
	{
	  ccttime = time(NULL);
	  if (way == 0)
	    fprintf(logstream,"[%s] Client %d => ",chomp(ctime(&ccttime)),clnum);
	  else
	    fprintf(logstream,"[%s] Client %d <=\n ",chomp(ctime(&ccttime)),clnum);
	  fprintf(logstream,"%s\n",msg);
	  fclose (logstream);
	}
    }
}

/*
** Envoie une fausse banniere a un client ayant plante l'identification / la demande de service
*/

int reply_as_fake_webserver(S_OPTIONS *options, int cl_num)
{
  int nw;

  if (options->srv_options->fake_webserver != NULL)
    {
      nw = write(cl_num,options->srv_options->fake_webserver,strlen(options->srv_options->fake_webserver));
      return (nw);
    }
  return (0);
}

/*
** Prend une chaine de caractere et s'assure que les caracteres seront affichables.
** On remplace tous les caracteres non presents dans l'ensemble decimal [32,126] par un '.' (dec 46).
*/

char *ascii_this_string(char *str)
{
  char *bk;

  bk=str;
  while (*str)
    {
      if (*str != '\n' && (*str > 126 || *str < 32))
	*str = 46;
      str++;
    }
  return (bk);
}
