/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: main.c,v 1.9 2003/06/10 18:17:41 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

int main(int ac, char **av)
{
  /*
  ** Activation debug ?
  ** Changez la valeur suivante a 1 ou 2 et lisez doc/fr/LISEZMOI.developpeurs.
  */
  GLOBAL_DEBUG = 0;

  /* 
  ** Allocation memoire 
  */

  options = (S_OPTIONS *) malloc(sizeof(S_OPTIONS));
  memset (options,0,sizeof(S_OPTIONS));

  options->srv_options = (S_SERVER *) malloc(sizeof(S_SERVER));
  memset (options->srv_options,0,sizeof(S_SERVER));

  options->cl_options = (S_CLIENT *) malloc(sizeof(S_CLIENT));
  memset (options->cl_options,0,sizeof(S_CLIENT));

  options->cl_options->http_proxy_chain_addrs = (char **) malloc (sizeof(char **));
  options->cl_options->http_proxy_chain_ports = (char **) malloc (sizeof(char **));
  memset(options->cl_options->http_proxy_chain_addrs,0,sizeof(char **));
  memset(options->cl_options->http_proxy_chain_ports,0,sizeof(char **));

  /*
  ** Verifie les arguments passes dans la requete
  */
  
  /* Remplit la structure d'options en fonction des arguments */
  parse_args(ac, (char **) av, options);

  /*
  ** Ceci est obligatoire pour le respect du flag -D.
  ** On ne passe pas la structure S_OPTIONS aux fonctions d'erreur/debug/etc 
  **   donc OPT_DEBUG() ne peut fonctionner et on ne verifie que GLOBAL_DEBUG...
  */
  if (!OPT_DEBUG())
    GLOBAL_DEBUG = 0;
  cctt_debug(stderr,"%s","Init : Arguments ligne de commande : OK");

  /* Verifie si le traitement va pouvoir etre realise */
  check_args(options);
  cctt_debug(stderr,"%s","Init : Verification des arguments  : OK");

  /*
  ** Verifie le fichier de configuration
  */

  /* remplit la structure d'options en fonction du fichier de configuration */
  parse_conf_file(options);

  /* Verifie que la configuration demandee est correcte */
  check_conf_file(options);

  /*
  ** Verifications de derniere minute  
  */

  /* Le type de canal demande existe ? */
  check_channel(options);

  /* Identification et cle sont elles configurees ? */
  do_ident_exist(options);

  /* Le shell du serveur est il positionne (conf_file_SRV.c) ? */
  check_srv_shell_configuration(options);

  /*
  ** Traitement
  */

  cctt_debug(stderr,"%s","Init : Initialisation terminee avec succes");

  fprintf (stderr,
	   "CCTT v0.1.7, Copyright (C) 2002,2003 Simon Castro\n"
	   "CCTT comes with ABSOLUTELY NO WARRANTY; please, look at COPYING for more informations.\n"
	   "\n");

  /* Verbose ? Affichage des options demandees */
  if (OPT_VERBOSE()) verbose_show_options(options);
  
  /* Si on en est la, c'est que tout c'est bien passe */
  if (OPT_CHK_CONF_FILE())
    {
      fprintf (stdout,"\nConfiguration File : %s - Syntax Ok\n",options->conf_file);
      exit (1);
    }

  cctt_debug(stderr,"%s","Main : C'est parti pour le traitement");

  /* Si on est en mode client, on va chercher la fonction appropriee */
  if (OPT_CL_OR_SRV())
    do_channel(options,-1,NULL,0);
  else /* Sinon, on lance le serveur */
    do_server(options);

  /* Pour le warning en compilation -Wall */
  return(0);
}

