/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: parse_cmd_functions.c,v 1.8 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

void parse_cmd_show_connections(S_OPTIONS *options, char *cmd)
{
  int cpt,max;
  struct sockaddr_in sockaddr;
  T_proxy_allowed_list *tmp_list;
  
  max = options->srv_options->connect_infos->sdtablesize;
  fprintf(stdout,"There is %d clients connected\n",options->srv_options->connect_infos->num_sd_connected);
  
  for (cpt=0;cpt<max;cpt++)
    if (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED)
      {
	sockaddr = options->srv_options->connect_infos->sd_table[cpt].sockaddr;
	fprintf(stdout,"  # Sd(%d - %d) : %s:%d (Quiet : %d ms, Up : %d ms) - close flag : %d\n",
		cpt,options->srv_options->connect_infos->sd_table[cpt].magic_number,
		inet_ntoa(sockaddr.sin_addr),ntohs(sockaddr.sin_port),
		(int) options->srv_options->connect_infos->sd_table[cpt].tvq.tv_usec,
		(int) options->srv_options->connect_infos->sd_table[cpt].tvu.tv_usec,
		options->srv_options->connect_infos->sd_table[cpt].close_flag);
	
	if (options->srv_options->connect_infos->sd_table[cpt].tos == CL_IDENT_ASKED_SHELL)
	  fprintf(stdout,"%s","    -> normal shell");

	if (options->srv_options->connect_infos->sd_table[cpt].tos == CL_IDENT_REV_SHELL)
	  fprintf(stdout,"%s","    -> reverse shell\n");

	if (options->srv_options->connect_infos->sd_table[cpt].tos == CL_IDENT_PROX_MOD)
	  fprintf(stdout,"    -> proxy mode (Remote client is %s:%d) -> %s:%d\n",
		  inet_ntoa(options->srv_options->connect_infos->sd_table[cpt].cl_prox_sockaddr.sin_addr),
		  ntohs(options->srv_options->connect_infos->sd_table[cpt].cl_prox_sockaddr.sin_port),
		  inet_ntoa(options->srv_options->connect_infos->sd_table[cpt].prox_to_sockaddr.sin_addr),
		  ntohs(options->srv_options->connect_infos->sd_table[cpt].prox_to_sockaddr.sin_port));
	
	if (options->srv_options->connect_infos->sd_table[cpt].tos == CL_IDENT_REV_PROX_MOD)
	  {
	    /* On se positionne sur le premier */
	    tmp_list = options->srv_options->proxy_allowed_list->first;
	    while (tmp_list && tmp_list->next != NULL)
	      {
		if (tmp_list->reverse == cpt)
	  fprintf(stdout,"    -> reverse proxy mode to %s:%d\n",
			  inet_ntoa(tmp_list->sockaddr.sin_addr),
			  ntohs(tmp_list->sockaddr.sin_port));
		
		tmp_list = tmp_list->next;
	      }
	    /* On n'oublie pas le dernier */
	    if (tmp_list != NULL)
	      if (tmp_list->reverse == cpt)
		fprintf(stdout,"    -> reverse proxy mode to %s:%d\n",
			inet_ntoa(tmp_list->sockaddr.sin_addr),
			ntohs(tmp_list->sockaddr.sin_port));
	  }

	if (options->srv_options->connect_infos->sd_table[cpt].tos == CL_IDENT_ASKED_SHELL)
	  {
	    if (options->srv_options->connect_infos->sd_table[cpt].pid_shell > 0)
	      fprintf(stdout," (Mgr pid :%d)\n",options->srv_options->connect_infos->sd_table[cpt].pid_shell);
	    else
	      fprintf(stdout,"%s"," (Manager killed)\n");
	  }
      }
}
  
void parse_cmd_show_params(S_OPTIONS *options, char *cmd)
{
  verbose_show_options(options);
}

void parse_cmd_help(S_OPTIONS *options, char *cmd)
{
  version();

  fprintf(stderr,
	  "\n"
	  "Valid commands are :\n"
	  "  help\t\t\t\tdisplay help\n"
	  "  show connections\t\tdisplay current connections\n"
	  "  show params\t\t\tdisplay initialisation parameters\n"
	  "  kill connection X\t\tKill connection on Sd X\n"
	  "  kill manager X\t\tKill manager of Sd X\n"
	  "  tell client X 'something'\tSend 'something' to client on Sd X\n"
	  "  quit\t\t\texit"
	  "\n");
}

void parse_cmd_quit(S_OPTIONS *options, char *cmd)
{
  cctt_verbose(stderr,"%s","Server is exiting.\n");
  exit (0);
}

void parse_cmd_kill_connection(S_OPTIONS *options, char *cmd)
{
  char *ptr;
  int sd=0,i;

  ptr = NULL;
  for (i=0;i<3;i++)
    if (cmd != NULL)
      ptr = strsep(&cmd," ");
  if (ptr != NULL)
    sd = atoi(ptr);
  if (sd > 2 && sd < options->srv_options->connect_infos->sdtablesize &&
      options->srv_options->connect_infos->sd_table[sd].used & SRV_SD_USED)
    {
      if (options->srv_options->connect_infos->sd_table[sd].pid_shell > 2)
	cctt_error(stderr,"[Server] : Manager exists for client %d ! Kill the manager first !\n",sd);
      else
	reset_client_connection(options, sd);
    }
  else
    cctt_error(stderr,"[Server] : Cannot kill connection '%s'.\n",ptr);
}

void parse_cmd_kill_manager(S_OPTIONS *options, char *cmd)
{
  char *ptr;
  int sd=0,i;

  ptr = NULL;
  for (i=0;i<3;i++)
    if (cmd != NULL)
      ptr = strsep(&cmd," ");
  if (ptr != NULL)
    sd = atoi(ptr);
  if (sd > 2 && sd < options->srv_options->connect_infos->sdtablesize &&
      options->srv_options->connect_infos->sd_table[sd].used & SRV_SD_USED &&
      options->srv_options->connect_infos->sd_table[sd].pid_shell > 2)
    {
      write(options->srv_options->connect_infos->sd_table[sd].server_manager[1],"0",1);
      usleep(1000); /* On attend un peu avant de tuer le manager (1 sec) */
      kill(options->srv_options->connect_infos->sd_table[sd].pid_shell,9);
      cctt_verbose(stderr,"Killing manager of process on sd '%d'.\n",sd);
      options->srv_options->connect_infos->sd_table[sd].pid_shell = 0;
    }
  else
    cctt_error(stderr,"[Server] : Cannot kill manager of connection '%s'!\n",ptr);
}

void parse_cmd_tell_client(S_OPTIONS *options, char *cmd)
{
  char *ptr;
  int sd=0,i;

  ptr = NULL;
  for (i=0;i<3;i++)
    if (cmd != NULL)
      ptr = strsep(&cmd," ");
  if (ptr != NULL)
    sd = atoi(ptr);
  if (sd > 2 && sd < options->srv_options->connect_infos->sdtablesize &&
      options->srv_options->connect_infos->sd_table[sd].used & SRV_SD_USED &&
      options->srv_options->connect_infos->sd_table[sd].tos == CL_IDENT_REV_SHELL)
    {
      if (cmd != NULL && strlen(cmd) > 0)
	{
	  log_reverse_session(sd,cmd,0);
	  do_server_has_something_to_encode(options,sd,strcat(cmd,"\n"),strlen(cmd)+1);
	}
      else
	cctt_error(stderr,"[Server] : Want to tell N.o.t.h.i.n.g to client '%d' ?!?\n",sd);
    }
  else
    cctt_error(stderr,"[Server] : Cannot tell anything to client '%s'!\n",ptr);
}
