/*
** CCTT - Covert Channel Tunneling Tool - v0.1.5
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: server_functions.c,v 1.6 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.5 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Sont regroupees ici les fonctions reseau relatives
** au serveur
*/

typedef struct S_linger 
{
  int   l_onoff;    /* linger active */
  int   l_linger;   /* how many seconds to linger for */
} T_linger;


void server_socket(S_OPTIONS *options)
{
  int reuse=1;
  int keepalive=1;
  T_linger linger;

  linger.l_onoff=1;
  linger.l_linger=1;

  if (OPT_SRV_TCP()) /* TCP */
    {
      /* 
      ** On affiche les erreurs mais on ne quitte pas
      ** => Le choix est laisse a la fonction de gestion du serveur
      */
      if ((options->srv_options->sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
	  cctt_error(stderr,"%s","[Server] : Socket creation !\n");
	  options->srv_options->sd = -1;
	}

      /* Socket en KeepAlive */
      setsockopt(options->cl_options->sd,SOL_SOCKET,SO_KEEPALIVE,(int *)&keepalive,sizeof(keepalive));

      /* Socket en REUSEADDR */
      setsockopt(options->srv_options->sd,SOL_SOCKET,SO_REUSEADDR,(int *)&reuse,sizeof(reuse));

      /* Socket en SOLINGER */
      setsockopt(options->srv_options->sd,SOL_SOCKET,SO_LINGER,(T_linger *)&linger,sizeof(linger));

      if (bind(options->srv_options->sd,
	       (struct sockaddr *)&(options->srv_options->sockaddr),
				    sizeof(struct sockaddr))<0)
	{
	  cctt_error(stderr,"%s","[Server] : Socket bind !\n");
	  options->srv_options->sd = -1;
	}
      if ((listen(options->srv_options->sd,SRV_LISTEN_NUM)) != 0)
	{
	  cctt_error(stderr,"%s","[Server] : Socket listen !\n");
	  options->srv_options->sd = -1;
	}
    }
  else /* UDP */
    {
      if ((options->srv_options->sd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
	{
	  cctt_error(stderr,"%s","[Server] : Socket creation !\n");
	  options->srv_options->sd = -1;
	}

      /* Socket en REUSEADDR */
      setsockopt(options->srv_options->sd,SOL_SOCKET,SO_REUSEADDR,(int *)&reuse,sizeof(reuse));

      if (bind(options->srv_options->sd,
	       (struct sockaddr *)&(options->srv_options->sockaddr),
				    sizeof(struct sockaddr))<0)
	{
	  cctt_error(stderr,"%s","[Server] : Socket bind !\n");
	  options->srv_options->sd = -1;
	}
    }
}

void server_socket_close(S_OPTIONS *options)
{
  close (options->srv_options->sd);
}

/*
** Ferme une connexion cliente
*/

void reset_client_connection(S_OPTIONS *options, int sd)
{
  /* 
  ** Le close (sd) ne fonctionne pas ?!?!? 
  ** => Quand arrivent deux clients.
  ** Le client n'est ferme que quand le second l'est aussi ?!?!?
  */
  if (OPT_SRV_TCP())
    {
      shutdown(sd,2);
      /* 
      ** Si ce client etait affecte a une connexion en mode proxy ET QUE
      ** on est pas en mode HTTP !!!
      */
      if (tell_me_about_encoding(options) != 2 || 
	  (tell_me_about_encoding(options) == 2 && 
	   options->srv_options->connect_infos->sd_table[sd].close_flag == 1))
      if (options->srv_options->connect_infos->sd_table[sd].proxy_auth == SRV_CL_PROX_YES)
	shutdown(options->srv_options->connect_infos->sd_table[sd].cl_req_sd,2);

      options->srv_options->connect_infos->sd_table[sd].used = SRV_SD_FREE;
      options->srv_options->connect_infos->sd_table[sd].ident = SRV_CL_IDENT_NO;
      options->srv_options->connect_infos->sd_table[sd].tos = 0;
      options->srv_options->connect_infos->sd_table[sd].pid_shell = 0;
      options->srv_options->connect_infos->sd_table[sd].proxy_auth = SRV_CL_PROX_NO;
      options->srv_options->connect_infos->sd_table[sd].cl_req_sd = 0;
      options->srv_options->connect_infos->sd_table[sd].rev_prox_mode = 0;
      options->srv_options->connect_infos->sd_table[sd].rev_prox_mode_sd = 0;
      options->srv_options->connect_infos->num_sd_connected -= 1;
      /* 
      ** On supprime les elements relatif a ce client s'il etait en reverse proxy_mode
      ** (voir client_tells_hes_proxying_to.c)
      ** SAUF si on est en mode HTTP !!!
      */
      if (tell_me_about_encoding(options) != 2 ||
	  (tell_me_about_encoding(options) == 2 && 
	   options->srv_options->connect_infos->sd_table[sd].close_flag == 1))
      remove_reversed_proxy_elem(options,sd);
      cctt_verbose(stderr,"Client connection on sd %d closed.\n",sd);
    }
  else
    {
      if (options->srv_options->connect_infos->sd_table[sd].proxy_auth == SRV_CL_PROX_YES)
	shutdown(options->srv_options->connect_infos->sd_table[sd].cl_req_sd,2);
      options->srv_options->connect_infos->sd_table[sd].used = SRV_SD_FREE;
      options->srv_options->connect_infos->sd_table[sd].ident = SRV_CL_IDENT_NO;
      options->srv_options->connect_infos->sd_table[sd].tos = 0;
      options->srv_options->connect_infos->sd_table[sd].pid_shell = 0;
      options->srv_options->connect_infos->sd_table[sd].proxy_auth = SRV_CL_PROX_NO;
      options->srv_options->connect_infos->sd_table[sd].cl_req_sd = 0;
      options->srv_options->connect_infos->num_sd_connected -= 1;
      cctt_verbose(stderr,"Client connection on sd %d closed.\n",sd);
    }
}

/*
** Marque une connexion cliente dans l'etat ferme sans la fermer...
** Cela permet de conserver le contexte au cas ou une autre connexion
** tenterait d'y acceder...
** Une autre fonction est chargee de supprimer ces connexions en attente
** de fermeture au bout d'un certain temps...
*/

void set_reset_client_connection(S_OPTIONS *options, int sd)
{
  options->srv_options->connect_infos->sd_table[sd].close_flag = 1;

}


