/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: server_was_asked_a_shell.c,v 1.6 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Cette fonction gere le service : Offrir un shell au client
*/

void server_was_asked_a_shell(S_OPTIONS *options, int cl_num)
{
  int sec_fils; /* pid du deuxieme fils */
  int test; /* flag de retour wait3 */
  int pid;  /* pour wait3 */
  int fils_father[2]; /* pipe entre fils et pere */
  int father_fils[2];  /* pipe entre pere et fils */
  int i,k,sock_size;

  /* Lecture/Ecriture entre pere et fils */
  int nrshell=0;
  int nrsocket=0;
  char buffer[SOCKET_BUFFER_SIZE+1]; /* La taille du buffer est precisee dans includes/definitions.h */

  /* Set de Fds pour le select */
  fd_set fdset_read;
  int highest_fd;
  struct timeval tv; /* Structure de temps pour le select */

  /* On positionne le flag precisant le type de service */
  options->srv_options->connect_infos->sd_table[cl_num].tos = CL_IDENT_ASKED_SHELL;

  sock_size = sizeof(struct sockaddr_in);

  /* Initialisation des pipes pour la communication entre le manager (premier fils) et le shell (second fils) */
  pipe(fils_father);
  pipe(father_fils);

  /* Initialisation du pipe de communication entre le serveur et le manager */
  pipe(options->srv_options->connect_infos->sd_table[cl_num].server_manager);

  cctt_verbose(stderr,"Client %d asked a shell... Forking server process.\n",cl_num);

  if ((options->srv_options->connect_infos->sd_table[cl_num].pid_shell = fork()) == 0) 
    {
      /*
      ** La, on est dans le premier fils qui va se charger de gerer le shell
      */

      cctt_verbose(stderr,"Client %d asked a shell... Forking Shell process.\n",cl_num);

      if ((sec_fils = fork()) == 0)
	{
	  
	  /*
	  ** La, on est dans le deuxieme fils
	  ** On ferme l'un des cotes du pipe.
	  ** On ferme stdin, stdout et stderr.
	  ** On redirige stdin, stdout et stderr vers les pipe 
	  */
	  close(fils_father[0]);
	  close(father_fils[1]);
	  for (i=0;i<3;i++)
	    close (i);
	  dup2(father_fils[0],0);
	  dup2(fils_father[1],1);
	  dup2(fils_father[1],2);
	  
	  /* On appelle le reverse shell */
	  exec_rshell_server(options);
	}
      else
	{
	  /* 
	  ** La, on est dans le premier fils qui se charge de gerer le shell 
	  */
	  if (sec_fils == -1) /* La, on est en erreur fork */
	    cctt_error(stderr,"%s","[Server] : Fork cannot create another process !\n");
	  else /* La, on est dans le premier fils */
	    {
	      /*
	      ** On ferme les extremites des pipes
	      ** On prepare le Set pour le select
	      */
	      close(fils_father[1]);
	      close(father_fils[0]);
	      FD_ZERO(&fdset_read);
	      FD_SET(fils_father[0],&fdset_read);
	      FD_SET(cl_num,&fdset_read);

	      /* On ferme l'une des extremites du pipe de communication entre server et manager */
	      close(options->srv_options->connect_infos->sd_table[cl_num].server_manager[1]);
	      k = options->srv_options->connect_infos->sd_table[cl_num].server_manager[0];
	      FD_SET(k,&fdset_read);

	      /* Quel est le plus grand Fd a ajouter dans
	      ** le select pour la supervisation ?
	      */
	      highest_fd = (fils_father[0]>cl_num?
			    ( k > fils_father[0] ? k+1 : fils_father[0]+1 ):
			    ( k > cl_num ? k+1 : cl_num+1));
	      /* 
	      ** Tant que le shell est UP
	      */
	      while ((pid = wait3(&test,WNOHANG,NULL)) != sec_fils)
		{
		  /* On met a jour la structure de temps (0,1 sec) */
		  tv.tv_sec = 0;
		  tv.tv_usec = 100;

		  /*
		  ** Si le select retourne une erreur, la fonction retourne
		  ** et l'erreur est geree dans la fonction appelante.
		  */
		  if ((select(highest_fd,&fdset_read,NULL,NULL,&tv)) < 0)
		    {
		    cctt_error(stderr,"[Server] : Client %d ... Select Error !\n",cl_num);
		      kill(sec_fils,1);
		      _exit (0);
		    }
		  else
		    {
		      /*
		      ** Si des donnees se presentent sur le pipe (i.e. : elles
		      ** proviennent du shell), on les lit et on les ecrit
		      ** sur la socket.
		      */
		      if (FD_ISSET(fils_father[0],&fdset_read))
			{
			  memset(buffer,0,SOCKET_BUFFER_SIZE+1);
			  nrshell = read(fils_father[0],buffer,SOCKET_BUFFER_SIZE);
			  if (nrshell > 0)
			    do_server_has_something_to_encode(options,cl_num,buffer,nrshell);
			}
		      /*
		      ** Si des donnees se presentent sur la socket (i.e. : elles
		      ** proviennent du client, on les lit et on les ecrit sur
		      ** le pipe (i.e. : vers le shell).
		      */
		      if (FD_ISSET(cl_num,&fdset_read))
			{
			  memset(buffer,0,SOCKET_BUFFER_SIZE+1);
			  /*
			  ** Celui-ci est necessaire.
			  ** Le Serveur principal possede egalement un select sur le sd du client 
			  ** et met a jour des informations si le select retourne convenablement.
			  ** Il ne faut donc pas que le select du manager retire les donnees
			  ** de la socket en premier.
			  */
			  usleep(5);
			  nrsocket = recvfrom((OPT_SRV_UDP()?options->srv_options->sd:cl_num),
					      buffer,SRV_SOCKET_BUFFER_SIZE,0,
					      (struct sockaddr *) &(options->srv_options->connect_infos->sd_table[cl_num].sockaddr),
					      &sock_size);
			  if (nrsocket > 0)
			    do_server_has_something_to_decode(options,father_fils[1],buffer,nrsocket);
			  else
			    kill(sec_fils,1);
			}
		      /*
		      ** Si des donnees proviennent du serveur, ca ne peut
		      ** etre qu'un exit (?) => on kill le fils
		      */
		      if (FD_ISSET(k,&fdset_read))
			{
			  memset(buffer,0,SOCKET_BUFFER_SIZE+1);
			  nrsocket = read(k,buffer,SOCKET_BUFFER_SIZE);
			  if (nrsocket > 0)
			    kill(sec_fils,1);
			}
		      /*
		      ** On reinitialise le set du select pour eviter que l'appel soit bloquant.
		      */
		      FD_ZERO(&fdset_read);
		      FD_SET(fils_father[0],&fdset_read);
		      FD_SET(cl_num,&fdset_read);
		      FD_SET(k,&fdset_read);
		    }
		}
	      if (test == 0)
		cctt_verbose(stderr,"Client %d asked a shell ... It's done.\n",cl_num);
	      else /* Le shell a plante */
		cctt_error(stderr,"%s","[Server] : Execvp couldn't start the command shell !\n");
	      /* 
	      ** Le Shell est termine, on n a plus besoin de ce processus 
	      */
	      _exit (0);
	    }
	}
    }
  else
    {
      /* La, on est dans le pere */
      if (options->srv_options->connect_infos->sd_table[cl_num].pid_shell == -1) /* La, on est en erreur fork */
	cctt_error(stderr,"%s","[Server] : Fork cannot create another process !\n");

      cctt_verbose(stderr,"Shell forked pid is %d.\n",options->srv_options->connect_infos->sd_table[cl_num].pid_shell);

      /* On ferme une extremite du pipe de com. entre server et manager */
      close(options->srv_options->connect_infos->sd_table[cl_num].server_manager[0]);

      /* On se contente de retourner vers le serveur normal */
    }
}
