/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: srv_build_and_send_http_functions.c,v 1.3 2003/06/11 13:14:19 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Fonctions relatives au fonctionnement HTTP du serveur
*/

/*
** Une fonction debile pour recherche le content-length dont la casse peut
** etre aleatoire selon les proxies...
*/

char *my_strstr_ci(char *source, char *pattern)
{
  char *fst_ptr;
  char *nav_ptr;

  nav_ptr = source;

  while((fst_ptr = index(nav_ptr,*pattern)) != NULL)
    {
      if (strncasecmp(fst_ptr,pattern,strlen(pattern)) == 0)
        return (fst_ptr);
      if (*(fst_ptr+1))
        nav_ptr = fst_ptr+1;
      else
        return (NULL);
    }
  return (NULL);
}

/*
** Cette fonction recoit un buffer avec une requete HTTP post du client
** Elle est chargee de verifier les headers, de renvoyer les donnees.
** !!! ATTENTION !!!
** Il faut remettre dans le buffer de reception les donnees supra-numeraires dans le cas
** ou le read principal du serveur contient plusieurs requetes du client.
**
** Les defines sont dans includes/configuration.h
*/

/*
** Verifie l'URI presente dans une requete HTTP contenant une PREMIERE ligne du type "XXX URI XXX\n"
** Retourne 0 si l'URI n'existe pas ( = une premiere ligne a ete trouvee)
** Retourne -1 si une premiere ligne correcte n'a pas ete trouvee
** Retourne 1 si l'URI est correcte
*/

int srv_match_http_uri(S_OPTIONS *options,char *buffer,int nbytes)
{
  int cpt,pos_cr=0,pos_sp1=0,pos_sp2=0;
  char *tmp;
  int ret=0;

  for (cpt=0 ; cpt<nbytes && pos_cr==0 ; cpt++)
    {
      if (buffer[cpt] == '\n')
	pos_cr = cpt; /* On stoppe au premier \n */
      if (buffer[cpt] == ' ' && pos_sp1 != 0 && pos_sp2 == 0)
	pos_sp2 = cpt;
      if (buffer[cpt] == ' ' && pos_sp1 == 0)
	pos_sp1 = cpt;
    }

  if (pos_cr == 0) /* On n'a pas de premiere ligne dans ce buffer */
    return (-1);
  if (pos_sp1 == 0 || pos_sp2 == 0) /* Pas d'espaces dans la premiere ligne */
    return (-1);

  /* On se prepare notre matcher d'URI */
  tmp = (char *) malloc(pos_sp2-pos_sp1+1);
  memset(tmp,0,pos_sp2-pos_sp1+1);
  strncpy (tmp,buffer+pos_sp1+1,pos_sp2-pos_sp1-1);

  if ((strlen(tmp) == strlen(options->srv_options->http_mode_uri)) &&
      strncmp(tmp,options->srv_options->http_mode_uri,strlen(tmp)) == 0)
    ret=1; /* OK l'URI est bonne */

  if (*tmp)
    free (tmp);

  return (ret);
}

/*
** Verifie si le header d'une requete POST est correct (i.e. on a bien
** un double \n a un endroit du buffer)
** renvoie -1 si aucun \n\n n'a ete trouve
** renvoie 0 si on a juste un header et que nbytes se finit par le deuxieme \n
** renvoie la position suivant le double \n si c'est ok
*/

int srv_check_http_post_header(S_OPTIONS *options,char *buffer,int nbytes)
{
  int cpt,pos=-1;

  for (cpt=0 ; cpt<nbytes && pos==-1 ; cpt++)
    {
      if (buffer[cpt] == '\n'
	  && ((cpt+1 < nbytes) && ((buffer[cpt+1] == '\r') || (buffer[cpt+1] == '\n')))) /* If CRLF */
	{
	  if (cpt+2 < nbytes)
            {
              if ((buffer[cpt+2] == '\n' || buffer[cpt+2] == '\r') && cpt+3 < nbytes)
                pos = cpt+3;
              else
	    pos = cpt+2;
            }
	  else
	    pos = 0;
	}
    }
  return (pos);
}

/*
** Va chercher un content-length dans le header
** renvoie -1 si aucun champ de ce type n'a ete trouve
**   ou si le champ etait incorrect.
** renvoie la valeur du content-length si c'est ok
*/

int srv_get_http_post_content_length(S_OPTIONS *options,char *buffer,int pos_data)
{
  char *pos_cl;
  char *tmp;
  int debut=0,pos_cl1=0,pos_cl2=0;

  /* On se fait une copie de ce qui nous interesse en positionnant un '\0' */
  tmp = (char *) malloc(pos_data+1);
  memset(tmp,0,pos_data+1);
  strncpy(tmp,buffer,pos_data);

  if ((pos_cl = my_strstr_ci(tmp,HTTP_STD_CLENGTH)) == NULL) /* Pas de champ trouve */
    {
      if (*tmp)
	free (tmp);
      return (-1);
    }

  /* On va recuperer la valeur du champ */
  debut = pos_data - strlen(pos_cl);
  for (; debut<pos_data && pos_cl2 == 0; debut++)
    {
      if ((buffer[debut] == '\n' || buffer[debut] == '\r') && pos_cl1 != 0)
	pos_cl2 = debut;
      if (buffer[debut] == ' ' && pos_cl1 == 0)
	pos_cl1 = debut;
    }

  if (*tmp)
    free (tmp);

  /* A t on qqc dans le champ ? */
  if (pos_cl1 == 0 || pos_cl2 == 0 || pos_cl2 - pos_cl1 <= 1)
    return (-1);

  /* Recupere le contenu du champ */
  tmp = (char *) malloc(pos_cl2-pos_cl1+1);
  memset (tmp,0,pos_cl2-pos_cl1+1);
  strncpy(tmp,buffer+pos_cl1+1,pos_cl2-pos_cl1-1);

  /* Verifie qu'on a bien des digits ! */
  for (pos_cl1=0;pos_cl1<strlen(tmp);pos_cl1++)
    if (!isdigit(tmp[pos_cl1])) /* Rate ! */
      {
	if (*tmp)
	  free (tmp);
	return (-1);
      }
  if (*tmp)
    {
      debut = (int) atoi(tmp);
      free (tmp);
      return (debut);
    }
  return (-1);
}

/*
** Tente d'envoyer un fichier correspondant a une fake URI specifiee.
** Renvoie 0 si ca a fonctionne.
** Renvoie -1 sinon.
*/

int send_fake_uri_file(S_OPTIONS *options,char *fake,int sd)
{
  FILE *fake_file;
  int nb_oct;
  char *tmp;

  /*
  ** Remove the first '/' if it exist
  */
  if (*fake == '/')
    fake = fake+1;

  if ((fake_file = fopen(fake,"r")) == NULL)
    {
      cctt_debug(stderr,"[HTTP] -> Out : Cannot open '%s' fake uri file",fake);
      return (-1);
    }
  else
    {
      fseek(fake_file,0,SEEK_END);
      nb_oct = ftell(fake_file);
      if (nb_oct < 1)
	{
	  cctt_debug(stderr,"[HTTP] -> Out : '%s' fake uri file is empty",fake);
	  fclose(fake_file);
	  return (-1);
	}
      fseek(fake_file,0,SEEK_SET);
      tmp = (char *) malloc (nb_oct);
      memset(tmp,0,nb_oct);
      fread(tmp,nb_oct-1,1,fake_file);
      fclose(fake_file);
      build_and_send_http_post_response(options,sd,tmp,nb_oct-1);
    }
  return (0);
}

/*
** Match une fake url dans une requete ne contenant pas notre URI standard
** Renvoie -1 en erreur pour que la suite puisse envoyer l'erreur page.
** Renvoie 0 si une fake url a ete trouvee.
*/

int srv_match_http_fake_urls(S_OPTIONS *options,int sd,char *buffer,int nbytes)
{
  int cpt,pos_cr=0,pos_sp1=0,pos_sp2=0;
  char *tmp;

  /* 
  ** La fonction precedente srv_match_http_uri() nous a dit qu'il y avait
  ** une ligne de type "XXX XXX XXX". On recupere donc le premier et le deuxieme pattern
  */

  for (cpt=0 ; cpt<nbytes && pos_cr==0 ; cpt++)
    {
      if (buffer[cpt] == '\n')
	pos_cr = cpt; /* On stoppe au premier \n */
      if (buffer[cpt] == ' ' && pos_sp1 != 0 && pos_sp2 == 0)
	pos_sp2 = cpt;
      if (buffer[cpt] == ' ' && pos_sp1 == 0)
	pos_sp1 = cpt;
    }
  
  if (pos_sp1 != 3) /* Pas de GET */
    return (-1);
  tmp = (char *) malloc(pos_sp1+1);
  memset(tmp,0,pos_sp1+1);
  strncpy(tmp,buffer,pos_sp1);
  if (strncmp(tmp,"GET",3) != 0) /* Et pas de GET */
    {
      if (*tmp) free (tmp);
      return (-1);
    }

  if (pos_sp2 == pos_sp1+1) /* Pas de 2nd champ */
    return (-1);
  tmp = (char *) malloc(pos_sp2-pos_sp1+1);
  memset(tmp,0,pos_sp2-pos_sp1+1);
  strncpy (tmp,buffer+pos_sp1+1,pos_sp2-pos_sp1-1);

  /* On va chercher si l'URI envoyee correspond a une URI configuree */
  if (options->srv_options->http_mode_fake_url_filename != NULL)
    for (cpt=0;options->srv_options->http_mode_fake_url_filename[cpt] != NULL;cpt++)
      {
	if (strlen(tmp) == strlen(options->srv_options->http_mode_fake_url_filename[cpt]) &&
	    strncmp(tmp,options->srv_options->http_mode_fake_url_filename[cpt],strlen(tmp)) == 0)
	  {
	    cctt_debug(stderr,"[HTTP] -> In : Match a fake Uri '%s'",tmp);
	    /* 
	    ** Si on trouve on le fichier correspondant a cette fake uri
	    ** on l'envoie sinon on retourne -1 et la page d'erreur sera envoyee
	    */
	    pos_cr = send_fake_uri_file(options,tmp,sd);
	    if (*tmp)
	      free (tmp);
	    return (pos_cr);
	  }
      }
  if (*tmp)
    free (tmp);
  return (-1);
}

/*
** Cette fonction gere les "erreurs" et renvoie NULL en structure quand une erreur
**  a ete trouvee.
** !!! ATTENTION !!!
** On ne supprime pas les eventuels top et bottom padding ici pour permettre
** a la reinjection de fonctionner... Ils sont supprimes dans la fonction appelante.
*/

T_basic_decode srv_decode_http_post(S_OPTIONS *options,int sd,char *buffer,int nbytes)
{
  T_basic_decode basdec;
  int res,pos_data,cl;
  int reinject_size;
  int cctt_client=0;

  basdec.decoded = NULL;
  basdec.size = 0;

  if (nbytes == 0)
    {
      cctt_debug(stderr,"HTTP : Wont decode a 0 bytes HTTP post from %d\n",sd);
      return (basdec);
    }

  /* D'abord on verifie si l'URI correspond bien a notre URI de traitement configuree */
  if ((res = srv_match_http_uri(options,buffer,nbytes)) <= 0)
    {
      /* Si res == -1, on n'a pas de premiere ligne correcte */
      if (res == 0) 
	res = srv_match_http_fake_urls(options,sd,buffer,nbytes);
      if (res == -1)
	{
	  cctt_debug(stderr,"HTTP POST : Didn't match URI in client '%d' request ... Sending error page.",sd);
	  build_and_send_http_post_response(options,sd,
					    options->srv_options->http_mode_error_page,
					    strlen(options->srv_options->http_mode_error_page));
	}
      return (basdec);
    }
  /* Ensuite on verifie qu'on a bien un header ET des donnees */
  if ((res = srv_check_http_post_header(options,buffer,nbytes)) <= 0)
    {
      cctt_debug(stderr,"HTTP POST : Couldn't find any correct header for client '%d'.",sd);
      return (basdec);
    }
  pos_data = res;
  /* On va maintenant chercher le content-length pour nos donnees dans le header seulement */
  if ((res = srv_get_http_post_content_length(options,buffer,pos_data)) <= 0)
    {
      cctt_debug(stderr,"HTTP POST : No content-length for client '%d'.",sd);
      return (basdec);
    }
  cl = res;

  /* 
  ** On sait ou commence les donnees et ou elles se terminent. 
  ** => On verifie qu'on a toutes les donnees dans le buffer
  */

  if (OPT_DEBUG2()) cctt_debug(stderr,"[HTTP] -> In : Client %d : Total_size(%d) / Pos_data(%d) / CL(%d).",sd,nbytes,pos_data,cl);

  if (pos_data+cl == nbytes)
    {
      /* On a un buffer compose uniquement de cette requete */
      basdec.decoded = (char *) malloc(cl+1);
      memset(basdec.decoded,0,cl+1);
      memcpy(basdec.decoded,buffer+pos_data,cl);
      basdec.size = cl;
      return (basdec);
    }
  else
    {
      if (pos_data+cl < nbytes)
	{
	  /* 
	  ** Il reste des donnees dans le buffer qu'il faut replacer dans le buffer de read principal
	  **   apres avoir traite la requete courante...
	  ** Ces donnees ne font pas partie de cette requete et commencent a pos_data+cl+1
	  */
	  basdec.decoded = (char *) malloc(cl+1);
	  memset(basdec.decoded,0,cl+1);
	  memcpy(basdec.decoded,buffer+pos_data,cl);
	  basdec.size = cl;

	  /*
	  ** !!! ATTENTION !!! 
	  ** En mode proxy, ces donnees ne doivent pas etre reinjectees dans le buffer correspondant
	  ** au sd principal de cette fonction qui correspond au sd de l'application derriere le 
	  ** serveur CCTT.
	  ** => Il faut les reinjecter dans le buffer du client CCTT qui lui est monitore a intervalle
	  ** regulier !!!
	  */
	  for (res=0;cctt_client == 0 && res<options->srv_options->connect_infos->sdtablesize;res++)
	    if (options->srv_options->connect_infos->sd_table[res].cl_req_sd == sd)
	      cctt_client = res;
	  
	  cctt_client = (cctt_client == 0 ? sd : cctt_client);

	  reinject_size = nbytes-(pos_data+cl);
	  options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_ptr = (char *) malloc(reinject_size+1);
	  memset(options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_ptr,0,reinject_size+1);
	  memcpy(options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_ptr,
		  buffer+pos_data+cl,
		  reinject_size);
	  options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_size = reinject_size;
	  if (OPT_DEBUG2()) cctt_debug(stderr,"Http : Reinjection overhead (%d bytes) dans le buffer client %d.",reinject_size,cctt_client);
	  return (basdec);
	}
      else
	{
	  /*
	  ** La requete comporte un champ content-length > a ce qui est disponible dans le buffer...
	  **
	  */
	  cctt_debug(stderr,"[HTTP] -> In : Client sended a %d bytes CL but there is only %d bytes in the buffer... Doing nothing and reinjecting ...",cl,nbytes);

	  for (res=0;cctt_client == 0 && res<options->srv_options->connect_infos->sdtablesize;res++)
	    if (options->srv_options->connect_infos->sd_table[res].cl_req_sd == sd)
	      cctt_client = res;
	  
	  cctt_client = (cctt_client == 0 ? sd : cctt_client);
	  reinject_size = nbytes;
	  options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_ptr = (char *) malloc(reinject_size+1);
	  memset(options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_ptr,0,reinject_size+1);
	  memcpy(options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_ptr,
		  buffer,reinject_size);
	  options->srv_options->connect_infos->sd_table[cctt_client].tmpbuf_size = reinject_size;
	  if (OPT_DEBUG2()) cctt_debug(stderr,"Http : Reinjection overhead (%d bytes) dans le buffer client %d.",reinject_size,cctt_client);
	  return (basdec);
	}
    }
  return (basdec);
}

/*
** Cree le header d'une reponse HTTP
** Les defines sont dans includes/configuration.h
*/

char *build_http_post_response_header(S_OPTIONS *options, int body_size)
{
  char *header,*ptr;
  int tmp_size;

  /* Setting the response field */
  tmp_size = strlen(HTTP_STD_OK)+2;
  header = (char *) malloc (tmp_size+1);
  ptr = header;
  memset(header,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_OK,strlen(HTTP_STD_OK));
  ptr = ptr+strlen(HTTP_STD_OK);
  *ptr = '\n';

  /* Setting the date field */
  tmp_size = strlen(HTTP_STD_DATE)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_DATE,strlen(HTTP_STD_DATE));
  ptr = ptr+strlen(HTTP_STD_DATE);
  *ptr = '\n';

  /* Setting the Server field */
  tmp_size = strlen(HTTP_STD_SERVER)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_SERVER,strlen(HTTP_STD_SERVER));
  ptr = ptr+strlen(HTTP_STD_SERVER);
  *ptr = '\n';

  /* Setting the Content-Length */
  tmp_size = strlen(HTTP_STD_CLENGTH)+nb_char_in_int(body_size)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_CLENGTH,strlen(HTTP_STD_CLENGTH));
  ptr = ptr+strlen(HTTP_STD_CLENGTH);
  strncpy(ptr,get_str_from_int(body_size),nb_char_in_int(body_size));
  ptr = ptr+nb_char_in_int(body_size);
  *ptr = '\n';

  /* Setting the Content-Type */
  tmp_size = strlen(HTTP_STD_CTYPE_SRV)+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,HTTP_STD_CTYPE_SRV,strlen(HTTP_STD_CTYPE_SRV));
  ptr = ptr+strlen(HTTP_STD_CTYPE_SRV);
  *ptr = '\n';

  /* Setting the Proxy-Connection: Keep-Alive field */
  tmp_size = strlen("Proxy-Connection: Keep-Alive")+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,"Proxy-Connection: Keep-Alive",strlen("Proxy-Connection: Keep-Alive"));
  ptr = ptr+strlen("Proxy-Connection: Keep-Alive");
  *ptr = '\n';

  /* Setting the Pragma: nocache field */
  tmp_size = strlen("Pragma: nocache")+2;
  header = realloc(header,strlen(header)+tmp_size+1);
  ptr = header+strlen(header);
  memset(ptr,0,tmp_size+1);
  strncpy(ptr,"Pragma: nocache",strlen("Pragma: nocache"));
  ptr = ptr+strlen("Pragma: nocache");
  *ptr = '\n';

  /* Don't forget the last '\n' */
  *(++ptr) = '\n';

  return (header);
}

int build_and_send_http_post_response(S_OPTIONS *options, int sd, char *buffer, int size)
{
  char *header,*request;
  int body_size,request_size;
  int nwrite=-1;

  char *rebuilded,*rebuilded2;
  int rebuilded_size=0,rebuilded_size2=0;

  /*
  ** Est-ce qu'un Top padding doit etre ajoute ?
  */
  if (options->cl_options->http_mode_top_padding)
    {
      rebuilded_size = size+strlen(options->cl_options->http_mode_top_padding);
      rebuilded = (char *) malloc(rebuilded_size);
      memcpy(rebuilded,options->cl_options->http_mode_top_padding,
	     strlen(options->cl_options->http_mode_top_padding));
      memcpy(rebuilded+strlen(options->cl_options->http_mode_top_padding),buffer,size);
    }
  else
    {
      rebuilded_size = size;
      rebuilded = buffer;
    }
  /*
  ** Est-ce qu'un Bottom padding doit etre ajoute ?
  */
  if (options->cl_options->http_mode_bottom_padding)
    {
      rebuilded_size2 = rebuilded_size + strlen(options->cl_options->http_mode_bottom_padding);
      rebuilded2 = (char *) malloc(rebuilded_size2);
      memcpy (rebuilded2,rebuilded,rebuilded_size);
      memcpy(rebuilded2+rebuilded_size,options->cl_options->http_mode_bottom_padding,
	     strlen(options->cl_options->http_mode_bottom_padding));
      free (rebuilded);
      rebuilded = rebuilded2;
      rebuilded_size = rebuilded_size2;
    }

  body_size = rebuilded_size;
  buffer = rebuilded;

  /* Construit le header */
  header = build_http_post_response_header(options,body_size);
  request_size = strlen(header)+body_size;

  /* Construit la requete */
  request = (char *) malloc(request_size+1);
  memset(request,0,request_size+1);
  strncpy(request,header,strlen(header));
  memcpy(request+strlen(header),buffer,body_size);
  
  cctt_debug(stderr,"[HTTP] -> Out : A %d bytes (cl : %d) HTTP POST (num x) request is about to be sent on sd %d.",request_size,body_size,sd);
  
  nwrite = sendto (sd,request,request_size,0,(const struct sockaddr *) &options->srv_options->connect_infos->sd_table[sd].sockaddr,sizeof(options->srv_options->connect_infos->sd_table[sd].sockaddr));
  
  /* Fait le menage */
  if (*request)
    free (request);
  
  return (nwrite);
}
