/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: srv_check_client_request.c,v 1.6 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Verifie le magic number
*/

int check_and_add_magic_number(S_OPTIONS *options, int cl_num, char struct_options, char *magic_number)
{
  int cpt;
  int magic=0;

  /* Verifie qu'on a bien des digits */
  for (cpt=0;cpt<strlen(magic_number);cpt++)
    if (!isdigit(magic_number[cpt]))
      return (0);
  
  if ((magic = atoi(magic_number)) <= 0)
    return (0);

  cctt_debug(stderr,"[Ident] : Checking if '%d' magic number already exist.",magic);

  /* 
  ** Verifie que ce magic number n'est pas deja utilise 
  ** Ce genre de boucle est tres bourrin et gravement consommateur ! :(
  */
  for (cpt=0;cpt<options->srv_options->connect_infos->sdtablesize;cpt++)
    if (cpt != cl_num && (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED) &&
	options->srv_options->connect_infos->sd_table[cpt].magic_number == magic)
      {
	if (struct_options == CL_IDENT_ASKED_SHELL ||
	    struct_options == CL_IDENT_REV_SHELL ||
	    struct_options == CL_IDENT_LIST_PROX ||
	    struct_options == CL_IDENT_PROX_MOD ||
	    struct_options == CL_IDENT_REV_PROX_MOD)
	  return (0);
      }
  
  options->srv_options->connect_infos->sd_table[cl_num].magic_number = magic;
  return (magic);
}

void srv_mirror_clients_context(S_OPTIONS *options,int src,int dst)
{
  T_proxy_allowed_list *tmp_list;
  int cpt;

  options->srv_options->connect_infos->sd_table[dst].proxy_auth =
    options->srv_options->connect_infos->sd_table[src].proxy_auth;
  options->srv_options->connect_infos->sd_table[dst].tos =
    options->srv_options->connect_infos->sd_table[src].tos;
  options->srv_options->connect_infos->sd_table[dst].tvq.tv_usec =
    options->srv_options->connect_infos->sd_table[src].tvq.tv_usec;
  options->srv_options->connect_infos->sd_table[dst].tvu.tv_usec =
    options->srv_options->connect_infos->sd_table[src].tvu.tv_usec;
  options->srv_options->connect_infos->sd_table[dst].tvr.tv_usec =
    options->srv_options->connect_infos->sd_table[src].tvr.tv_usec;
  options->srv_options->connect_infos->sd_table[dst].basdec_alph_pos =
    options->srv_options->connect_infos->sd_table[src].basdec_alph_pos;
  options->srv_options->connect_infos->sd_table[dst].cl_req_sd =
    options->srv_options->connect_infos->sd_table[src].cl_req_sd;
  options->srv_options->connect_infos->sd_table[dst].pid_shell =
    options->srv_options->connect_infos->sd_table[src].pid_shell;
  options->srv_options->connect_infos->sd_table[dst].rev_prox_mode =
    options->srv_options->connect_infos->sd_table[src].rev_prox_mode;
  options->srv_options->connect_infos->sd_table[dst].rev_prox_mode_sd =
    options->srv_options->connect_infos->sd_table[src].rev_prox_mode_sd;

  options->srv_options->connect_infos->sd_table[dst].server_manager[0] =
    options->srv_options->connect_infos->sd_table[src].server_manager[0];
  options->srv_options->connect_infos->sd_table[dst].server_manager[1] =
    options->srv_options->connect_infos->sd_table[src].server_manager[1];

  /* Flush de l'ancien buffer */
  if (options->srv_options->connect_infos->sd_table[src].tmpbuf_size >= 0)
    {
      if (OPT_DEBUG2()) cctt_debug(stderr,"Mirroring : Flushing %d bytes from %d",
				   options->srv_options->connect_infos->sd_table[src].tmpbuf_size,
				   src);
      do_channel(options,src,
		 options->srv_options->connect_infos->sd_table[src].tmpbuf,
		 options->srv_options->connect_infos->sd_table[src].tmpbuf_size);
      if (OPT_DEBUG2()) cctt_debug(stderr,"Mirroring : Flushing %d : done",src);
    }

  /* Recuperation de la structure d'infos sockaddr pour les clients en mode proxy */

  if (options->srv_options->connect_infos->sd_table[dst].tos == CL_IDENT_PROX_MOD)
    {
      memcpy(&(options->srv_options->connect_infos->sd_table[dst].cl_prox_sockaddr),
	     &(options->srv_options->connect_infos->sd_table[src].cl_prox_sockaddr),
	     sizeof(options->srv_options->connect_infos->sd_table[src].cl_prox_sockaddr));
      memcpy(&(options->srv_options->connect_infos->sd_table[dst].prox_to_sockaddr),
	     &(options->srv_options->connect_infos->sd_table[src].prox_to_sockaddr),
	     sizeof(options->srv_options->connect_infos->sd_table[src].prox_to_sockaddr));
    }

  /*
  ** Il nous faut egalement aller modifier la liste des elements proxy
  ** pour la mettre a jour avec ce nouveau numero de client
  */

  tmp_list = options->srv_options->proxy_allowed_list->first;
  while (tmp_list->next != NULL)
    {
      if (tmp_list->reverse == src)
	tmp_list->reverse = dst;
      tmp_list = tmp_list->next;
    }
  if (tmp_list->reverse == src)
    tmp_list->reverse = dst;

  /*
  ** Et il nous faut aller modifier les clients qui pointaient vers l'ancien sd aussi...
  ** pfiuuuu...
  */

  for (cpt=0;cpt<options->srv_options->connect_infos->sdtablesize;cpt++)
    if (options->srv_options->connect_infos->sd_table[cpt].cl_req_sd == src)
      options->srv_options->connect_infos->sd_table[cpt].cl_req_sd = dst;
}


/*
** Cette fonction permet de verifier que le type de connexion demandee
** par le client est convenable et positionne un flag les informations
** du client.
** Elle verifie egalement le type de connexion demande et appelle la
** fonction concernee.
*/

int srv_check_client_request(S_OPTIONS *options, int cl_num, char struct_options, char *magic_number)
{
  int cpt,magic;

  /* Verifie et ajoute le magic number */
  if ((magic = check_and_add_magic_number(options, cl_num, struct_options, magic_number)) == 0)
    {
      cctt_debug(stderr,"[Ident] : Won't give existing magic number !");
      return (0);
    }

  if (struct_options == CL_IDENT_ASKED_SHELL)
    {
      if (OPT_SRV_PROXY_ONLY())
	{ 
	  cctt_verbose(stderr,"%s","Server is working in proxy_only mode (client wanted shell mode) ...\n");
	  return (0);
	}
      else
	{
	  server_was_asked_a_shell(options,cl_num);
	  return (1);
	}
    }

  if (struct_options == CL_IDENT_REV_SHELL)
    {
      if (OPT_SRV_PROXY_ONLY())
	{ 
	  cctt_verbose(stderr,"%s","Server is working in proxy_only mode (client wanted reverse-shell mode) ...\n");
	  return (0);
	}
      else
	{
	  options->srv_options->connect_infos->sd_table[cl_num].tos = CL_IDENT_REV_SHELL;
	  log_reverse_session(cl_num,inet_ntoa(options->srv_options->connect_infos->sd_table[cl_num].sockaddr.sin_addr),1);
	  return (1);
	}
    }
  
  if (struct_options == CL_IDENT_LIST_PROX)
    {
      /* 
      ** On n'a pas besoin de sauvegarder quoi que ce soit... 
      ** La fonction appelee balance la reponse et reset la connexion.
      ** On retourne 1 quand meme (ce n'est pas une erreur )
      **    et le client sera vire du pool au prochain select.
      */
      send_proxy_list_and_close(options, cl_num);
      return (2);
    }
  if (struct_options == CL_IDENT_PROX_MOD)
    {
      options->srv_options->connect_infos->sd_table[cl_num].tos = CL_IDENT_PROX_MOD;
      options->srv_options->connect_infos->sd_table[cl_num].rev_prox_mode_sd = cl_num;
      return (1);
    }
  if (struct_options == CL_IDENT_REV_PROX_MOD)
    {
      options->srv_options->connect_infos->sd_table[cl_num].tos = CL_IDENT_REV_PROX_MOD;
      options->srv_options->connect_infos->sd_table[cl_num].rev_prox_mode_sd = cl_num;
      return (1);
    }

  /*
  ** La meme chose mais le client etait deja identifie et on doit le rebasculer sur les
  **   applications existantes ...
  */

  if (struct_options == CL_IDENT_ASKED_SHELL_C)
    {
      if (OPT_SRV_PROXY_ONLY())
	{ 
	  cctt_verbose(stderr,"Server is working in proxy_only mode (client %d - %d wanted shell mode in continuation !?!?!) ...\n",cl_num,magic);
	  return (0);
	}
      else
	{
	  cctt_verbose(stderr,"Client %d - %d wants to continue a shell existing session ... Not Implemented !!!\n",cl_num,magic);
	  return(0);
	}
      
    }
  
  if (struct_options == CL_IDENT_REV_SHELL_C)
    {
      if (OPT_SRV_PROXY_ONLY())
	{ 
	  cctt_verbose(stderr,"Server is working in proxy_only mode (client %d - %d wanted reverse-shell mode in continuation !?!?!) ...\n",cl_num,magic);
	  return (0);
	}
      else
	{
	  options->srv_options->connect_infos->sd_table[cl_num].tos = CL_IDENT_REV_SHELL;
	  cctt_verbose(stderr,"Client %d - %d wants to continue a rev_shell existing session ... searching ...\n",cl_num,magic);

	  for (cpt=0;cpt<options->srv_options->connect_infos->sdtablesize;cpt++)
	    if (cpt != cl_num && (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED)
		&& magic == options->srv_options->connect_infos->sd_table[cpt].magic_number)
	      {
		cctt_verbose(stderr,"Mirroring : Client '%d' took client '%d' context (%d)...\n",cl_num,cpt,magic);
		srv_mirror_clients_context(options,cpt,cl_num);
		reset_client_connection(options,cpt);
		return(1);
	      }
	}
    }

  if (struct_options == CL_IDENT_PROX_MOD_C)
    {
      options->srv_options->connect_infos->sd_table[cl_num].tos = CL_IDENT_PROX_MOD;
      cctt_verbose(stderr,"Client '%d - %d' wants to continue a prox_mod existing session ... searching ...\n",cl_num,magic);

      for (cpt=0;cpt<options->srv_options->connect_infos->sdtablesize;cpt++)
	if (cpt != cl_num && (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED)
	    && magic == options->srv_options->connect_infos->sd_table[cpt].magic_number)
	  {
	    cctt_verbose(stderr,"Mirroring : Client '%d' took client '%d' context (%d)...\n",cl_num,cpt,magic);
	    srv_mirror_clients_context(options,cpt,cl_num);
	    reset_client_connection(options,cpt);
	    return(1);
	  }
    }

  if (struct_options == CL_IDENT_REV_PROX_MOD_C)
    {
      options->srv_options->connect_infos->sd_table[cl_num].tos = CL_IDENT_REV_PROX_MOD;
      cctt_verbose(stderr,"Client '%d - %d' wants to continue a rev_prox existing session ... searching ...\n",cl_num,magic);

      for (cpt=0;cpt<options->srv_options->connect_infos->sdtablesize;cpt++)
	if (cpt != cl_num && (options->srv_options->connect_infos->sd_table[cpt].used & SRV_SD_USED)
	    && magic == options->srv_options->connect_infos->sd_table[cpt].magic_number)
	  {
	    cctt_verbose(stderr,"Mirroring : Client '%d' took client '%d' context (%d)...\n",cl_num,cpt,magic);
	    srv_mirror_clients_context(options,cpt,cl_num);
	    reset_client_connection(options,cpt);
	    return(1);
	  }
    }

  return (0);
}
