/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: srv_ident_client.c,v 1.6 2003/06/10 15:21:17 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

/*
** Ces fonctions permettent d'appeler l'identification du client par le serveur.
** On a un tableau de pointeur sur fonctions dans src/conf_file/conf_file_IDENT.c
**   qui contient un mot cle present dans le fichier de configuration auquel
**   est associe un pointeur sur fonction.
** La definition de structure est presente dans includes/srv_ident_client.h
** La fonction definie ici doit etre (bien entendu) complementaire de la fonction
**  presente au sein du client qui est chargee de verifier que l'identification
**  est convenable (voir src/channels/cl_identification.c).
** Cette fonction doit placer le buffer dans structure (voir les commentaires pour la fonction
**  cl_identification_clear) et analyser les champs ensuite.
**
** 3 valeurs de retour (d'identification) sont possibles (voir includes/server.h) :
** SRV_CL_IDENT_YES : Le client s'est identifie
** SRV_CL_IDENT_ON : Le client est en cours d'identification
** SRV_CL_IDENT_NO : Le client a plante l'identification
*/

/*
** Fonction d'identification avec codage succinct (voir cl_identification_basic() pour la partie cliente)
** Identique sur le principe a la fonction srv_ident_client_clear
** voir basic_encoding.c
*/

int srv_ident_client_basic(S_OPTIONS *options, char *buffer, int sd, int nwr)
{
  int ret;
  T_cl_ident_clear struct_received;
  T_basic_decode basdec;
  char *magic_number;

  int reinject_size;

  /*
  ** On verifie que la clee decodee a une taille convenable
  **   Attention le type de service a deja ete positionne !!!
  */

  if (basdec.size < strlen(options->ident_key))
    {
      cctt_verbose(stderr,"%s","A identification buffer was too short to check IDENT_STRING.\n");
      return (SRV_CL_IDENT_NO);
    }
 
  /* On prepare la structure */
  memset(&struct_received,0,sizeof(T_cl_ident_clear));
  struct_received.options = *buffer;
  struct_received.key = (char *) malloc (nwr+1);
  memset(struct_received.key,0,nwr+1);
  if (*(buffer+1))
    strncpy(struct_received.key,buffer+1,nwr-1);

  /* On decode la cle avec l'alphabet defini dans basic_encoding.h */
  basdec = basic_decode(struct_received.key,
			be_get_key_length(options->ident_key),
			CCTT_BASIC_ENCODING_ALPHABET,
			options,
			sd);

  /* On verifie la cle du client */
  if (basdec.decoded 
      && (strlen(options->ident_key) <= basdec.size &&
	  (strncmp(basdec.decoded,options->ident_key,strlen(options->ident_key)) == 0)))
    {
      /* Identification correcte, on va cherche le magic number */
      if (strlen(struct_received.key) == strlen(options->ident_key))
	{
	  cctt_verbose(stderr,"[Ident] : Client %d didn't send any magic number !.",sd);
	  return (SRV_CL_IDENT_NO);
	}
      magic_number = (char *) malloc(basdec.size-strlen(options->ident_key));
      memset(magic_number,0,basdec.size-strlen(options->ident_key));
      strncpy(magic_number,basdec.decoded+strlen(options->ident_key),basdec.size-strlen(options->ident_key));

      cctt_debug(stderr,"[Ident] : Client sended '%s' magic number.",magic_number);

      /*
      ** Attention ! Des donnees peuvent deja figurer dans le buffer juste apres le magic number...
      ** C'est pour ca qu'on a besoin d'une taille fixe...
      */
      if (nwr > strlen(options->ident_key)+1+7)
	{
	  reinject_size = nwr - (strlen(options->ident_key)+1+7);
	  options->srv_options->connect_infos->sd_table[sd].tmpbuf_ptr = (char *) malloc(reinject_size+1);
	  memset(options->srv_options->connect_infos->sd_table[sd].tmpbuf_ptr,0,reinject_size+1);
	  memcpy(options->srv_options->connect_infos->sd_table[sd].tmpbuf_ptr,
		 struct_received.key+strlen(options->ident_key)+7,
		 reinject_size);
	  options->srv_options->connect_infos->sd_table[sd].tmpbuf_size = reinject_size;
	  if (OPT_DEBUG2()) cctt_debug(stderr,"Http : Reinjection overhead (%d bytes) dans le buffer client %d.",reinject_size,sd);
	}

      /* On verifie la requete demandee par le client */
      if ((ret = srv_check_client_request(options, sd, struct_received.options,magic_number)) > 0)
	if (ret == 1 || ret == 2) /* Avec 2 la socket est deja fermee ...*/
	  return (SRV_CL_IDENT_YES); /* Verification de la requete OK */
    }
  
  /* Ident. plantee, on renvoie failed et la socket sera ferme cash */
  return (SRV_CL_IDENT_NO);
}

/*
** Fonction d'identification minimale : 
** La structure definie dans includes/cl_identification.h contient
** un octet de demande de service + la cle.
** Le serveur met un read du buffer dans la structure, puis l'analyse
** et accepte ou non la demande.
** Si la cle ne correspond pas, on ferme la socket cash.
** 
*/ 

int srv_ident_client_clear(S_OPTIONS *options, char *buffer, int sd, int nwr)
{
  int ret;
  T_cl_ident_clear struct_received;
  char *magic_number;

  int reinject_size;

  /* 
  ** On verifie tout d'abord que le buffer qui nous parvient ici est au moins
  ** egal en taille a celui de la cle configuree + demande de service
  */
  if (nwr < strlen(options->ident_key) + 1)
    {
      cctt_verbose(stderr,"%s","A identification buffer was too short to check IDENT_STRING.\n");
      return (SRV_CL_IDENT_NO);
    }

  /* On prepare la structure */
  memset(&struct_received,0,sizeof(T_cl_ident_clear));
  struct_received.options = *buffer;
  struct_received.key = (char *) malloc (nwr+1);
  memset(struct_received.key,0,nwr+1);
  if (*(buffer+1))
    memcpy(struct_received.key,buffer+1,nwr-1);

  /* On verifie la cle du client */
  if (struct_received.key 
      && (strlen(struct_received.key) >= strlen(options->ident_key) &&
	  (strncmp(struct_received.key,options->ident_key,strlen(options->ident_key))) == 0))
    {
      /* Identification correcte, on va cherche le magic number */
      if (strlen(struct_received.key) == strlen(options->ident_key))
	{
	  cctt_verbose(stderr,"[Ident] : Client %d didn't send any magic number !.",sd);
	  return (SRV_CL_IDENT_NO);
	}
      magic_number = (char *) malloc(7);
      memset(magic_number,0,8);
      strncpy(magic_number,struct_received.key+strlen(options->ident_key),7);

      cctt_debug(stderr,"[Ident] : Client sended '%s' magic number.",magic_number);

      /*
      ** Attention ! Des donnees peuvent deja figurer dans le buffer juste apres le magic number...
      ** C'est pour ca qu'on a besoin d'une taille fixe...
      */
      if (nwr > strlen(options->ident_key)+1+7)
	{
	  reinject_size = nwr - (strlen(options->ident_key)+1+7);
	  options->srv_options->connect_infos->sd_table[sd].tmpbuf_ptr = (char *) malloc(reinject_size+1);
	  memset(options->srv_options->connect_infos->sd_table[sd].tmpbuf_ptr,0,reinject_size+1);
	  memcpy(options->srv_options->connect_infos->sd_table[sd].tmpbuf_ptr,
		 struct_received.key+strlen(options->ident_key)+7,
		 reinject_size);
	  options->srv_options->connect_infos->sd_table[sd].tmpbuf_size = reinject_size;
	  if (OPT_DEBUG2()) cctt_debug(stderr,"Http : Reinjection overhead (%d bytes) dans le buffer client %d.",reinject_size,sd);
    }
  
      /* On verifie la requete demandee par le client */
      if ((ret = srv_check_client_request(options, sd, struct_received.options,magic_number)) > 0)
	if (ret == 1 || ret == 2) /* Avec 2 la socket est deja fermee ...*/
	  return (SRV_CL_IDENT_YES); /* Verification de la requete OK */
    }
  /* Ident. plantee, on renvoie failed et la socket sera ferme cash */
  return (SRV_CL_IDENT_NO);
}

/*
** Fonction de recherche du pointeur sur fonction a executer
*/

int srv_ident_client(S_OPTIONS *options,char *buffer, int sd, int nwr)
{
  int i,value=0;
  T_basic_decode basdec;
  char *decoded_buffer;
    
  cctt_verbose(stderr,"%s","Server is authenticating client.\n");

  /*
  ** On decode d'abord le buffer si jamais besoin est ...
  ** => on enleve le codage basic, les eventuels headers HTTP, etc...
  */

  basdec = do_server_has_something_to_decode_locally(options,sd,buffer,nwr);

  if (!(basdec.decoded) || basdec.size <= 0)
    return (SRV_CL_IDENT_NO);

  decoded_buffer = (char *) malloc(basdec.size+1);
  memset(decoded_buffer,0,basdec.size+1);
  memcpy(decoded_buffer,basdec.decoded,basdec.size);

  /*
  ** Attention, si le mode est en basic encoding, la requete d'identification n'est
  ** pas encore decodee !!!
  */

  /* On va chercher la fonction a executer */
  for (i=0;ident_type_array[i].ident_name != NULL;i++)
    if ((strcmp(options->identification,ident_type_array[i].ident_name) == 0))
      value = ident_type_array[i].ident_type_server(options,decoded_buffer,sd,basdec.size);
  return (value);
}
