/*
** CCTT - Covert Channel Tunneling Tool - v0.1.7
** Copyright (C) 2002,2003 Simon Castro - scastro@entreelibre.com
** $Id: verbose.c,v 1.8 2003/06/11 13:14:19 simsim Exp $
**
** This file is part of CCTT - v0.1.7 (C) 2002,2003 Simon Castro.
** 
** Cctt is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** Cctt is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with Cctt; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../includes/definitions.h"

void verbose_show_options(S_OPTIONS *options)
{
  T_proxy_allowed_list *tmp_list;
  int i;

  if (!OPT_CL_OR_SRV() && OPT_LOGGING_SYSLOG()) openlog("Cctt Server",LOG_NDELAY|LOG_PID,get_syslog_facility(DEFAULT_SYSLOG_FACILITY));

  fprintf(stderr,VERSION);
  if (!OPT_CL_OR_SRV() && OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),VERSION);
  
  if (OPT_CL_OR_SRV()) /* Fonctionnement du client ? */
    {
      fprintf(stderr,"\nAfter initialisation, Cctt will work in CLIENT Mode (verbose mode) :\n");
      fprintf(stderr,"  # The configuration file is set to %s.\n",options->conf_file); 
      fprintf(stderr,"  # Connexion to next hop (%s:%d) will use the ",
	      inet_ntoa(options->cl_options->sockaddr.sin_addr),
	      ntohs(options->cl_options->sockaddr.sin_port));

      fprintf(stderr,(OPT_CL_TCP()?"tcp protocol.\n":""));
      fprintf(stderr,(OPT_CL_UDP()?"udp protocol.\n":""));

      for (i=0;options->cl_options->http_proxy_chain_addrs[i] != NULL;i++)
	fprintf (stderr,"    $ HttpProxyChain is used. Proxy %d is %s:%s (connection timeout is %d msec)\n",
		 i,options->cl_options->http_proxy_chain_addrs[i],
		 options->cl_options->http_proxy_chain_ports[i],
		 options->cl_options->http_proxy_chain_delays[i]);

      if (OPT_CL_CHAN_PROXY())
	{
	  fprintf(stderr,"  # Timeout for first proxy is %d msecs\n",options->cl_options->proxy_delay);
	  fprintf(stderr,"  # The server behind the (last) proxy will be %s:%d ",
		  options->cl_options->dest_server_address,
		  options->cl_options->dest_server_port);
	  fprintf(stderr,(OPT_CL_DEST_TCP()?"(tcp protocol).\n":""));
	  fprintf(stderr,(OPT_CL_DEST_UDP()?"(udp protocol).\n":""));
	}
      if (OPT_CL_PROX_MOD())
	{
	  fprintf(stderr,"  # client has to work in proxy_mod : \n");
	  fprintf(stderr,"    # it will set a listenning ");
	  fprintf(stderr,(OPT_CL_PMODE_TCP()?"tcp socket ":""));
	  fprintf(stderr,(OPT_CL_PMODE_UDP()?"udp socket ":""));
	  fprintf(stderr,"on %s:%d\n",
		  inet_ntoa(options->cl_options->local_sockaddr.sin_addr),
		  ntohs(options->cl_options->local_sockaddr.sin_port));
	  fprintf(stderr,"    # and will ask the server to forward connection to %s:%d.\n",
		  options->cl_options->dest_server_proxy_mode_address,
		  options->cl_options->dest_server_proxy_mode_port);
	}
      if (OPT_CL_REV_PROX_MOD())
	{
	  fprintf(stderr,"  # client has to work in reverse_proxy_mod : \n");
	  fprintf(stderr,"    # it will tell the server it forwards incoming connections to %s:%d.\n",
		  options->cl_options->dest_server_proxy_mode_address,
		  options->cl_options->dest_server_proxy_mode_port);
	}
      fprintf(stderr,(OPT_RSHELL()?"  # Client will bind a reverse shell.\n":""));
      fprintf(stderr,(OPT_CL_LIST_MOD()?"  # Client will ask the server proxy_mod list.\n":""));
      fprintf(stderr,(OPT_CL_NORMAL_MOD()?"  # Client will ask a shell to the server.\n":""));
      fprintf(stderr,"  # The identification mode is set to %s (key: %s).\n",
	      options->identification,
	      options->ident_key);

      if (tell_me_about_encoding(options) == 2)
	{
	  fprintf(stderr,"  # Http mode is used :\n");
	  fprintf(stderr,"    # Valid data matching URI : '%s'.\n",options->cl_options->http_mode_uri);
	  fprintf(stderr,"    # Per connection request number : %d.\n",
		  options->cl_options->http_mode_req_per_con);
	  fprintf(stderr,"    # Delay (msecs) between each connection : %d.\n",
		  options->cl_options->http_mode_del_bet_con);
	  fprintf(stderr,"    # Maximal data size to add in HTTP data field : %d bytes.\n",
		  options->cl_options->http_mode_datasize);
	  fprintf(stderr,"    # Hostname used : '%s'.\n",
		  options->cl_options->http_mode_hostname);
	  fprintf(stderr,"    # Content-type used : '%s'.\n",
		  options->cl_options->http_mode_contenttype);
	  if (options->cl_options->http_mode_fake_urls != NULL)
	    {
	      i = 0;
	      while (options->cl_options->http_mode_fake_urls[i] != NULL)
		{
		  fprintf(stderr,"    # Fake request %d is '%s'.\n",i,
			  options->cl_options->http_mode_fake_urls[i]);
		  i++;
		}
	    }
	  if (options->cl_options->http_mode_fake_urls != NULL)
	    fprintf (stderr,"    # Fake urls frequency is set to %d.\n",
		     options->cl_options->http_mode_fake_urls_freq);

	  if (options->srv_options->http_mode_top_padding_size != 0)
	    fprintf(stderr,"    # Top padding is '%d' bytes long.\n",
		    options->srv_options->http_mode_top_padding_size);
	  if (options->srv_options->http_mode_bot_padding_size != 0)
	    fprintf(stderr,"    # Bottom padding is '%d' bytes long.\n",
		    options->srv_options->http_mode_bot_padding_size);
	  
	  if (options->cl_options->http_mode_top_padding)
	    fprintf(stderr,"    # Top padding file is %d bytes long.\n",
		    strlen(options->cl_options->http_mode_top_padding));
	  if (options->cl_options->http_mode_bottom_padding)
	    fprintf(stderr,"    # Bottom padding file is %d bytes long.\n",
		    strlen(options->cl_options->http_mode_bottom_padding));

	}

      fprintf(stderr,"  # And last, the channel type is set to %s.\n",options->channel);

      if (tell_me_about_encoding(options) == 2 && OPT_CL_NORMAL_MOD())
	fprintf (stderr,"%s","  # Warning : This functionnality isn't done for this channel type !\n");

      fprintf(stderr,"\n");
    }
  else  /* Fonctionnement du serveur ? */
    {
      fprintf(stderr,"\nAfter initialisation, Cctt will work in SERVER Mode (verbose) :\n");
      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),"\nVerbose Mode - After initialisation, Cctt will work in SERVER Mode :\n");

      fprintf(stderr,"  # The configuration file is set to %s.\n",options->conf_file); 
      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),"  # The configuration file is set to %s.\n",options->conf_file);

      if (OPT_SRV_TCP())
	{
	  fprintf(stderr,"  # The server will listen on TCP %s:%d\n",
	      inet_ntoa(options->srv_options->sockaddr.sin_addr),
	      ntohs(options->srv_options->sockaddr.sin_port));
      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # The server will listen on TCP %s:%d\n",
				       inet_ntoa(options->srv_options->sockaddr.sin_addr),
				       ntohs(options->srv_options->sockaddr.sin_port));
	}
      else
	{
	  fprintf(stderr,"  # The server will listen on UDP %s:%d\n",
		  inet_ntoa(options->srv_options->sockaddr.sin_addr),
		  ntohs(options->srv_options->sockaddr.sin_port));
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # The server will listen on UDP %s:%d\n",
					   inet_ntoa(options->srv_options->sockaddr.sin_addr),
					   ntohs(options->srv_options->sockaddr.sin_port));
	}

      if (OPT_LOGGING())
	{
	  fprintf(stderr,"  # Logging to %s is set.\n",DEFAULT_LOG_FILE);
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),"  # Logging to %s is set.\n",DEFAULT_LOG_FILE);
	}
      else
	{
	  fprintf(stderr,"  # Logging is not set.\n");
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),"  # Logging is not set.\n");
	}

      if (OPT_LOGGING_SYSLOG())
	{
	  fprintf(stderr,"  # Logging to syslog (%s:%s) is set.\n",DEFAULT_SYSLOG_FACILITY,DEFAULT_SYSLOG_PRIORITY);
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),"  # Logging to syslog (%s:%s) is set.\n",DEFAULT_SYSLOG_FACILITY,DEFAULT_SYSLOG_PRIORITY);
	}
      else
	fprintf(stderr,"  # Logging to syslog is not set.\n");

      /* Si la liste d'autorisation comporte des informations */
      if (options->srv_options->proxy_allowed_list_elements_number != 0)
	{

	  fprintf(stderr,"  # NB : Allowed proxying list contains %d elements :\n",
		  options->srv_options->proxy_allowed_list_elements_number);
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # NB : Allowed proxying list contains %d elements :\n",
					   options->srv_options->proxy_allowed_list_elements_number);

	  /* On se positionne sur le premier */
	  tmp_list = options->srv_options->proxy_allowed_list->first;

	  /* Tant que le suivant n'est pas nul */
	  while (tmp_list && tmp_list->next != NULL)
	    {
	      if (tmp_list->reverse == 0)
		{
		  fprintf(stderr,"    => %s:%s:%d.\n",
			  tmp_list->label,
			  inet_ntoa(tmp_list->sockaddr.sin_addr),
			  ntohs(tmp_list->sockaddr.sin_port));
		  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
						   "    => %s:%s:%d.\n",
						   tmp_list->label,
						   inet_ntoa(tmp_list->sockaddr.sin_addr),
						   ntohs(tmp_list->sockaddr.sin_port));
		}
	      else
		{
		  fprintf(stderr,"    => %s:%s:%d - reversed on sd %d.\n",
		      tmp_list->label,
		      inet_ntoa(tmp_list->sockaddr.sin_addr),
		      ntohs(tmp_list->sockaddr.sin_port),
		      tmp_list->reverse);
	      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					       "    => %s:%s:%d - reversed on sd %d.\n",
					       tmp_list->label,
					       inet_ntoa(tmp_list->sockaddr.sin_addr),
					       ntohs(tmp_list->sockaddr.sin_port),
					       tmp_list->reverse);
		}
	      tmp_list = tmp_list->next;
	    }

	  /* On n'oublie pas le dernier */
	  if (tmp_list != NULL)
	    {
	      if (tmp_list->reverse == 0)
		{
		  fprintf(stderr,"    => %s:%s:%d.\n",
			  tmp_list->label,
			  inet_ntoa(tmp_list->sockaddr.sin_addr),
			  ntohs(tmp_list->sockaddr.sin_port));
		  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
						   "    => %s:%s:%d.\n",
						   tmp_list->label,
						   inet_ntoa(tmp_list->sockaddr.sin_addr),
						   ntohs(tmp_list->sockaddr.sin_port));
		}
	      else
		{
		  fprintf(stderr,"    => %s:%s:%d - reversed on sd %d.\n",
		  tmp_list->label,
		  inet_ntoa(tmp_list->sockaddr.sin_addr),
		    ntohs(tmp_list->sockaddr.sin_port),
		    tmp_list->reverse);
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
						   "    => %s:%s:%d - reversed on sd %d.\n",
					   tmp_list->label,
					   inet_ntoa(tmp_list->sockaddr.sin_addr),
					   ntohs(tmp_list->sockaddr.sin_port),
					   tmp_list->reverse);
		}
	    }
	}

      fprintf(stderr,"  # The identification mode is set to %s (key: %s).\n",
	      options->identification,
	      options->ident_key);
      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
				       "  # The identification mode is set to %s (key: %s).\n",
				       options->identification,
				       options->ident_key);

      fprintf(stderr,"  # The shell location is set to %s (name : %s).\n",
	      options->srv_options->srv_shell_loc,
	      options->srv_options->srv_shell_cmd);
      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
				       "  # The shell location is set to %s (name : %s).\n",
				       options->srv_options->srv_shell_loc,
				       options->srv_options->srv_shell_cmd);

      if (OPT_SRV_PROXY_ONLY())
	{
	  fprintf(stderr,"  # Server will work in proxy only mode.\n");
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # Server will work in proxy only mode.\n");
	}

      if (options->srv_options->perm_uid != 0 && options->srv_options->perm_gid != 0)
	{
	  fprintf (stderr,"  # Permission restriction activated : uid %d, gid %d.\n",
		   options->srv_options->perm_uid,
		   options->srv_options->perm_gid);
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # Permission restriction activated : uid %d, gid %d.\n",
					   options->srv_options->perm_uid,
					   options->srv_options->perm_gid);
	}
      if (options->srv_options->perm_chroot != NULL)
	{
	  fprintf (stderr,"  # Permission restriction activated : chroot %s.\n",
		   options->srv_options->perm_chroot);
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # Permission restriction activated : chroot %s.\n",
					   options->srv_options->perm_chroot);
	}

      if (options->srv_options->fake_webserver != NULL)
	{
	  fprintf(stderr,"  # Reply as a fake webserver is activated.\n");
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # Reply as a fake webserver is activated.\n");
	}	  

      if (tell_me_about_encoding(options) == 2)
	{
	  fprintf(stderr,"  # Http mode is used :\n");
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "  # Http mode is used :\n");

	  fprintf(stderr,"    # Valid data matching URI : '%s'.\n",
		  options->srv_options->http_mode_uri);
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "    # Valid data matching URI : %s\n",
					   options->srv_options->http_mode_uri);

	  fprintf(stderr,"    # Error page is %d bytes long.\n",
		  strlen(options->srv_options->http_mode_error_page));
	  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					   "    # Error page is %d bytes long.\n",
					   strlen(options->srv_options->http_mode_error_page));

	  if (options->srv_options->http_mode_top_padding_size != 0)
	    {
	      fprintf(stderr,"    # Top padding is '%d' bytes long.\n",
		      options->srv_options->http_mode_top_padding_size);
	      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					       "    # Top padding is '%d' bytes long.\n",
					       options->srv_options->http_mode_top_padding_size);
	    }
	  if (options->srv_options->http_mode_bot_padding_size != 0)
	    {
	      fprintf(stderr,"    # Bottom padding is '%d' bytes long.\n",
		      options->srv_options->http_mode_bot_padding_size);
	      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					       "    # Bottom padding is '%d' bytes long.\n",
					       options->srv_options->http_mode_bot_padding_size);
	    }

	  if (options->cl_options->http_mode_top_padding)
	    {
	      fprintf(stderr,"    # Top padding file is %d bytes long.\n",
		      strlen(options->cl_options->http_mode_top_padding));
	      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					       "    # Top padding file is %d bytes long.\n",
					       strlen(options->cl_options->http_mode_top_padding));
	    }
	  if (options->cl_options->http_mode_bottom_padding)
	    {
	      fprintf(stderr,"    # Bottom padding file is %d bytes long.\n",
		    strlen(options->cl_options->http_mode_bottom_padding));
	      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
					       "    # Bottom padding file is %d bytes long.\n",
					       strlen(options->cl_options->http_mode_bottom_padding));
	    }

	  if (options->srv_options->http_mode_fake_url_filename != NULL)
	    {
	      i = 0;
	      while (options->srv_options->http_mode_fake_url_filename[i] != NULL)
		{
		  fprintf(stderr,"    # Fake url filename %d is '%s'.\n",i,
			  options->srv_options->http_mode_fake_url_filename[i]);
		  if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
						   "    # Fake url filename %d is '%s'.\n",i,
						   options->srv_options->http_mode_fake_url_filename[i]);
		  i++;
		}
	    }
	}
      
      fprintf(stderr,"  # Automatic killing of connexions is set to :\n    # %d msecs for all connections\n    # %d msecs for closed_flag connections\n",options->srv_options->kill_quiet_del,options->srv_options->kill_quiet_del_cf);
      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),"  # Automatic killing of connexions is set to :\n    %d for all connections\n    %d for closed_flag connections\n",options->srv_options->kill_quiet_del,options->srv_options->kill_quiet_del_cf);

      fprintf(stderr,"  # And last, the channel type is set to %s.\n\n",options->channel);
      if (OPT_LOGGING_SYSLOG()) syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),
				       "  # And last, the channel type is set to %s.\n\n",options->channel);
      
      if (OPT_LOGGING_SYSLOG()) closelog();
    }
}

/*
** Cette fonction est utilisee par toutes les fonctions d'affichage (verbeux, debug et erreur).
** Elle prend une chaine de format et une va_list et renvoie le char * correspondant. 
**
*/

char *cctt_display_loop(char *fmt,va_list params)
{
  char *s;
  int d;
  
  char *res;
  int cpt_fmt,cpt_res;

  /* On alloue tout d'abord suffisamment de memoire dans le buffer temporaire par rapport au buffer de format fmt */
  res = (char *) malloc(strlen(fmt)+1);
  memset(res,0,strlen(fmt)+1);

  /* On parse la chaine de format */
  for (cpt_fmt=0,cpt_res=0;fmt[cpt_fmt];cpt_fmt++)
    {
      if (fmt[cpt_fmt] == '%' || (cpt_fmt != 0 && fmt[cpt_fmt-1] == '%'))
        {
	  /* On est sur le caractere '%' */
          if (fmt[cpt_fmt] != '%')
            {
	      /* On est sur le caractere de format */
              if (fmt[cpt_fmt] == 's')
                {
                  s = va_arg(params, char *); /* Recupere un pointeur sur l'argument */
		  if (!s)
		    return (NULL);
		  /* 
		  ** Ce qui suit est pourri ... Je ne sais pas exactement comment gerer la taille exacte
		  ** alors j'alloue plus que necessaire :(
		  */
                  res = realloc(res,strlen(fmt)+strlen(s)+strlen(res));
                  strncpy(res+cpt_res,s,strlen(s)); /* Add the new arg */
                  cpt_res += (strlen(s));
                }
              if (fmt[cpt_fmt] == 'd')
                {
                  d = va_arg(params, int);
                  s = get_str_from_int(d);
		  if (!s)
		    return (NULL);
		  /* 
		  ** Ce qui suit est pourri ... Je ne sais pas exactement comment gerer la taille exacte
		  ** alors j'alloue plus que necessaire :(
		  */
                  res = realloc(res,strlen(fmt)+strlen(s)+strlen(res));
                  strncpy(res+cpt_res,s,strlen(s)); /* Add the new arg */
                  cpt_res += (strlen(s));
                }
            }
        }
      else
        {
	  /* Ajout le caractere dans le buffer resultat */
          res[cpt_res] = fmt[cpt_fmt];
          cpt_res++;
        }
    }

  res[cpt_res] = '\0';

  return (res);
}

int cctt_verbose(FILE *stream, char *fmt, ...)
{
  va_list params;
  char *res;
  int cpt_res;
  
  if (! OPT_VERBOSE()) 
    return (0);
  if (!fmt || !stream)
    return (0);

  va_start(params,fmt);
  if ((res = cctt_display_loop(fmt,params)) == NULL)
    return (0);
  va_end(params);

  cpt_res = fprintf(stream,"%s","Verbose Mode : ");
  cpt_res += fprintf(stream,"%s",res);

  /* Si l'option syslog est egalement activee ... */
  if (!OPT_CL_OR_SRV() && OPT_LOGGING_SYSLOG())
    {
      openlog("Cctt Server",LOG_NDELAY|LOG_PID,get_syslog_facility(DEFAULT_SYSLOG_FACILITY));
      syslog(get_syslog_priority(DEFAULT_SYSLOG_PRIORITY),"%s",res);
      closelog();
    }

  if (res)
    free (res);
  return (cpt_res);
}
